#!/usr/bin/perl

###############################################################################
#                                                                             #
# Filename: domain_settings.cgi                                               #
#                                                                             #
# Description: CGI needed to display and modify domain configurations.        #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use VirtualDomain;
use VirtualServ;
use VirtualFtp;
use DisplayMain;
use MonSettings;
use CookieData;
use DomainMail;
use NetConfig;
use Authorize;
use Quotaall;
use Mail;
use Safe;
use SSL;
use CGI;

###############################################################################
# ValidateFormData
###############################################################################

sub ValidateFormData
{
	my ($FormDataRef)	= @_;
	my $Return			= 0;
	my $Key;

	# Run through each of the keys...
	foreach $Key (keys(%{ $FormDataRef }))
	{
		# If the data is defined...
		if (defined($FormDataRef->{$Key}))
		{
			# Trim the whitespace.
			$FormDataRef->{$Key} =~ s/^\s*(.*?)\s*$/$1/;
		}
	}
	
	# Validate the data...
	if (($FormDataRef->{'name'} !~ /^(\w|-|\.|_)+$/) ||
		($FormDataRef->{'name'} =~ /^domain\d+$/))
	{
		# Set the return code.
		$Return = 750000;
	}
	elsif ($FormDataRef->{'alias'} !~ /^(\w|-|\.|_)*$/)
	{
		# Set the return code.
		$Return = 750001;
	}
	elsif (($FormDataRef->{'ip_address'} !~ /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/) ||
		   (1 > $1) || (254 < $1) || (0 > $2) || (254 < $2) ||
		   (0 > $3) || (254 < $3) || (0 > $4) || (254 < $4))
	{
		# Set the return code.
		$Return = 750002;
	}
	elsif (($FormDataRef->{'network_mask'} !~ /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/))
	{
		# Set the return code.
		$Return = 750003;
	}
	elsif ($FormDataRef->{'admin_name_login'} !~ /^(\w|-|_)*$/)
	{
		# Set the return code.
		$Return = 750005;
	}
	elsif (($FormDataRef->{'disk_quota'} !~ /^\+*([0-9]*)\.?([0-9]*)$/) || 
	       (!($1 || ('0' eq $1) || $2 || ('0' eq $2))))
	{
		# Set the return code.
		$Return = 750008;
	}
	elsif ($FormDataRef->{'fpx_active'})
    {
		# Check the passwords...
		if ($FormDataRef->{'password_frontpage'} !~ /^\S+$/)
		{
			# Set the return code.
			$Return = 750009;
		}
		elsif ($FormDataRef->{'password_frontpage'} ne $FormDataRef->{'password_confirm_frontpage'})
		{
			# Set the return code.
			$Return = 750010;
		}
	}
	elsif ($FormDataRef->{'ssl_active'})
	{
		my @DomainList	= VirtualDomain::list_vws();
		my $Item;

		# Run through the domain list...
		foreach $Item (@DomainList) 
		{
			# If the addresses match but the domain does not...
			if (($FormDataRef->{'ip_address'} eq $Item->{'vdip'}) &&
				($FormDataRef->{'old_name'}   ne $Item->{'vdn'}))
			{
				# Check for an existing virtual IP squatter...
				$Return = 750025 if(('yes' eq $Item->{'vdssl'}) && ( -e "$Item->{'dom_hom'}/$Item->{'dom_grp'}/cert/certificate" ));
			}

			# Bail out if needed...
			last if($Return);
		}
	}

	return $Return;
}

###############################################################################
# ConfigureSSL
###############################################################################

sub ConfigureSSL
{
	my ($DomainRef, $FormDataRef)	= @_;
	my $SSLData						= new SSL("/home/domain/$DomainRef->{'dom_grp'}/cert", '', 'key', 'request', 'certificate');
	my $Return						= 0;

	# Clear out the SSL data.
	$SSLData->SetPrivateKey();
	$SSLData->SetCertificateSigningRequest();
	$SSLData->SetCertificate();
	
	# If the configuration is self-signed...
	if ('self' eq $FormDataRef->{'certificate_type'})
	{
		# Generate the private key, CSR, and certificate...
		unless ((0 == $SSLData->SetSubject($FormDataRef)) &&
				(defined($SSLData->GetCertificate(1))))
		{
			# Set the return code.
			$Return = 750017;
		}
	}
	else
	{
		# Clean off goo.
		$FormDataRef->{'private_key'}	=~ s/\r//g;
		$FormDataRef->{'certificate'}	=~ s/\r//g;

		# Generate the private key, CSR, and certificate...
		unless ((0 == $SSLData->SetPrivateKey($FormDataRef->{'private_key'})) &&
				(0 == $SSLData->SetCertificate($FormDataRef->{'certificate'})))
		{
			# Set the return code.
			$Return = 750018;
		}
		else
		{
			# Clear out the CSR.
			$SSLData->SetCertificateSigningRequest();
		}
	}

	return $Return;
}

###############################################################################
# ModifyDomain
###############################################################################

sub ModifyDomain
{
	my ($FormDataRef)	= @_;
	my $Return			= 0;

	# Convert to lower case.
	$FormDataRef->{'name'}	= "\L$FormDataRef->{'name'}";
	$FormDataRef->{'alias'} = "\L$FormDataRef->{'alias'}";

	# Try to open/create a lock file...
	if (open(LOCKFILE, ">> /var/lock/server.lock"))
	{
		# Try to lock the code chunk...
		if (flock(LOCKFILE, 2))
		{
			my @DomainList	= VirtualDomain::list_vws();
			my $DomainRef;
			my $Domain;
			my $Index		= '';
			my $Item;

			# Run through the domain list...
			foreach $Item (@DomainList) 
			{
				# If the domain name matches...
				if ($FormDataRef->{'old_name'} eq $Item->{'vdn'})
				{
					# Get the index.
					$Index = $Item->{'idx'};
					last;
				}
			}

			# Get the domain information.
			$DomainRef = $DomainList[$Index];
			
			# Make sure there is a site...
			if (('' ne $Index) &&
				( -d $DomainRef->{'docroot'} ))
			{
				my $ReloadRoot = 0;

				# Check for a new name...
				if ($FormDataRef->{'name'} ne $FormDataRef->{'old_name'}) 
				{
					$ReloadRoot = 1;

					# Run through the domain list...
					foreach $Item (@DomainList) 
					{
						# If the domain name matches...
						if ($FormDataRef->{'name'} eq $Item->{'vdn'})
						{
							# Set the return code.
							$Return = 750013;
							last;
						}
					}
                }
				
				# If there already is a domain and we are trying to take up more space...
				$Return = 750014 if((0 == $FormDataRef->{'disk_quota'}) && (1 != scalar(@DomainList)));
				
				# If things are still going good...
				unless ($Return)
				{
					# Check for a different quota...
					if ((0 != $FormDataRef->{'old_disk_quota'}) &&
						($FormDataRef->{'disk_quota'} ne $FormDataRef->{'old_disk_quota'}))
					{
						# Quota for domain configured and check for disk space.
						my ($SpaceFree)	= Quotaall::comDomainQuota();
						my $SpaceUnused	= ($SpaceFree + $FormDataRef->{'old_disk_quota'});
						
						# Check for quota limits...
						if ($FormDataRef->{'disk_quota'} >= $SpaceUnused) 
						{
							# Set the return code.
							$Return = 750014;
						}
					}

					# If things are still going good...
					unless ($Return)
					{
						my ($AddressIP, $Name, $Group)	= VirtualDomain::ip_domain($FormDataRef->{'ip_address'}, $DomainRef->{'vdn'});
						my $SSLState 					= ($FormDataRef->{'ssl_active'}) ? 'Yes' : '';
                        
						# Make sure the IP address is not in use by another domain...
						if ('' ne $AddressIP)
						{
							# Run through the domain list...
							foreach $Item (@DomainList) 
							{
								# If the addresses match but the domain does not...
								if (($FormDataRef->{'ip_address'} eq $Item->{'vdip'}) &&
									($FormDataRef->{'old_name'}   ne $Item->{'vdn'}))
								{
									# Check for an existing virtual IP squatter...
									$Return = 750024 if(VirtualFtp::getFtpStatus($Item->{'vdip'}, $Item->{'dom_grp'}));
									
									# Check for an existing virtual IP squatter...
									$Return = 750025 if(($FormDataRef->{'ssl_active'}) && ('yes' eq $Item->{'vdssl'}) && ( -e "$Item->{'dom_hom'}/$Item->{'dom_grp'}/cert/certificate" ));
								}

								# Bail out if needed...
								last if($Return);
							}
							
							# If things are still going good...
							unless ($Return)
							{
								# Make sure that Virtual FTP is not used on the shared IP...
								$Return = 750011 if($FormDataRef->{'ftp_active'});
							}
						}

						# If things are still going good...
						unless ($Return)
						{
							my @NetworkInterfaces	= NetConfig->listNetInterfaces();
							my $NetworkInterface;
							my $VirtualIP;
							my $Netmask;
                            
							# If SSL is disabled...
							unless ($FormDataRef->{'ssl_active'})
							{
								my $SSLData = new SSL("/home/domain/$DomainRef->{'dom_grp'}/cert", '', 'key', 'request', 'certificate');
							
								# Clean out the files...
								$SSLData->SetPrivateKey();
								$SSLData->SetCertificateSigningRequest();
								$SSLData->SetCertificate();

								# Clean out the directory.
								rmdir("/home/domain/$DomainRef->{'dom_grp'}");
							}
							else
							{
								# Try to configure SSL.
								$Return = ConfigureSSL($DomainRef, $FormDataRef);
							}

							# If things are still going good...
							unless ($Return)
							{
								# Delete the domain and recreate it.
								VirtualDomain::del_vws($DomainRef->{'vdn'}, 
													   $DomainRef->{'dom_grp'}, 
													   $DomainRef->{'vdadm'}, 
													   "$DomainRef->{'vdip'}:$DomainRef->{'port'}");
								VirtualDomain::create_vd($FormDataRef->{'ip_address'}, 
														 $FormDataRef->{'name'}, 
														 $FormDataRef->{'admin_name_login'}, 
														 $FormDataRef->{'alias'}, 
														 $DomainRef->{'dom_grp'},
														 $FormDataRef->{'ssi_active'},
														 $FormDataRef->{'cgi_active'},
														 $SSLState,
														 $FormDataRef->{'fpx_active'});
								
								# Run through the list of network interfaces...
								foreach $NetworkInterface (@NetworkInterfaces) 
								{
									# Try to find the address in the interface list...
									if ($NetworkInterface->{'ipaddress'} eq $DomainRef->{'vdip'})
									{
										# Cache the network information.
										$VirtualIP	= $NetworkInterface->{'virtual'};
										$Netmask	= $NetworkInterface->{'netmask'};
										last;
									}
								}
	
								# If there is a matching IP or network mask...
								if (($DomainRef->{'vdip'} ne $FormDataRef->{'ip_address'}) || 
									($Netmask ne $FormDataRef->{'network_mask'}))  
								{
									my ($CurrentAddressIP, $CurrentName, $CurrentGroup) = VirtualDomain::ip_domain($DomainRef->{'vdip'}, $DomainRef->{'vdn'});
	
									# Try to match for an existing interface...
									if ((($CurrentAddressIP eq '') && ($VirtualIP)) || 
										(($DomainRef->{'vdip'} eq $FormDataRef->{'ip_address'}) && 
										 ($Netmask ne $FormDataRef->{'network_mask'}) && 
										 ($VirtualIP))) 
									{
										# Delete the interface.
										NetConfig->deleteNetInterface('', $DomainRef->{'vdip'});
										
										# Delete the host route associated with that interface. 
										# Assume that all virtual domains reside on eth0:X alias devices.
										RouteConfig->deleteRoute($DomainRef->{'vdip'}, '0.0.0.0', '', 'eth0');
									}
	
									# Create the virtual IP address (again)...
									if ('yes' eq VirtualDomain::create_vip($FormDataRef->{'ip_address'},
																		   $FormDataRef->{'network_mask'},
																		   $DomainRef->{'dom_grp'},
																		   $FormDataRef->{'admin_name_login'},
																		   $DomainRef->{'vdn'}))
									{
										# Remove VFTP support for the old IP address ....
										VirtualDomain::delFtp($DomainRef->{'vdip'}, $DomainRef->{'dom_grp'});
	
										# Fix up the hosts file.
										VirtualServ::setHostFile($DomainRef->{'vdn'}, '', $FormDataRef->{'ip_address'}, '', $DomainRef->{'vdn'});
									}
									else
									{
										# Set the return code.
										$Return = 750016;
									}
								}
	
								# If things are still going good...
								unless ($Return)
								{
									# If the domain name changed...
									if ($FormDataRef->{'name'} ne $DomainRef->{'vdn'}) 
									{
										# Delete the old link.
										unlink("/home/domain/$DomainRef->{'vdn'}");
										
										# If the new directory is there (should be) then...
										if ( -e "/home/domain/$DomainRef->{'dom_grp'}")
										{
											# Create the new directory link.
											symlink("/home/domain/$DomainRef->{'dom_grp'}", "/home/domain/$FormDataRef->{'name'}");
										}
										
										# Check if default domain is being changed then changed the postmaster entry from /etc/alias
										if ('domain1' eq $DomainRef->{'dom_grp'}) 
										{   
											# Overwrite a new postmaster alias.
											Usable::write_postmaster_alias("postmaster\@$FormDataRef->{'name'}");
										}
	
										# Realign the mail domain.
										Mail::set_mail_domain($DomainRef->{'dom_grp'}, $FormDataRef->{'name'});	
	
										# Fix up the hosts file.
										VirtualServ::setHostFile($FormDataRef->{'name'}, '', $FormDataRef->{'ip_address'}, '', $DomainRef->{'vdn'});	
									}
	
									# If a new administrator is defined then change the current administrator...
									if ($DomainRef->{'vdadm'} ne $FormDataRef->{'admin_name_login'}) 
									{
										my $UserGID		= (getpwnam($DomainRef->{'vdadm'}))[3];
										my $UserDomain;
										my $UserID		= getpwnam($FormDataRef->{'admin_name_login'});
										
										# If the user exists...
										if ($UserID)
										{
											# Set the return code.
											$Return = 750015;
										}
										elsif (1 != Users::usrMod($FormDataRef->{'admin_name_login'}, 
																  $DomainRef->{'vdadm'}, 
																  $FormDataRef->{'admin_name_login'}, 
																  "/home/domain/$DomainRef->{'dom_grp'}/users/$FormDataRef->{'admin_name_login'}", 
																  $UserGID, 
																  '', 
																  $DomainRef->{'dom_grp'}))
										{
											# Set the return code.
											$Return = 750019;
										}
										else
										{
											# Get the use alias of old Primary Domain Administrator (PDA).
											my $Alias = DomainMail::userAlias($DomainRef->{'vdadm'}, $DomainRef->{'vdn'});
											
											# Delete the old alias and create a new one.
											DomainMail::modAlias($DomainRef->{'vdadm'});
											DomainMail::modAlias($FormDataRef->{'admin_name_login'}, $FormDataRef->{'name'}, $Alias);
										}
									}
	
									# If things are still going good...
									unless ($Return)
									{
										my ($User, $Level) = Authorize::getTargetUser();
										
										# If the domain alias changed...
										if ($FormDataRef->{'alias'} ne $DomainRef->{'vdalias'}) 
										{
											# Update the mail list.
											Mail::set_mail_domain_aliases($DomainRef->{'dom_grp'}, $FormDataRef->{'alias'});
										}
										
										# Enforce consistency in the virtusertable...
										if (($FormDataRef->{'name'} ne $DomainRef->{'vdn'}) || 
											($FormDataRef->{'alias'} ne $DomainRef->{'vdalias'})) 
										{
											# Normalize the aliases.
											Virtusertable::normalize_aliases($DomainRef->{'vdn'}, $FormDataRef->{'name'});
										}
	
										# If mail support is selected...
										if ($FormDataRef->{'mail_active'}) 
										{
											# Turn on mail.
											Mail::enable_mail($DomainRef->{'dom_grp'}, $Level);
										}
										else 
										{
											# Turn off mail.
											Mail::disable_mail($DomainRef->{'dom_grp'}, $Level);
										}
	
										# Update the mail settings.
										Mail::set_mail_domain($DomainRef->{'dom_grp'}, $FormDataRef->{'name'});
										Mail::set_mail_domain_aliases($DomainRef->{'dom_grp'}, $FormDataRef->{'alias'});	       
										Safe::system('/etc/rc.d/init.d/sendmail', 'restart');
	
										# Make sure the IP address is not in use by another domain...
										if ($FormDataRef->{'ftp_active'})
										{
											# Enable virtual FTP.
											VirtualDomain::setFtp($FormDataRef->{'ip_address'}, $DomainRef->{'dom_grp'}, $FormDataRef->{'admin_name_login'}, $FormDataRef->{'name'});
										}
										else
										{  
											# Delete the current configuration...
											VirtualDomain::delFtp($FormDataRef->{'ip_address'}, $DomainRef->{'dom_grp'});
										}
	
										# If things are still going good...
										unless ($Return)
										{
											my $Monitor	= new MonSettings($FormDataRef->{'old_name'});
											my $Port	= (($FormDataRef->{'ssl_active'}) ? 443 : 80);
											my $ConfigurationFound;
											
											# To make sure front page configuration exists for the domain, 
											# check for _vti_bin and _vti_pvt directories.
											if (( -d "/home/domain/$DomainRef->{'dom_grp'}/web/_vti_bin") && 
												( -d "/home/domain/$DomainRef->{'dom_grp'}/web/_vti_pvt")) 
											{
												$ConfigurationFound = 1;
											}
											
											# If activation is needed...
											if (($FormDataRef->{'fpx_active'}) && 
												($DomainRef->{'vdfpx'} ne 'yes')) 
											{
												# Activate FrontPage extensions.
												VirtualDomain::activateFpx($FormDataRef->{'name'}, 
																		   $Port, 
																		   $FormDataRef->{'admin_name_login'}, 
																		   $FormDataRef->{'password_frontpage'}, 
																		   $DomainRef->{'dom_grp'});
											}
											
											# In cases where FrontPage extension files exist for this web site,
											# run resetPasswordFpx to reset the FrontPage user when
											#  1. FrontPage support is being re-activated or the password is being changed, or
											#  2. the primary domain admin is being changed
											if (1 == $ConfigurationFound) 
											{
												# Branch for each of the cases listed above.
												if ((('yes' ne $DomainRef->{'vdfpx'}) && 
													 ($FormDataRef->{'fpx_active'})) || 
													($FormDataRef->{'password_frontpage'}))
												{
													# Case 1: FP re-activation or password change
													VirtualDomain::resetPasswordFpx($FormDataRef->{'name'},
																					$Port,
																					$DomainRef->{'vdadm'},
																					$FormDataRef->{'admin_name_login'},
																					$FormDataRef->{'password_frontpage'},
																					$DomainRef->{'dom_grp'});
												}
												elsif ($DomainRef->{'vdadm'} ne $FormDataRef->{'admin_name_login'})
												{
													# Case 2: The PDA is being changed -- use the new username for the password.
													VirtualDomain::resetPasswordFpx($FormDataRef->{'name'},
																					$Port,
																					$DomainRef->{'vdadm'},
																					$FormDataRef->{'admin_name_login'},
																					$FormDataRef->{'admin_name_login'},
																					$DomainRef->{'dom_grp'});
												}
											}
	
											# Open the configuration.
											if (0 == $Monitor->open())
											{
												# Get the service check state.
												$Monitor->modifyDomainName($FormDataRef->{'name'});

												# Close the configuration.
												$Monitor->close();
											}
											
											# Restart apache apache server now.
											Safe::system("(/etc/rc.d/init.d/httpd stop; /bin/sleep 2; /etc/rc.d/init.d/httpd start)&");
	
											# Quota for domain should go here.
											my $GroupID = getgrnam($DomainRef->{'dom_grp'});
											 
											# Set the quota.
											Quotaall::setquota($GroupID, $FormDataRef->{'disk_quota'}, 1);
										}
									}
								}
							}
						}
					}
				}

				# If the root is to be reloaded...
				if ((0 == $Return) && ($ReloadRoot))
				{
					my $cgi = new CGI;
					
					# Unlock the lock file.
					flock(LOCKFILE, 8);
					
					# Close the file.
					close(LOCKFILE);
					
					# External redirect.
					print $cgi->header(-expires=>'now'),
						  $cgi->start_html({ -onLoad=>"javascript: parent.location.href='/login/index.cgi?domain=$FormDataRef->{'name'}';"}),
						  $cgi->end_html();
					exit(0);
				}
			}
			else
			{
				# Set the return code.
				$Return = 750020;
			}
		}
		else
		{
			# Set the return code.
			$Return = 750012;
		}

		# Close the file.
		close(LOCKFILE);
	}
	else
	{
		# Set the return code.
		$Return = 750012;
	}

	return $Return;
}

###############################################################################
# RenderDefault
###############################################################################

sub RenderDefault
{
	my ($cgi, $DisplayDataRef, $Redirect) = @_;
	my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
	my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'domain_settings.lrc', 'common.lrc' ]);
	my $Cookie				= new CookieData();
	my %_R					= ( 'DOMAIN_NAME'				=> '',
								'DOMAIN_ALIAS'				=> '',
								'IP_ADDRESS_LABEL'			=> '',
								'NETMASK'					=> '',
								'ADMIN_FULL_NAME'			=> '',
								'DOMAIN_ADMINISTRATOR'		=> '',
								'PASSWORD'					=> '',
								'PASSWORD_CONFIRM'			=> '',
								'DOMAIN_DISK_QUOTA'			=> '',
								'MB'						=> '',
								'MAIL_SUPPORT'				=> '',
								'FTP_SUPPORT'				=> '',
								'USE_SSL'					=> '',
								'CONFIGURE_SSL'				=> '',
								'SSI_SUPPORT'				=> '',
								'SSI_SUPPORT_LABEL'			=> '',
								'CGI_SUPPORT'				=> '',
								'CGI_SUPPORT_LABEL'			=> '',
								'FRONTPAGE_SUPPORT'			=> '',
								'FRONTPAGE_SUPPORT_LABEL'	=> '',
								'ENABLED'					=> '',
								'DISABLED'					=> '',
								'APPLY'						=> '' );
	my %FormData			= (	'name'							=> '',
								'old_name'						=> '',
								'alias'							=> '',
								'ip_address'					=> '',
								'network_mask'					=> '',
								'admin_name_login'				=> '',
								'disk_quota'					=> '',
								'old_disk_quota'				=> '',
								'mail_active'					=> '',
								'ftp_active'					=> '',
								'ssi_active'					=> '',
								'cgi_active'					=> '',
								'fpx_active'					=> '',
								'password_frontpage'			=> '', 
								'password_confirm_frontpage'	=> '',
								'ssl_active'					=> 0,
								'certificate_type'				=> 'none' );
	my @DomainList			= VirtualDomain::list_vws();
	my @NetworkList			= NetConfig->listNetInterfaces();
	my %Parameters;
	my $CookieFile			= '/usr/ins/intel/gui/cookies/domain_modify-';
	my $User;
	my $UserGID;
	my $Key;
	my $SSLDetails;
	my $DomainRef;
	my $Domain;
	my $Level;
	my $Index;
	my $Item;
	my $Feedback;
	my $Result;
	my $html;

    # Get the string resources.
	$StringResource->GetStrings(\%_R);
	
	# Check if user is authorized to run this script
	unless (Authorize::checkAuthorization()) { return 0; }

	# Get the user and priveledge information.
	($User, $Level) = Authorize::getTargetUser();
	$UserGID		= (getpwnam($User))[3];

	# Get the domain and level of the user.
	($Domain, $Level) = Authorize::getTargetDomain();

	# Make a unique name for the cookie.
	$CookieFile .= $Domain . '-' . $cgi->remote_user();
	
	# Run through the domain list...
	foreach $Item (@DomainList) 
	{
		# If the domain name matches...
		if ($Domain eq $Item->{'vdn'})
		{
			# Get the index.
			$Index = $Item->{'idx'};
			last;
		}
	}

	# Get the domain information.
	$DomainRef = $DomainList[$Index];

	# Attach to the cookie file.
	$Cookie->Attach($CookieFile);
	
	# Make sure the dirty bit has not been set...
	unless ($Cookie->GetValueKeyed('ssl-dirty-bit'))
	{
		# If the domain is configured for SSL...
		if (('yes' eq $DomainRef->{'vdssl'}) &&
			( -e "$DomainRef->{'dom_hom'}/$DomainRef->{'dom_grp'}/cert/certificate" ))
		{
			# SSL is active.
			$FormData{'ssl_active'}			= 1;
			$FormData{'certificate_type'}	= ( -e "$DomainRef->{'dom_hom'}/$DomainRef->{'dom_grp'}/cert/request" ) ? 'self' : 'ca';
	
			# If getting SSL data...
			if ((10000 == $UserGID) &&
				(defined($cgi->param('apply'))) ||
				(defined($cgi->param('configure_ssl'))))
			{
				my %CertificateData;
				my $SSLData			= new SSL("$DomainRef->{'dom_hom'}/$DomainRef->{'dom_grp'}/cert", '', 'key', 'request', 'certificate');
	
				# If the configuration is self-signed...
				if ('self' eq $FormData{'certificate_type'})
				{
					# Get the private key and certificate...
					$SSLData->GetSubject(\%CertificateData);
				}
				else
				{
					# Get the private key and certificate...
					$CertificateData{'private_key'} = $SSLData->GetPrivateKey();
					$CertificateData{'certificate'} = $SSLData->GetCertificate();
	
				}
				
				# Run through all of the certificate data...
				foreach $Key (keys(%CertificateData))
				{
					# Save the value.
					$Cookie->SetValueKeyed($Key, $CertificateData{$Key});
				}
			}
		}
	}
	else
	{
		# Get the SSL data.
		$FormData{'ssl_active'}			= $Cookie->GetValueKeyed('ssl_active');
		$FormData{'certificate_type'}	= $Cookie->GetValueKeyed('certificate_type');
	}
	
	# Detach from the cookie.
	$Cookie->Detach();

	# If modifying the domain...
	if ((10000 == $UserGID) &&
		(defined($cgi->param('apply'))) ||
		(defined($cgi->param('configure_ssl'))))
	{
		my @KeysSSL	= ( 'country',
						'province',
						'locality',
						'organization',
						'organizational_unit',
						'common_name',
						'e_mail',
						'private_key',
						'certificate' );

		# Get the form data from the form...
		foreach $Key (keys(%FormData))
		{
			# Get the data...
			$FormData{$Key} = (defined($cgi->param($Key))) ? $cgi->param($Key) : $FormData{$Key};
		}

		# Attach to the cookie file.
		$Cookie->Attach($CookieFile);

		# Get the SSL data.
		$FormData{'ssl_active'}			= $Cookie->GetValueKeyed('ssl_active');
		$FormData{'certificate_type'}	= $Cookie->GetValueKeyed('certificate_type');

		# Run through the SSL keys...
		foreach $Key (@KeysSSL)
		{
			# Import the SSL entry.
			$FormData{$Key} = $Cookie->GetValueKeyed($Key);

			# Trim the whitespace.
			$FormData{$Key} =~ s/^\s*(.*?)\s*$/$1/ if($FormData{$Key});
		}

		# Detach from the cookie.
		$Cookie->Detach();

		# If modifying the domain...
		if (defined($cgi->param('apply')))
		{
			# Validate the form data.
			if (0 == ($Result = ValidateFormData(\%FormData)))
			{
				# Try to modify the domain...
				if (0 == ($Result = ModifyDomain(\%FormData)))
				{
					# Set the result.
					$DisplayDataRef->{'result'}	= $Result;
				}
			}

			# If there was an error...
			if ($Result)
			{
				# Get the error string.
				$Feedback = $StringResource->_S($Result);
			}
		}
		else
		{
			# Validate the data...
			if (($FormData{'name'} !~ /^(\w|-|\.|_)+$/) ||
				($FormData{'name'} =~ /^domain\d+$/))
			{
				# Set the return code.
				$Result = 750021;
			}
			elsif (($FormData{'ip_address'} !~ /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/) ||
				   (1 > $1) || (254 < $1) || (0 > $2) || (254 < $2) ||
				   (0 > $3) || (254 < $3) || (0 > $4) || (254 < $4))
			{
				# Set the return code.
				$Result = 750022;
			}
			else
			{
				my @DomainList	= VirtualDomain::list_vws();
				my $Item;
				
				# Run through the domain list...
				foreach $Item (@DomainList) 
				{
					# If the addresses match but the domain does not...
					if (($FormData{'ip_address'} eq $Item->{'vdip'}) &&
						($FormData{'old_name'}   ne $Item->{'vdn'}))
					{
						# Check for an existing virtual IP squatter...
						$Result = 750025 if(('yes' eq $Item->{'vdssl'}) && ( -e "$Item->{'dom_hom'}/$Item->{'dom_grp'}/cert/certificate" ));
					}

					# Bail out if needed...
					last if($Result);
				}
			}

			# If there are no problems...
			unless (defined($Result))
			{
				my $URL = "/cgi-bin/common/ssl_forms.cgi?type=default&cookie-file=domain_modify-" . $Domain . '-' . $cgi->remote_user();
	
				# Attach to the cookie file.
				$Cookie->Attach($CookieFile);
	
				# Run through all of the keys...
				foreach $Key (keys(%FormData))
				{
					# Save the data.
					$Cookie->SetValueKeyed($Key, $FormData{$Key});
				}

				# Set the POST URI.
				$Cookie->SetValueKeyed('post_uri', "/cgi-bin/user=$User/domain=$Domain/domain/domain_settings.cgi?type=default");
	
				# Set the root link.
				$Cookie->SetValueKeyed('root-link', 'domain_settings');
	
				# Set up the cookie crumbs.
				$Cookie->SetValueKeyed('b-class0', 'domain_settings');
				$Cookie->SetValueKeyed('b-label0', 'IDT:7500');
				$Cookie->SetValueKeyed('b-rpath0', '/usr/ins/intel/gui/locale');
				$Cookie->SetValueKeyed('b-uri0',   "/cgi-bin/user=$User/domain=$Domain/domain/domain_settings.cgi?type=default");
	
				# Prime the common name.
				$Cookie->SetValueKeyed('common_name', $FormData{'name'});
	
				# Try to prime the e-mail address...
				if (($FormData{'name'}) && ($FormData{'admin_name_login'}))
				{
					# Prime the common name.
					$Cookie->SetValueKeyed('e_mail', $FormData{'admin_name_login'} . '@' . $FormData{'name'});
				}
				else
				{
					# Prime the common name.
					$Cookie->SetValueKeyed('e_mail');
				}
	
				# Detach from the cookie.
				$Cookie->Detach();
	
				# Redirect to the SSL forms.
				print $cgi->redirect( -url => $URL );
				return;
			}
			else
			{
				# Get the error string.
				$Feedback = $StringResource->_S($Result);
			}
		}
	}
	elsif (defined($cgi->param('cookie-file')))
	{
		# Attach to the cookie file.
		$Cookie->Attach($CookieFile);

		# Set the dirty bit.
		$Cookie->SetValueKeyed('ssl-dirty-bit', 1);

		# Get the SSL data.
		$FormData{'ssl_active'}			= $Cookie->GetValueKeyed('ssl_active');
		$FormData{'certificate_type'}	= $Cookie->GetValueKeyed('certificate_type');

		# Run through the SSL keys...
		foreach $Key (keys(%FormData))
		{
			# Import the SSL entry.
			$FormData{$Key} = $Cookie->GetValueKeyed($Key);

			# Trim the whitespace.
			$FormData{$Key} =~ s/^\s*(.*?)\s*$/$1/ if($FormData{$Key});
		}

		# Detach from the cookie.
		$Cookie->Detach();
	}
	else
	{
		# Attach to the cookie file.
		$Cookie->Attach($CookieFile);
		
		# Delete the cookie.
		$Cookie->Delete();
		
		# Set up the quota parameters.
		$Parameters{'domain'}	= $Domain;

		$FormData{'name'}				= $DomainRef->{'vdn'} if(defined($DomainRef->{'vdn'}));
		$FormData{'alias'}				= $DomainRef->{'vdalias'} if(defined($DomainRef->{'vdalias'}));
		$FormData{'ip_address'}			= $DomainRef->{'vdip'} if(defined($DomainRef->{'vdip'}));
		$FormData{'admin_name_login'}	= $DomainRef->{'vdadm'} if(defined($DomainRef->{'vdadm'}));
		$FormData{'disk_quota'}			= (Quotaall::repquota(\%Parameters))[1] / 1024;
		$FormData{'mail_active'}		= DomainMail::statMail($DomainRef->{'vdn'});
		$FormData{'ftp_active'}			= VirtualFtp::getFtpStatus($DomainRef->{'vdip'}, $DomainRef->{'dom_grp'});
		$FormData{'ssi_active'}			= $DomainRef->{'vdssi'} if(defined($DomainRef->{'vdssi'}));
		$FormData{'cgi_active'}			= $DomainRef->{'vdcgi'} if(defined($DomainRef->{'vdcgi'}));
		$FormData{'fpx_active'}			= $DomainRef->{'vdfpx'} if(defined($DomainRef->{'vdfpx'}));

		# Run through the network interfaces...
		foreach $Item (@NetworkList) 
		{
			# If the domain IP matches the item IP address...
			if ($DomainRef->{vdip} eq $Item->{'ipaddress'})
			{
				# Get the network mask.
				$FormData{'network_mask'} = $Item->{'netmask'};
				last;
			}
		}
	}

	# Set up the quota parameters.
	$Parameters{'domain'}		= $Domain;
	
	# Get the old name and disk quota.
	$FormData{'old_name'}		= $DomainRef->{'vdn'} if(defined($DomainRef->{'vdn'}));
	$FormData{'old_disk_quota'}	= (Quotaall::repquota(\%Parameters))[1] / 1024;
	
	# Attach to the cookie file.
	$Cookie->Attach($CookieFile);
	
	# Run through all of the keys...
	foreach $Key (keys(%FormData))
	{
		# Save the data.
		$Cookie->SetValueKeyed($Key, $FormData{$Key});
	}
	
	# Detach from the cookie.
	$Cookie->Detach();

	# If the server administrator is viewing the page...
    if (10000 == $UserGID)
	{
		# Set the help indicator.
		$DisplayDataRef->{'helpIndicator'} = 'server';
		
		# Re-align the checkboxes...
		$FormData{'mail_active'}	= ($FormData{'mail_active'}) ? 'checked' : '';
		$FormData{'ftp_active'}		= ($FormData{'ftp_active'}) ? 'checked' : '';
		$FormData{'ssi_active'}		= ($FormData{'ssi_active'}) ? 'checked' : '';
		$FormData{'cgi_active'}		= ($FormData{'cgi_active'}) ? 'checked' : '';
		$FormData{'fpx_active'}		= ($FormData{'fpx_active'}) ? 'checked' : '';

		# Set the SSL status text...
		if ($FormData{'ssl_active'})
		{
			# Using SSL...
			$SSLDetails = '<font color=green>' . (('self' eq $FormData{'certificate_type'}) ? $StringResource->_S('SSL_CONFIGURED_SELF') : $StringResource->_S('SSL_CONFIGURED_CA')) . '<font>';
		}
		else
		{
			# Not using SSL.
			$SSLDetails = '<font color=red>' . $StringResource->_S('SSL_NOT_CONFIGURED') . '<font>';
		}

		$html = <<HERE;
<form method=post action=domain_settings.cgi enctype="application/x-www-form-urlencoded">
	<input type=hidden name=type value=default>
	<input type=hidden name=old_name value=$FormData{'old_name'}>
	<input type=hidden name=old_disk_quota value=$FormData{'old_disk_quota'}>
	<table border=0 width=375 bgcolor=#eeeeee>
	<tr>
		<td>
			&nbsp;
			<table border=0 width=100%>
			<tr>
				<td><b>$_R{'DOMAIN_NAME'}</b></td>
				<td><input type=text size=25 maxlength=72 name=name value='$FormData{'name'}'></td>
			</tr>
			<tr>
				<td><b>$_R{'DOMAIN_ALIAS'}</b></td>
				<td><input type=text size=25 maxlength=72 name=alias value='$FormData{'alias'}'></td>
			</tr>
			<tr>
				<td><b>$_R{'IP_ADDRESS_LABEL'}</b></td>
				<td><input type=text size=15 maxlength=15 name=ip_address value='$FormData{'ip_address'}'></td>
			</tr>
			<tr>
				<td><b>$_R{'NETMASK'}</b></td>
				<td><input type=text size=15 maxlength=15 name=network_mask value='$FormData{'network_mask'}'></td>
			</tr>
			<tr>
				<td colspan=2>&nbsp</td>
			</tr>
			<tr>
				<td nowrap><b>$_R{'DOMAIN_ADMINISTRATOR'}</b></td>
				<td><input type=text size=20 maxlength=32 name=admin_name_login value='$FormData{'admin_name_login'}'></td>
			</tr>
			<tr>
				<td nowrap><b>$_R{'DOMAIN_DISK_QUOTA'}</b></td>
				<td><input type=text size=4 maxlength=10 name=disk_quota value='$FormData{'disk_quota'}'>$_R{'MB'}</td>
			</tr>
			<tr>
				<td nowrap><b>$_R{'USE_SSL'}</b></td>
				<td><input type=submit name=configure_ssl value='$_R{'CONFIGURE_SSL'}' class=clsButton></td>
			</tr>
			<tr>
				<td colspan=2>&nbsp;$SSLDetails<br>&nbsp;</td>
			</tr>
			<tr>
				<td nowrap colspan=2><input type=checkbox value=Yes name=mail_active $FormData{'mail_active'}>&nbsp;<b>$_R{'MAIL_SUPPORT'}</b></td>
			</tr>
			<tr>
				<td nowrap colspan=2><input type=checkbox value=Yes name=ftp_active $FormData{'ftp_active'}>&nbsp;<b>$_R{'FTP_SUPPORT'}</b></td>
			</tr>
			<tr>
				<td nowrap colspan=2><input type=checkbox value=Yes name=ssi_active $FormData{'ssi_active'}>&nbsp;<b>$_R{'SSI_SUPPORT'}</b></td>
			</tr>
			<tr>
				<td nowrap colspan=2><input type=checkbox value=Yes name=cgi_active $FormData{'cgi_active'}>&nbsp;<b>$_R{'CGI_SUPPORT'}</b></td>
			</tr>
			<tr>
				<td nowrap colspan=2><input type=checkbox value=Yes name=fpx_active $FormData{'fpx_active'}>&nbsp;<b>$_R{'FRONTPAGE_SUPPORT'}</b></td>
			</tr>
			<tr>
				<td nowrap><b>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;$_R{'PASSWORD'}</b></td>
				<td><input type=password size=20 maxlength=25 name=password_frontpage value='$FormData{'password_frontpage'}'></td>
			</tr>
			<tr>
				<td nowrap><b>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;$_R{'PASSWORD_CONFIRM'}</b></td>
				<td><input type=password size=20 maxlength=25 name=password_confirm_frontpage value='$FormData{'password_confirm_frontpage'}'></td>
			</tr>
			<tr>
				<td colspan=2>&nbsp</td>
			</tr>
			<tr>
				<td colspan=2 align=right><input type=submit name=apply value='$_R{'APPLY'}' class=clsButton></td>
			</tr>
			</table>
		</td>
	</tr>
	</table>
</form>
HERE
	}
	else
	{
		# Set the help indicator.
		$DisplayDataRef->{'helpIndicator'} = 'domain';
		
		# Get the display status.
		$FormData{'ssi_active'}		= ($FormData{'ssi_active'}) ? "<font color=green>$_R{'ENABLED'}</font>" : "<font color=red>$_R{'DISABLED'}</font>";
		$FormData{'cgi_active'}		= ($FormData{'cgi_active'}) ? "<font color=green>$_R{'ENABLED'}</font>" : "<font color=red>$_R{'DISABLED'}</font>";
		$FormData{'fpx_active'}		= ($FormData{'fpx_active'}) ? "<font color=green>$_R{'ENABLED'}</font>" : "<font color=red>$_R{'DISABLED'}</font>";

		$html = <<HERE;
<table border=0 width=375 bgcolor=white>
	<tr>
		<td><b>$_R{'DOMAIN_NAME'}</b></td>
		<td>$FormData{'name'}</td>
	</tr>
	<tr>
		<td><b>$_R{'DOMAIN_ALIAS'}</b></td>
		<td>$FormData{'alias'}</td>
	</tr>
	<tr>
		<td><b>$_R{'IP_ADDRESS_LABEL'}</b></td>
		<td>$FormData{'ip_address'}</td>
	</tr>
	<tr>
		<td><b>$_R{'NETMASK'}</b></td>
		<td>$FormData{'network_mask'}</td>
	</tr>
	<tr>
		<td colspan=2>&nbsp</td>
	</tr>
	<tr>
		<td nowrap><b>$_R{'DOMAIN_ADMINISTRATOR'}</b></td>
		<td>$FormData{'admin_name_login'}</td>
	</tr>
	<tr>
		<td nowrap><b>$_R{'DOMAIN_DISK_QUOTA'}</b></td>
		<td>$FormData{'disk_quota'} $_R{'MB'}</td>
	</tr>
	<tr>
		<td colspan=2>&nbsp</td>
	</tr>
	<tr>
		<td nowrap><b>$_R{'SSI_SUPPORT_LABEL'}</b></td>
		<td>$FormData{'ssi_active'}</td>
	</tr>
	<tr>
		<td nowrap><b>$_R{'CGI_SUPPORT_LABEL'}</b></td>
		<td>$FormData{'cgi_active'}</td>
	</tr>
	<tr>
		<td nowrap><b>$_R{'FRONTPAGE_SUPPORT_LABEL'}</b></td>
		<td>$FormData{'fpx_active'}</td>
	</tr>
</table>
HERE
	}

	# Fill out the display information.
	$DisplayDataRef->{'q'}				= $cgi;
	$DisplayDataRef->{'html'}			= $html;
	$DisplayDataRef->{'class'}			= 'domain_settings';
	$DisplayDataRef->{'id'}				= 7500;
	$DisplayDataRef->{'title'}			= $StringResource->_S('IDT:7500');
	$DisplayDataRef->{'root-link'}		= 'domain_settings';
	$DisplayDataRef->{'result'}			= $Result;
	$DisplayDataRef->{'feedback'}		= $Feedback;

	return 0;
}

###############################################################################
# Main
###############################################################################

sub Main
{
	my $cgi			= shift;
	my %DisplayData;
	my $FormType	= $cgi->param('type');
	my $Redirect	= 0;

	# Safe default.
	$FormType = 'default' unless(defined($FormType));
	
	SWITCH:
	{
		if ('default' eq $FormType) { $FormType = RenderDefault($cgi, \%DisplayData); last SWITCH; }
		$FormType = 0;
	}

	# If there is still some form data...
	if ($FormType)
	{
		# Set the redirect.
		$Redirect = 1;
		goto SWITCH;
	}
	else
	{
		# Display the page.
		DisplayMain->printDocument(\%DisplayData);
	}
}

###############################################################################
# The main body of the script.
###############################################################################

my $cgi = new CGI;

Main($cgi);
