package DNSConfig;

###############################################################################
#                                                                             #
# Filename: DNSConfig.pm                                                      #
#                                                                             #
# Description: Manages the DNS (BIND) master configuration file.              #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

use vars qw(@ISA @EXPORT @EXPORT_OK);
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw( ); 

use lib '/usr/ins/intel/gui/perllib';

use strict;
use Daemon;
use Safe;

############

####################################################
# Return code key
# 200 - could not read file
# 201 - insufficient arguments for this function
# 202 - improper arguments
# 203 - cannot add a reserved zone
# 204 - cannot delete a reserved zone
# 205 - cannot toggle a reserved zone
# 206 - zone already exists (for add)
# 207 - zone does not exist (for del)
# 208 - zone does not exist (for toggle)
# 209 - improper zone type (1=master,2=slave) specified
# 210 - zone file already exists when trying to create a master.
# 211 - cannot toggle a zone to its same type.
# 220 - unable to open file
# 221 - unable to get exclusive lock on file
# 222 - unable to close file
# 230 - no zone matched that description
####################################################


my $cr = "\n";
my $semicolon = "\;";
my $doublequote = '"';
my $weasel = 'ferocious';


# private function
sub undblquote
{
	my ($useless,$quotedstring) = @_;
	# expects a text string with some contents.
	# if text string starts with " and ends with ", filter out.

	my ($t1,$t2);
	my (@a1,@a2);

	return '' if ($quotedstring eq '');

	$t1 = $quotedstring;

	if ( ($t1 =~ /^"(\W*)/ ) )
	{
		# Then we have quotes on beginning.

		@a1 = split('"',$t1);
		$t1 =  $a1[1];
	}
	if ( ($t1 =~ /"(\W*)^/ ) )
	{
		# Then we have quotes on beginning.

		@a1 = split('"',$t1);
		$t1 =  $a1[0];
	}
	return $t1;
}





# private function
sub parseconfig
{
	my ($useless,$filehandle,$zonedata) = @_;
	# Expects to receive an open filehandle from the caller.
	
	my $unusable;
	my $lookfor_options;
	my $lookfor_zone;
	my $opts_collect;

	my $zonecount;

	# Little temporary values.

	my (@a1,@a2,@a3,$i,$j,$t1,$t2,$t3,$t4,$strsave);
	
	# Begin operation.

	$unusable = 0;
	$lookfor_options = 0;
	$lookfor_zone = 0;
	$opts_collect = '';
	$zonecount = 0;


	while (<$filehandle>)
	{
		chomp($_);

		# ignore lines with only white space
		next if !(/(.+)/);

		# strip leading whitespace
		
		$t4 = (/\s+/);
		if ( ($` eq '') && ($' ne '') )
		{
			# If the text before string match is null, and the text after is not null
			$_ = $';
		}


		# ignore commented lines
		next if (/^#/);
		next if (/^\/\//);
		next if (/^$semicolon/);

		# strip out everything after+including semicolon from the end of any line
		if (/;/)
		{
			$_ = $`;
		}

		# strip out // style comments from the end of any line
		if (/\/\//)
		{
			$_ = $`;
		}

		next if ($_ eq '');

#		print ("    PARSER reading line [" . $_ . "]\n");

		
		if ($lookfor_options)
		{
			# Start ignoring options here.
			
			if ( /^\}/ )
			{
				# If line starts with the end of options marker
#				print("----End of option marker found!\n");
				$lookfor_options = 0;
				next;
			}
			else
			{
#				print("----grabbed option line above.\n");
				next;
			}

		}


		if (!($lookfor_options))
		{

			if ( /^options(\s*)\{/ )
			{
				# If we find the options header here, let's start watching.
#				print("----Options key discovered!\n");				

				$lookfor_options = 1;
				next;
			}
		}
		
		# Otherwise, carry on with normal routine of finding zones.
				
				
#		print ("----Getting non-option line [" . $_ . "]\n");	


		if ($lookfor_zone)
		{
			# Start catching zone data here.
			
			if ( /^\}/ )
			{
				# If line starts with the end of zone marker
#				print("*---End of zone marker found!\n");
				$lookfor_zone = 0;
				$zonecount++;
				next;
			}
			elsif ( /^file(\s*)/ )
			{
				# If we find the zone file value here, catch it.

				@a1 = split;

				$t1 = $a1[1];
				$t2 = undblquote(0,$t1);

#				print ("----Filtered zone filename is [" . $t2 . "] \n");

				$$zonedata[$zonecount][1] = $t2;

				next;
			}
			elsif ( /^type(\s*)/ )
			{
				# If we find the zone type value here, catch it.

				@a1 = split;

				$t1 = $a1[1];
				$t2 = undblquote(0,$t1);

#				print ("----Filtered zone type is [" . $t2 . "] \n");

				if ('master' eq lc($t2))
				{
					$$zonedata[$zonecount][2] = 1;
				}
				elsif ('slave' eq lc($t2))
				{
					$$zonedata[$zonecount][2] = 2;
				}
				else
				{
					$$zonedata[$zonecount][2] = 0; #undefined
				}
				next;
			}

			elsif ( /^masters(\s*)/ && ($$zonedata[$zonecount][2] eq 2) )
			{
				# If we find the zone masters value here, catch it if we are a slave.

#				print ("----RAW zone masters value is [" . $_ . "] \n");

				$t1 = $_;

				$t1 =~ s/^masters//;
#				print ("----Semi-filtered masters value is [" . $t1 . "] \n");

				$t1 =~ s/\{//;

#				print ("----Semi-filtered masters value is [" . $t1 . "] \n");

				$t1 =~ s/^(\s*)//;
#				print ("----Semi-filtered masters value is [" . $t1 . "] \n");
				


				$$zonedata[$zonecount][3] = $t1;


				next;
			}



			else
			{
#				print("----grabbed zone data line above.\n");
				next;
			}

		}

		if (!($lookfor_zone))
		{

			if ( /^zone(\s*)/ && /\{/ )
			{
				# If we find the zone header here, let's start watching.
#				print("*---Zone key discovered\n");				

				@a1 = split;

				$t1 = $a1[1];
				$t2 = undblquote(0,$t1);

#				print ("----Filtered zone name is [" . $t2 . "] \n");

				if ( ($t2 eq '.') || ($t2 eq '0.0.127.in-addr.arpa') || ($t2 eq 'bind') || ($t2 eq 'localhost') )
				{
					# These domains are hidden - not to be deleted, replaced, or modified
					# Also not to be shown to server GUI.  Hide now.
#					print ("--*-Hidden Domain!  Skipping.\n");
					next;
				}


				$$zonedata[$zonecount][0] = $t2;
				# Got zone name.  Next, wait for file name and type keys.

				$lookfor_zone = 1;
				next;
			}

		}



	}		# END of WHILE loop.
	

	return 0;

}



# public function
sub zone_list
{
	my ($useless,$zonedata) = @_;
	# Expects to use /home/named/var/named/named.conf as a config file

	my $unusable;


	open (BINDCONFIG, "</home/named/var/named/named.conf") or return 220;
	flock (BINDCONFIG, 2) or return 221;

	$unusable = parseconfig(0,\*BINDCONFIG,$zonedata);

	close (BINDCONFIG) or return 222;

	return $unusable;

}


# public function
sub zone_getinfo
{
	my ($useless,$zonedata,$zname1) = @_;
	# Expects to use /home/named/var/named/named.conf as a config file

	# Get information for one zone name only.

	my $unusable;
	my $i;
	my $isfound;

	my @allzonedata;

	if ($zname1 eq '')
	{
		# please give arguments!
		return 201;
	}


	open (BINDCONFIG, "</home/named/var/named/named.conf") or return 220;
	flock (BINDCONFIG, 2) or return 221;

	$unusable = parseconfig(0,\*BINDCONFIG,\@allzonedata);

	close (BINDCONFIG) or return 222;

	if ($unusable)
	{
		return $unusable;
	}

	$isfound = 0;
	$$zonedata[0] = '';
	$$zonedata[1] = '';
	$$zonedata[2] = '';
	$$zonedata[3] = '';

	for ($i = 0; $i < scalar(@allzonedata); $i++)
	{
		# Iterate through each record in this array.

		if ( ($zname1 eq $allzonedata[$i][0]) && (!($isfound)) )
		{
			# Found a match!

			$isfound = 1;
			$$zonedata[0] = $allzonedata[$i][0];
			$$zonedata[1] = $allzonedata[$i][1];
			$$zonedata[2] = $allzonedata[$i][2];
			$$zonedata[3] = $allzonedata[$i][3];
		}

	}	# End of FOR loop.

	if ($isfound)
	{
		return 0;
	}

	return 230;

}



# public function
sub zone_add
{
	my ($useless,$new_zname,$new_zfilename,$new_ztype,$new_zmaster) = @_;
	# Expects to use /home/named/var/named/named.conf as a config file
	# passed variable $new_zfilename should be called as a reference ("\$newfile") to return output.

	my $unusable;

	my @zonedata;

	my $isfound;
	my ($i,$j);
	my $t1;
	my @a1;

	my $tempfilename;

	if ( ($new_zname eq '') || ($new_ztype eq '') )
	{
		# please give arguments!
		return 201;
	}


	open (BINDCONFIG, "</home/named/var/named/named.conf") or return 220;
	flock (BINDCONFIG, 2) or return 221;


	$unusable = parseconfig(0,\*BINDCONFIG,\@zonedata);


	if ($unusable)
	{
		return $unusable;
		# Pass back any error codes we get from parsing.
	}

	if ( ($new_zname eq '.') || ($new_zname eq 'bind') || ($new_zname eq 'localhost') || ($new_zname eq '0.0.127.in-addr.arpa') )
	{
		# Can't add a zone which is already reserved!
		return 203;
	}


	for ($i = 0;  $i < scalar(@zonedata); $i++)
	{
		# Iterate through all zones.  See if we have a match.
		if ($new_zname eq $zonedata[$i][0])
		{
			# Match detected!
			$isfound = 1;
		}
	}

	if ($isfound)
	{
		# Trying to add a duplicate zone!
		close (BINDCONFIG) or return 222;
		return 206;
	}

	if ($new_ztype eq 1)
	{
		$tempfilename = 'master/';
	}
	elsif ($new_ztype eq 2)
	{
		$tempfilename = 'slave/';

		if ($new_zmaster eq '')
		{
			# No master given!  Required for slave operation.
			return 201;
		}
	}
	else
	{
		# Only support types 1 (master) and 2 (slave)
		return 209;
	}


	$tempfilename .= $new_zname . '.zone';

#	print ("Reporting new tentative zone filename of [ $tempfilename ].\n");

		
	if (-f "/home/named/var/named/$tempfilename")
	{
#		if ($new_ztype eq 1)
#		{
#			return 210;
#		}
#
#		else
#		{
#			print ("Removing stale file for slave zone data.\n");
			Safe::backtick("/bin/rm -f /home/named/var/named/$tempfilename");
#		}
	}


	####################################### Fix this (below)

	open (BINDNEW, ">>/home/named/var/named/named.conf") or return 220;

	print BINDNEW $cr;
	print BINDNEW 'zone "' . $new_zname . '" {' . $cr;
	print BINDNEW "\ttype ";

	if ($new_ztype eq 1)
	{
		print BINDNEW "master" . $semicolon . $cr;
	}
	else
	{
		print BINDNEW "slave" . $semicolon . $cr;
	}

	print BINDNEW "\tfile " . '"' . $tempfilename . '"' . $semicolon . $cr;

	if ($new_ztype eq 2)
	{
		print BINDNEW "\tmasters { " . $new_zmaster . $semicolon . ' }' . $semicolon . $cr;
	}
	else
	{
		print BINDNEW "\tnotify yes" . $semicolon . $cr;
	}

	print BINDNEW "\tallow-update { none" . $semicolon . ' }' . $semicolon . $cr;
	print BINDNEW "\tallow-transfer { any" . $semicolon . ' }' . $semicolon . $cr;
	print BINDNEW "\tallow-query { any" . $semicolon . ' }' . $semicolon . $cr;

	print BINDNEW '}' . $semicolon . $cr;


	$$new_zfilename = $tempfilename;

	close (BINDNEW) or return 222;
	close (BINDCONFIG) or return 222;

	return 0;

}



# public function
sub zone_del
{
	my ($useless,$del_zname) = @_;
	# Expects to use /home/named/var/named/named.conf as a config file

	my $unusable;

	my @zonedata;

	my $isfound;
	my ($i,$j);
	my ($t1,$t2);
	my (@a1,@a2);

	my $catchall;
	my $zonefoundflag;

	my $tempfilename;

	if ($del_zname eq '')
	{
		# please give arguments!
		return 201;
	}

	open (BINDCONFIG, "</home/named/var/named/named.conf") or return 220;
	flock (BINDCONFIG, 2) or return 221;

	$unusable = parseconfig(0,\*BINDCONFIG,\@zonedata);

	$catchall = '';


	if ($unusable)
	{
		return $unusable;
		# Pass back any error codes we get from parsing.
	}

	if ( ($del_zname eq '.') || ($del_zname eq 'bind') || ($del_zname eq 'localhost') || ($del_zname eq '0.0.127.in-addr.arpa') )
	{
		# Can't delete a zone which is already reserved!
		return 204;
	}


	for ($i = 0;  $i < scalar(@zonedata); $i++)
	{
		# Iterate through all zones.  See if we have a match.
		if ($del_zname eq $zonedata[$i][0])
		{
			# Match detected!
			$isfound = 1;
		}
	}

	if (!($isfound))
	{
		# Trying to delete a non-existent zone
		close (BINDCONFIG) or return 222;
		return 207;
	}

	$zonefoundflag = 0;

	# Rewind to the beginning of the file.
	seek(BINDCONFIG,0,0);
	
	while(<BINDCONFIG>)
	{
		# ignore commented lines
		next if (/^#/);
		next if (/^\/\//);
		next if (/^$semicolon/);


		if ($zonefoundflag)
		{
			# Start catching zone data here.
			# Look specifically for end of zone marker, and "file" directives.

			if ( /^(\s*)\}/ )
			{
				# If line starts with the end of zone marker
				$zonefoundflag = 0;
				next;
			}
			elsif ( /^(\s*)file(\s*)/ )
			{
				# If we find the zone file value here, catch it.

				@a1 = split;

				if ($a1[0] eq '')
				{
					$t1 = shift(@a1);
				}

				$t1 = $a1[1];
				$t2 = undblquote(0,$t1);

#				print("DIAGNOSTIC 03 - about to delete file [ $t2 ]\n");
				if (-f "/home/named/var/named/$t2")
				{
					Safe::backtick("/bin/rm -f /home/named/var/named/$t2");
				}

				next;
			}
			else
			{
				# some other line within zone.
				next;
			}

		}

		if (!($zonefoundflag))
		{
			if ( /^(\s*)zone(\s*)/ && /\{/ )
			{
				@a1 = split;

				if ($a1[0] eq '')
				{
					$t1 = shift(@a1);
				}

				$t1 = $a1[1];
				$t2 = undblquote(0,$t1);


#				print("DIAGNOSTIC 01 - found zone proper name [ $t2 ]\n");

				if ($t2 eq $del_zname)
				{
#					print("DIAGNOSTIC 02 - zone name matched. ignoring.\n");
					$zonefoundflag = 1;
				}
				else
				{
#					print("DIAGNOSTIC 02 - zone name did not match. keeping.\n");
					$catchall .= $_;
				}

				next;
			}

			else
			{
				$catchall .= $_;
			}
		}

	}	# END of while loop.


	open (BINDNEW, ">/home/named/var/named/named.conf") or return 220;

	# Dirty work.
	print BINDNEW $catchall;


	close (BINDNEW) or return 222;
	close (BINDCONFIG) or return 222;

	return 0;

}


# public function
sub zone_toggle
{
	my ($useless,$tog_zname,$tog_ztype,$tog_zmaster) = @_;
	# Expects to use /home/named/var/named/named.conf as a config file

	# Can toggle from master to slaves
	# Can toggle from slave to master
	# Cannot toggle from/to same state

	my $unusable;

	my @zonedata;

	my $isfound;
	my ($i,$j);
	my ($t1,$t2,$t3,$t4);
	my (@a1,@a2);

	my $catchall;
	my $zonefoundflag;
	my $templine;

	my $savefilename;
	my $tempfilename;

	if ( ($tog_zname eq '') || ($tog_ztype eq '') )
	{
		# please give arguments!
		return 201;
	}

	open (BINDCONFIG, "</home/named/var/named/named.conf") or return 220;
	flock (BINDCONFIG, 2) or return 221;

	$unusable = parseconfig(0,\*BINDCONFIG,\@zonedata);

	$catchall = '';


	if ($unusable)
	{
		return $unusable;
		# Pass back any error codes we get from parsing.
	}

	if ( ($tog_zname eq '.') || ($tog_zname eq 'bind') || ($tog_zname eq 'localhost') || ($tog_zname eq '0.0.127.in-addr.arpa') )
	{
		# Can't toggle a zone which is already reserved!
		return 205;
	}

	if ( ($tog_ztype < 1) || ($tog_ztype > 2) )
	{
		# Undefined values for toggle.
		return 209;
	}


	for ($i = 0;  $i < scalar(@zonedata); $i++)
	{
		# Iterate through all zones.  See if we have a match.
		if ($tog_zname eq $zonedata[$i][0])
		{
			# Match detected!
			$isfound = 1;

			# Now see if it's already in that state.

			if ($zonedata[$i][2] eq $tog_ztype)
			{
				# Same state!  Quit now.
				return 211;
			}

		}

	}


	if (!($isfound))
	{
		# Trying to toggle a non-existent zone
		close (BINDCONFIG) or return 222;
		return 208;
	}

	$zonefoundflag = 0;

	# Rewind to the beginning of the file.
	seek(BINDCONFIG,0,0);
	
	while(<BINDCONFIG>)
	{
		# ignore commented lines
		next if (/^#/);
		next if (/^\/\//);
		next if (/^$semicolon/);


		if ($zonefoundflag)
		{
			# Start catching zone data here.
			# Look specifically for end of zone marker, and "file" directives.

			if ( /^(\s*)\}/ )
			{
				# If line starts with the end of zone marker
				$zonefoundflag = 0;
				$catchall .= $_;
				next;
			}
			elsif ( /^(\s*)type(\s*)/ )
			{
				# If we find the zone file value here, catch it.

				$templine = $_;
				$t1 = $_;

				if ($tog_ztype eq 1)
				{
					# going from slave to master.  Hang on...
					
					$t1 =~ s/slave/master/;
				}
				if ($tog_ztype eq 2)
				{
					# going from master to slave.  Hang on...
					
					$t1 =~ s/master/slave/;
				}

				$catchall .= $t1;
				next;
			}
			elsif ( /^(\s*)file(\s*)/ )
			{
				# If we find the zone file value here, catch it.

#				print ("Diagnostic 04 RAW LINE is [ $_ ]\n");

				$templine = $_;
				$t1 = $_;

				@a1 = split;
	
				if ($a1[0] eq '')
				{
					$t2 = shift(@a1);
				}

				$t2 = $a1[1];

#				print ("Diagnostic 04 INTERMEDIATE step [ $t2 ]\n");

				if ($t2 =~ /;/)
				{
					$t3 = $`;
				}	# Filter out all text before semicolon

				$t2 = undblquote(0,$t3);

#				print ("Diagnostic 04 FILENAME to change is [ $t2 ]\n");

				$savefilename = $t2;

				if ($tog_ztype eq 1)
				{
					# going from slave to master.  Hang on...
					
					$t1 =~ s/"slave\//"master\//;
#					print ("Replaced line (S->M) is $t1\n");

					$t2 =~ s/^slave/master/;

#					print ("Diagnostic 04a FILENAME changed is [ $t2 ]\n");

				}
				if ($tog_ztype eq 2)
				{
					# going from master to slave.  Hang on...
					
					$t1 =~ s/"master\//"slave\//;
#					print ("Replaced line (M->S) is $t1\n");

					$t2 =~ s/^master/slave/;

#					print ("Diagnostic 04a FILENAME changed is [ $t2 ]\n");
				}

				# Now move the file over.

				if ( -f "/home/named/var/named/$savefilename" )
				{
					Safe::backtick("/bin/mv -f /home/named/var/named/$savefilename /home/named/var/named/$t2");
#					print ("Diagnostic 04b FILE successfully moved.\n");
				}


				$catchall .= $t1;
				next;
			}

			elsif ( /^(\s*)notify(\s*)/ && ($tog_ztype eq 2) )
			{
				# If we find the zone notify directive, change to a masters directive

				$templine = "\tmasters { " . $tog_zmaster . $semicolon . " }". $semicolon . $cr;

				$catchall .= $templine;
				next;
			}

			elsif ( /^(\s*)masters(\s*)/ && ($tog_ztype eq 1) )
			{
				# If we find the zone file value here, catch it.

				$templine = "\tnotify yes" . $semicolon . $cr;

				$catchall .= $templine;
				next;
			}

			else
			{
				# some other line within zone.
				$catchall .= $_;
				next;
			}

		}

		if (!($zonefoundflag))
		{
			if ( /^(\s*)zone(\s*)/ && /\{/ )
			{
				$templine = $_;
				@a1 = split;

				if ($a1[0] eq '')
				{
					$t1 = shift(@a1);
				}

				$t1 = $a1[1];
				$t2 = undblquote(0,$t1);


#				print("DIAGNOSTIC 01 - found zone proper name [ $t2 ]\n");

				if ($t2 eq $tog_zname)
				{
#					print("DIAGNOSTIC 02 - zone name matched. toggling.\n");
					$zonefoundflag = 1;
				}

				$catchall .= $templine;
				next;
			}

			else
			{
				$catchall .= $_;
			}
		}

	}	# END of while loop.



	open (BINDNEW, ">/home/named/var/named/named.conf") or return 220;

	# Dirty work.
	print BINDNEW $catchall;


	close (BINDNEW) or return 222;

	close (BINDCONFIG) or return 222;

	return 0;

}


sub named_update
{
	my ($useless,$unused)	= @_;
	my $DNSDaemon			= new Daemon('named');

	# If the process is running...
	if ($DNSDaemon->GetStatus())
	{
		# Process is running so force a reload.
		Safe::spawn_daemon("/usr/sbin/ndc reload");
	}
}





#############
# END of FILE
#############
1;





