package DNSZone;

###############################################################################
#                                                                             #
# Filename: DNSZone.pm                                                        #
#                                                                             #
# Description: Manages the individual DNS zone data files.                    #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

use vars qw(@ISA @EXPORT @EXPORT_OK);
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw( ); 

use lib '/usr/ins/intel/gui/perllib';

use strict;
use NetConfig;
use Daemon;
use Safe;

############

####################################################
# Return code key
# 200 - could not read file
# 201 - bad or missing or too few values in SOA
# 202 - bad characters (may not be used)
# 203 - MX record without priority value (MXPRI) - try again
# 204 - malformed MX record
# 205 - bad record
# 210 - unsupported record type (only A,CNAME,MX,NS,PTR and SOA)
# 211 - missing data fields for zone_addrecord
# 212 - missing data fields for zone_delrecord
# 213 - missing data fields for zone_replrecord (old)
# 214 - missing data fields for zone_replrecord (new)
# 215 - record already exists for zone_addrecord
# 216 - record does not exist for zone_delrecord
# 217 - record does not exist, can't be replaced for zone_replrecord
# 218 - file open error (not used yet)
# 219 - file lock error (not used yet)
# 220 - missing data for zone constructor
# 221 - file create error for constructor
# 222 - file lock error for constructor
# 223 - bad characters in zone name for constructor
# 224 - improper IP address format for constructor
# 225 - bad Subnet IP address for constructor (rev)
# 226 - bad nameserver address for constructor (rev)
# 227 - bad postmaster e-mail domain for constructor (not used yet)
# 228 - file close error
# 231 - no entries in this array
# 233 - cannot delete this zone file - it is reserved
# 234 - file does not exist (for zone file delete)
# 235 - cannot change ownership of zone data file (zone_promotemaster)
# 240 - Invalid array format (zone_delmultiple)
# 250 - record already exists of type MX, with same values except for priority.
####################################################



my $semicolon = '\;';

###################################
# private function
sub strip_traildot
{
	my ($useless,$texttostrip) = @_;

	my $newtext = '';

	if ($texttostrip =~ /(\.)$/ )
        {
                # If the text ends with a period, strip it off.
                $newtext = $`;
        }
	else
	{
		$newtext = $texttostrip;
	}
	
	return $newtext;
}


###################################
# private function
sub testfor_traildot
{
	my ($useless,$checkme) = @_;

	if ($checkme =~ /(\.)$/ )
        {
                # If the text ends with a period, say so!
                return 1;
        }
	else
	{
		return 0;
	}
	
}


###################################
# private function
sub parse_zonefile
{
	my ($useless,$zonename,$filehandle,  $pz_soa,$pz_ns,$pz_mx,$pz_a,$pz_cname,$pz_ptr,$pz_nssub) = @_;
	# Expects an open filehandle
	
	###############################################################################
	# @pz_soa - Array Format
	# [0] Name of the zone in question (usually dynamic or '@')
	# [1] Name server (immediately after SOA token)
	# [2] Administrator email (in user@host.net format)
	# [3] Serial number
	# [4] Refresh value
	# [5] Retry value
	# [6] Expire value
	# [7] Default TTL value
	###############################################################################


	###############################################################################
	# @pz_ns - Array Format
	# - array of lists
	# - [domain,value]
	# -- domain is the domain name (should always be '@')
	# -- value is the name of the name server
	# -- IN and NS tags implied, and must be regenerated on file output.
	#
	# @pz_mx - Array Format
	# - array of lists
	# - [domain,value,mxpri]
	# -- domain is the domain name (usually '@' but can be other)
	# -- value is the name of the mail server
	# -- mxpri is the mail server priority, a positive integer between 0 and 2^31
	# -- IN and MX tags implied, and must be regenerated on file output.
	#
	# @pz_a - Array Format
	# - array of lists
	# - [domain,value]
	# -- domain is the domain name (can be '@', FQDN, or host name)
	# -- value is the IP address (not checked)
	# -- IN and A tags implied, and must be regenerated on file output.
	#
	# @pz_cname - Array Format
	# - array of lists
	# - [domain,value]
	# -- domain is the alias name (can be '@', FQDN, or host name)
	# -- value is the true address to which this alias points
	# -- IN and CNAME tags implied, and must be regenerated on file output.
	#
	# @pz_ptr - Array Format
	# - array of lists
	# - [domain,value]
	# -- domain is the address name
	# -- value is the host name (not checked)
	# -- IN and PTR tags implied, and must be regenerated on file output.
	#
	# @pz_nssub - Array Format
	# - array of lists
	# - [domain,value]
	# -- domain is the domain name (should be a sub-zone, never '@')
	# -- value is the name of the name server
	# -- IN and NS tags implied, and must be regenerated on file output.
	#
	###############################################################################

	my $count_ns;
	my $count_mx;
	my $count_a;
	my $count_cname;
	my $count_ptr;
	my $count_nssub;
	my $unusable;

	# Counters for array indexes, into above data structures.

	$count_ns = 0;
	$count_mx = 0;
	$count_a= 0;
	$count_cname = 0;
	$count_ptr = 0;
	$count_nssub = 0;

	my $flag_soa;
	my $flag_soa_header;
	my $flag_soa_values;
	my $soa_in_paren;
	my $soa_collect;
	my $prev_name;
	my $thiscurline;
	my $curorigin;

	my $iszonereverse;
	if ($zonename =~ /.in-addr.arpa(\.)?$/) 
	{
		$iszonereverse = 1;
	}
	else
	{
		$iszonereverse = 0;
	}


	# Prepare a variable with the full zonename (including trailing dot)
	# to be used later, during record parsing
	my $fqzn = $zonename;
	
	if (!(testfor_traildot(0,$fqzn)))
	{
		$fqzn .= '.';
	}


	# Define a Hash (not made from corned beef)

	my %cur_rec = {	'domain' => '',
			'ttl'    => '',
			'class'  => '',
			'type'   => '',
			'mxpri'  => '',
			'data'   => ''
		   };

	# Little temporary values.

	my (@a1,@a2,@a3,$i,$j,$t1,$t2,$t3,$t4,$strsave);
	
	# Begin operation.

	$unusable = 0;
	$thiscurline = '';
	$curorigin = '@';
	$flag_soa = 1;
	$flag_soa_header = 1;	# Flag to indicate we are SEARCHING for these.
	#flag_soa_values = 0;	# SOA values all collected, ready to parse.
	$soa_in_paren = 0;
	$soa_collect = '';
	$prev_name = '@';	# Assume '@' (current domain) to be used for all records without a domain name
				# until we find a record with a domain name.  (Then that goes here.)

	while (<$filehandle>)
	{
		chomp($_);

		$thiscurline = $_;

		# ignore commented lines
		next if (/^(\w)*#/);
		next if (/^(\w)*$semicolon/);

		# ignore good things for bad reasons. FIX LATER.
		next if (/^(\w)*\$TTL/);
##		next if (/^(\w)*\$ORIGIN/);
		next if (/^(\w)*\$INCLUDE/);
		next if (/^(\w)*\$GENERATE/);

	
		# ignore lines with only white space
		next if !(/(.+)/);

		# strip out comments from the end of any line
		if (/;/)
		{
			$_ = $`;
		}

		if (/^(\w)*\$ORIGIN/)
		{
			# Begin processing current origin tag.

			my $t1 = $_;
			my @a1 = split(/\s+/, $_);
			my $t2 = '';
			
			$t2 = shift(@a1);
			
			if ('' eq $t2)
			{
				# Clear out starting whitespace
				$t2 = shift(@a1);
			}
			
			if ("\$ORIGIN" eq $t2)
			{
				# Found the $ORIGIN tag
				$t2 = shift(@a1);
			}
			
			# Now, $t2 contains the origin for all records after this point.

			my $t3;

			$t3 = ($t2 =~ /[-a-zA-Z0-9.]+/);
			if ( ($` eq '') && ($' eq '') && ($& ne '') )
			{
				if (testfor_traildot(0,$t2))
				{
					# Make this the new origin.
					$curorigin = $t2;
				}
				else
				{
					# Add this to the current origin tag.
					
					my $neworigin = $t2;
					if ($curorigin ne '@')
					{
						$neworigin .= '.' . $curorigin;
					}
					$curorigin = $neworigin;
				}
			}

			next;
		}


		# No ORIGIN tags here, so start expecting the SOA and RRsets.

		if ($flag_soa_header)
		{

			if (/\s+SOA\s+/)
			{
				# If we find the SOA header here, let's parse it.

				@a1 = split(/\s+/, $_);
				if ('' eq $a1[0])
				{
					shift(@a1);		# Get rid of null strings this way.
				}

				$t2 = shift(@a1);
				
				if ( ($t2 eq '@') || ($t2 eq $fqzn) )
				{
					$$pz_soa[0] = '@';
					$prev_name = '@';
				}

				elsif (($t2 . $curorigin) eq $fqzn)
				{
					$$pz_soa[0] = '@';
					$prev_name = $t2;
				}

				elsif (!($t2 =~ /\D/))
				{
					# Oops, all digits, must be a TTL value.
					# Put it back now.
					
					unshift(@a1, $t2);
					$$pz_soa[0] = $prev_name;
				}

				else
				{
					# Other name.  Add it.
					
					$prev_name = $t2;

					if ($curorigin eq '@')
					{
						$$pz_soa[0] = $t2;
					}
					else
					{
						$$pz_soa[0] = $t2 . $curorigin;
						
					}
				
				}



				# Add handler for SOA TTL digits.

				$t2 = shift(@a1);

				if (!($t2 =~ /\D/))
				{
					$t2 = shift(@a1);
				}

				if ($t2 =~ /^IN/)
				{
					$t2 = shift(@a1);
				}

				if ($t2 =~ /^SOA/)
				{
					$t2 = shift(@a1);
				}
				else
				{
					# Import unsuccessful.  SOA marker missing
					return 201;
					next;
				}


				# $t2 = shift(@a1);
				if ($t2 =~ /\S+/)
				{
					$$pz_soa[1] = $t2;
				}


				$t2 = shift(@a1);
				if ($t2 =~ /\S+/)
				{
					@a2 = split(/\./, $t2);
					$t3 = shift(@a2);
					$t3 .= '@';
					$t3 .= join('.', @a2);

					$$pz_soa[2] = $t2;
				}
				
				$_ = join(" ", @a1);

			}

			if (/\(/)
			{
				# If we find an open parenthesis mark, start collecting values.
				$soa_in_paren = 1;
				
				# Split on whitespace
				@a1 = split(/\s+/, $_);

				if ('' eq $a1[0])
				{
					shift(@a1);		# Get rid of null strings this way.
				}

				$_ = join(' ', @a1);

				if (/\(/)
				{
					$_ = $';
				}

				next if ('' eq $_);

			}

		}	# End SOA header parsing routine.


		# Begin checking for SOA values inside parens.

		if ($soa_in_paren)
		{
			@a1 = split(/\s+|\)/);		# Split on whitespace or paren close.

			if ('' eq $a1[0])
			{
				shift(@a1);		# Get rid of null strings this way.
			}

			$t2 = shift(@a1);

			while ($t2 =~ /\w+/)			# While we find true alphanumeric characters
			{
				$soa_collect .= $t2 . ' ';
				$t2 = shift(@a1);
			}


			if (/\)/)
			{
				# This is the last line, since it contains an end of paren.
				$soa_in_paren = 0;
				$flag_soa_header = 0;
				$flag_soa_values = 1;

				@a3 = split(/\s+/,$soa_collect);
				if ('' eq $a3[0])
				{
					shift(@a3);
				}

				if (5 > scalar(@a3))
				{
					return 201;
				}

				$$pz_soa[3] = shift(@a3);
				$$pz_soa[4] = shift(@a3);
				$$pz_soa[5] = shift(@a3);
				$$pz_soa[6] = shift(@a3);
				$$pz_soa[7] = shift(@a3);

			}

			next;		# Assume nothing else left to keep in this line.

		}



		# Begin checking for regular records.

		@a1 = split(/\s+/);		# Split on whitespace

		if ('' eq $a1[0])
		{
			shift(@a1);		# Get rid of null strings this way.
		}


		%cur_rec = {	'domain' => '',
				'ttl'    => '',
				'class'  => '',
				'type'   => '',
				'mxpri'  => '',
				'data'   => ''
			   };

		# Work backwards from end of array.
		# Stage 1 - get record data.

		$t2 = pop(@a1);

		if ($t2 eq '')
		{
			# String match on empty element.  Skip.
			next;
		}
		else
		{
			$t3 = ($t2 =~ /[-a-zA-Z0-9.]+/);
			if ( ($` eq '') && ($' eq '') && ($& ne '') )
			{
				$cur_rec{'data'} = $t2;
			}
			else
			{
				# String match found invalid characters.
				next;
			}
		}

		# Stage 2 - get record type.

		$t2 = pop(@a1);

		if ($t2 eq '')
		{
			# String match on empty element.  Record type missing. Skip.
			next;
		}
		elsif (($t2 eq 'A') || ($t2 eq 'CNAME') || ($t2 eq 'NS') || ($t2 eq 'PTR'))
		{
			$cur_rec{'type'} = $t2;
		}
		elsif ($t2 eq 'MX')
		{
			# MX record missing priority value.
			return 203;
			next;
		}
		else
		{	# Check for MX data here.

			if (!($t2 =~ /\D+/))
			{	# If we do not match any non-digit characters

				$t3 = pop(@a1);
				if ($t3 eq 'MX')
				{	# Make sure we get the 'MX' record type, as this is the only
					# record type which supports this extra value.

					$cur_rec{'mxpri'} = $t2;
					$cur_rec{'type'} = $t3;
				}
				else
				{	# Found number but no MX!
					# Malformed records without MX.
					return 204;
					next;
				}

			}
			else
			{
				# Record type did not match.
				return 210;
				next;
			}

		}

		# Stage 3 - get IN class if it exists
		
		$t2 = pop(@a1);

		if ($t2 eq '')
		{
			# String match on empty element.  Class marker missing.
			goto EVALCURRECORD;
		}
		elsif (($t2 eq 'IN') || ($t2 eq 'CHAOS'))
		{
			$cur_rec{'class'} = 'IN';
		}
		else
		{
			# Matched another sequence.  Assume this may be TTL or name.
			push(@a1, $t2);
		}
			
		# Stage 4 - check for TTL record if it exists.

		$j = scalar(@a1);

			# take action now based on array size.

		if (0 eq $j)
		{
			goto EVALCURRECORD;
		}
		elsif (1 eq $j)
		{
			$t2 = pop(@a1);

			if ($t2 eq '')
			{
				# String match on empty element.
				$cur_rec{'domain'} = '';
				goto EVALCURRECORD;
			}

			elsif ($t2 eq '@')
			{
				# String contains the @ (Default) marker.
				$cur_rec{'domain'} = $t2;
				goto EVALCURRECORD;
			}
			else
			{
				$t3 = ($t2 =~ /[-a-zA-Z0-9.]+/);
				if ( ($` eq '') && ($' eq '') && ($& ne '') )
				{
					$cur_rec{'domain'} = $t2;

					if (! ($t2 =~ /\D/) )
					{
						# If $t2 is digits and only digits,

						#if ($cur_rec{'type'} = 'PTR')
						if ($iszonereverse eq 1)
						{
							# If we're a reverse zone, assume this is the IP portion, not a TTL value.

							$cur_rec{'domain'} = $t2;
						}
						else
						{
							$cur_rec{'ttl'} = $t2;
							$cur_rec{'domain'} = '';
						}
					}

					goto EVALCURRECORD;
				}
				else
				{
					# String match found invalid characters.
					# return 202;
					next;
				}
			}

		}
		elsif (2 eq $j)
		{
			$t2 = pop(@a1);
			$t3 = pop(@a1);

			if ($t3 eq '')
			{
				if ($t2 eq '')
				{
					# If both strings are empty, assume prior values and go on.
					$cur_rec{'ttl'} = '';
					$cur_rec{'domain'} = '';
					goto EVALCURRECORD;
				}
				elsif (!($t2 =~ /\D+/))
				{
					# If there are no characters which are not digits
					$cur_rec{'ttl'} = $t2;
					$cur_rec{'domain'} = '';
					goto EVALCURRECORD;
				}
				elsif ($t2 eq '@')
				{
					# String contains the @ marker.
					$cur_rec{'domain'} = $t2;
					goto EVALCURRECORD;
				}
				else
				{

					$t4 = ($t2 =~ /[-a-zA-Z0-9.]+/);
					if ( ($` eq '') && ($' eq '') && ($& ne '') )
					{
						$cur_rec{'domain'} = $t2;
						goto EVALCURRECORD;
					}
					else
					{
						# String match found invalid characters.
						# return 202;
						next;
					}

				}

			}
			else 
			{
				# Either $t3 is equal to the @ token, or
				# it contains no characters which are underscores or non-alpha-numeric.

				if ($t2 eq '')
				{
					# If the second element is empty, something's wrong.
					return 205;
					next;
				}
				elsif (!($t2 =~ /\D+/))
				{
					# If there are no characters which are not digits
					$cur_rec{'ttl'} = $t2;
				}
				else
				{
					# Something is wrong.  $t2 should be digits only.
					return 205;
					next;
				}

				if ($t3 eq '@')
				{
					# Found default domain marker.
					$cur_rec{'domain'} = $t3;
					goto EVALCURRECORD;
				}
				else
				{
					$t4 = ($t3 =~ /[-a-zA-Z0-9.]+/);

					if ( ($` eq '') && ($' eq '') && ($& ne '') )
					{
						# Awfully fancy string matching there, mister.
						$cur_rec{'domain'} = $t3;
						goto EVALCURRECORD;
					}
					else
					{
						return 205;
						next;
					}

				}

			}

		}		# End case for testing 2 array elements
		else
		{
				# Either more than two, or less than zero elements.  Skip.
				next;
		}

		EVALCURRECORD:

		if ($cur_rec{'domain'} eq '')
		{
			$cur_rec{'domain'} = $prev_name;
		}
		else
		{
			$prev_name = $cur_rec{'domain'};
		}


		# Next, start thinking about $ORIGIN tags, and stuff like that.
		
		if ($curorigin ne '@')
		{
			# If not working from the default origin tag
			
			if (!(testfor_traildot(0,$cur_rec{'domain'})))
			{
				# If there is no dot at the end of the record name,
				# let's put one there, and append the current $ORIGIN tag.
				
				$cur_rec{'domain'} .= '.' . $curorigin;
			}

			if ($cur_rec{'domain'} eq '@')
			{
				$cur_rec{'domain'} = $curorigin;
			}

		}
		
		# For sanity and ease of reading, we can strip off all the appended origin
		# data which includes the FQZN at the tail.  This is true since the parsed
		# zone file will have no $ORIGIN tags, and the default origin is '@',
		# which acts just as $fqzn.
		
		if ( $cur_rec{'domain'} =~ /(\.)$fqzn/ )
		{
			$cur_rec{'domain'} = $`;
		}
		

		# Now, place values into the data structures.

		$t2 = $cur_rec{'type'};

		if ($t2 eq "A")
		{
			if ($cur_rec{'domain'} eq '@')
			{
				$$pz_a[$count_a][0] = $fqzn;
			}
			else
			{
				$$pz_a[$count_a][0] = $cur_rec{'domain'};
			}

			$$pz_a[$count_a][1] = $cur_rec{'data'};
			$count_a++;
		}
		elsif ($t2 eq "CNAME")
		{
			if ($cur_rec{'domain'} eq '@')
			{
				$$pz_cname[$count_cname][0] = $fqzn;
			}
			else
			{
				$$pz_cname[$count_cname][0] = $cur_rec{'domain'}
			}

			$$pz_cname[$count_cname][1] = $cur_rec{'data'};
			$count_cname++;
		}
		elsif ($t2 eq "MX")
		{
			if ($fqzn eq $cur_rec{'domain'})
			{
				$cur_rec{'domain'} = '@';
			}
			
			$$pz_mx[$count_mx][0] = $cur_rec{'domain'};
			$$pz_mx[$count_mx][1] = $cur_rec{'data'};
			$$pz_mx[$count_mx][2] = $cur_rec{'mxpri'};
			$count_mx++;
		}
		elsif ($t2 eq "NS")
		{
			if ( ('@' eq $cur_rec{'domain'}) || ($zonename eq $cur_rec{'domain'}) || ($fqzn eq $cur_rec{'domain'}) )
			{
				$$pz_ns[$count_ns][0] = '@';
				$$pz_ns[$count_ns][1] = $cur_rec{'data'};
				$count_ns++;
			}
			else
			{
				$$pz_nssub[$count_nssub][0] = $cur_rec{'domain'};
				$$pz_nssub[$count_nssub][1] = $cur_rec{'data'};
				$count_nssub++;
			}

		}
		elsif ($t2 eq "PTR")
		{
			$$pz_ptr[$count_ptr][0] = $cur_rec{'domain'};
			$$pz_ptr[$count_ptr][1] = $cur_rec{'data'};
			$count_ptr++;
		}
		else
		{
			# Unsupported record type for this implementation.
			# ignore.
			$unusable = 0;
		}		


	}		# END of WHILE loop.
	
	return 0;

}



###################################
# private function
sub zone_report
{
	my ($useless,$zonename, $pz_soa,$pz_ns,$pz_mx,$pz_a,$pz_cname,$pz_ptr,$pz_nssub) = @_;
	my ($t1,$t2,$i);
	
	print ("Final Report:\n");

	print ("--SOA information\n");
	print ("---- [" .  join(',',@$pz_soa)  . "]\n");

	print ("--NS information\n");
	for ($i = 0; $i < scalar(@$pz_ns); $i++)
	{
		print ("----RR: $$pz_ns[$i][0]\tIN\tNS\t$$pz_ns[$i][1]\n");
	}
	print ("--MX information\n");
	for ($i = 0; $i < scalar(@$pz_mx); $i++)
	{
		print ("----RR: $$pz_mx[$i][0]\tIN\tA\t$$pz_mx[$i][2] $$pz_mx[$i][1]\n");
	}
	print ("--A information\n");
	for ($i = 0; $i < scalar(@$pz_a); $i++)
	{
		print ("----RR: $$pz_a[$i][0]\tIN\tA\t$$pz_a[$i][1]\n");
	}
	print ("--CNAME information\n");
	for ($i = 0; $i < scalar(@$pz_cname); $i++)
	{
		print ("----RR: $$pz_cname[$i][0]\tIN\tCNAME\t$$pz_cname[$i][1]\n");
	}
	print ("--PTR information\n");
	for ($i = 0; $i < scalar(@$pz_ptr); $i++)
	{
		print ("----RR: $$pz_ptr[$i][0]\tIN\tPTR\t$$pz_ptr[$i][1]\n");
	}
	print ("--NS Subzone information\n");
	for ($i = 0; $i < scalar(@$pz_nssub); $i++)
	{
		print ("----RR: $$pz_nssub[$i][0]\tIN\tNS\t$$pz_nssub[$i][1]\n");
	}

	print ("--Zone Summary:\n");
	print ("----" . scalar(@$pz_ns) . " NS records.\n");
	print ("----" . scalar(@$pz_mx) . " MX records.\n");
	print ("----" . scalar(@$pz_a) . " A records.\n");
	print ("----" . scalar(@$pz_cname) . " CNAME records.\n");
	print ("----" . scalar(@$pz_ptr) . " PTR records.\n");
	print ("----" . scalar(@$pz_nssub) . " NS Subzone records.\n");

	return 0;
}		


###################################
# private function
sub zone_output
{
	my ($useless,$zonename,$zonefilehandle,$pz_soa,$pz_ns,$pz_mx,$pz_a,$pz_cname,$pz_ptr,$pz_nssub) = @_;
	my ($t1,$t2,$i);
	
		print $zonefilehandle ("\; Zone file for $zonename\n");
		print $zonefilehandle ("\; Generated on " . scalar(localtime) . "\n");
		print $zonefilehandle ("\; Data may be overwritten at any time!  Do not modify this file.\n");

		print $zonefilehandle ("\; \n");
		print $zonefilehandle ("\; Begin SOA Record \n");
		print $zonefilehandle ("\; \n");
		print $zonefilehandle ("\$TTL $$pz_soa[7] \n");


		print $zonefilehandle ("@ IN SOA $$pz_soa[1] $$pz_soa[2] \( \n");
		print $zonefilehandle ("\t\t\t\t$$pz_soa[3] \; serial \n");
		print $zonefilehandle ("\t\t\t\t$$pz_soa[4] \; refresh \n");
		print $zonefilehandle ("\t\t\t\t$$pz_soa[5] \; retry \n");
		print $zonefilehandle ("\t\t\t\t$$pz_soa[6] \; expire \n");
		print $zonefilehandle ("\t\t\t\t$$pz_soa[7] \) \; default_ttl \n");

		print $zonefilehandle ("\n");
		print $zonefilehandle ("\; Begin NS (nameserver) Records \n");
		print $zonefilehandle ("\; \n");


	for ($i = 0; $i < scalar(@$pz_ns); $i++)
	{
		if ( ($$pz_ns[$i][0] ne '') && ($$pz_ns[$i][1] ne '') )
		{
			print $zonefilehandle (sprintf("%-24s",$$pz_ns[$i][0]) . "\tIN\tNS\t$$pz_ns[$i][1]\n");
		}
	}

		print $zonefilehandle ("\n");
		print $zonefilehandle ("\; Begin MX (mail exchange) Records \n");
		print $zonefilehandle ("\; \n");

	for ($i = 0; $i < scalar(@$pz_mx); $i++)
	{
		if ( ($$pz_mx[$i][0] ne '') && ($$pz_mx[$i][1] ne '') && ($$pz_mx[$i][2] ne '' ) )
		{
			print $zonefilehandle (sprintf("%-24s",$$pz_mx[$i][0]) . "\tIN\tMX\t$$pz_mx[$i][2] $$pz_mx[$i][1]\n");
		}
	}

		print $zonefilehandle ("\n");
		print $zonefilehandle ("\; Begin A (host address) Records \n");
		print $zonefilehandle ("\; \n");


	for ($i = 0; $i < scalar(@$pz_a); $i++)
	{
		if ( ($$pz_a[$i][0] ne '') && ($$pz_a[$i][1] ne '') )
		{
			print $zonefilehandle (sprintf("%-24s",$$pz_a[$i][0]) . "\tIN\tA\t$$pz_a[$i][1]\n");
		}
	}

		print $zonefilehandle ("\n");
		print $zonefilehandle ("\; Begin CNAME (host alias) Records \n");
		print $zonefilehandle ("\; \n");

	for ($i = 0; $i < scalar(@$pz_cname); $i++)
	{
		if ( ($$pz_cname[$i][0] ne '') && ($$pz_cname[$i][1] ne '') )
		{
			print $zonefilehandle (sprintf("%-24s",$$pz_cname[$i][0]) . "\tIN\tCNAME\t$$pz_cname[$i][1]\n");
		}
	}

		print $zonefilehandle ("\n");
		print $zonefilehandle ("\; Begin PTR (reverse address) Records \n");
		print $zonefilehandle ("\; \n");

	for ($i = 0; $i < scalar(@$pz_ptr); $i++)
	{
		if ( ($$pz_ptr[$i][0] ne '') && ($$pz_ptr[$i][1] ne '') )
		{
			print $zonefilehandle (sprintf("%-24s",$$pz_ptr[$i][0]) . "\tIN\tPTR\t$$pz_ptr[$i][1]\n");
		}
	}

		print $zonefilehandle ("\n");
		print $zonefilehandle ("\; Begin NS (name server) Records for sub-zones\n");
		print $zonefilehandle ("\; \n");

	for ($i = 0; $i < scalar(@$pz_nssub); $i++)
	{
		if ( ($$pz_nssub[$i][0] ne '') && ($$pz_nssub[$i][1] ne '') )
		{
			print $zonefilehandle (sprintf("%-24s",$$pz_nssub[$i][0]) . "\tIN\tNS\t$$pz_nssub[$i][1]\n");
		}
	}

		print $zonefilehandle ("\n");
		print $zonefilehandle ("\; End of zone file.\n");


	return 0;
}		


###################################
# public function
sub zone_getinfo
{
	my ($useless,$zonename,$zonefilename,$zone_soa,$zone_ns,$zone_mx,$zone_a,$zone_cname,$zone_ptr,$zone_nssub) = @_;
	my @results;

	my $i;
	my $unused;
	my $unusable;

	open (ZFILE1, "</home/named/var/named/$zonefilename") or return 218;
	flock (ZFILE1, 2) or return 219;

	$unusable = parse_zonefile(0,$zonename,\*ZFILE1,$zone_soa,$zone_ns,$zone_mx,$zone_a,$zone_cname,$zone_ptr,$zone_nssub);

	close (ZFILE1) or return 228;

	if ($unusable ne 0)
	{
		# parse_zonefile returned error code $unusable
		return $unusable;
	}

#	zone_report(0,$zonename,\@zone_soa,\@zone_ns,\@zone_mx,\@zone_a,\@zone_cname,\@zone_ptr,\@zone_nssub);	
	return 0;
}


###################################
# public function
sub zone_promotemaster
{
	my ($useless,$zonename,$zonefilename) = @_;
	my @results;

	my $i;
	my $t1;
	my @a1;
	my $found;
	my $unused;
	my $unusable;

	my (@zone_soa,@zone_ns,@zone_mx,@zone_a,@zone_cname,@zone_ptr,@zone_nssub);

	my $fqzn = $zonename;
	
	if (!(testfor_traildot(0,$fqzn)))
	{
		$fqzn .= '.';
	}


	if ( ($zonename eq '') || ($zonefilename eq '') )
	{
		# Missing data fields.  Please try again.
		return 211;
	}

	Safe::backtick("/bin/chown root.root /home/named/var/named/$zonefilename");
	
	open (ZFILE1, "</home/named/var/named/$zonefilename") or return 218;
	flock (ZFILE1, 2) or return 219;

	$unusable = parse_zonefile(0,$zonename,\*ZFILE1,\@zone_soa,\@zone_ns,\@zone_mx,\@zone_a,\@zone_cname,\@zone_ptr,\@zone_nssub);

	############
	# Add records here.
	############

	$found = 0;

	my $iii;
	for ($iii = 4; $iii < 8; $iii++)
	{
		# Begin reducing the Zone SOA values to common denominators

		my $strin = $zone_soa[$iii];
		my $num;
		my $time_d;
		
		my $secs=0;
		my $mins=0;
		my $hours=0;
		my $days=0;
		my $weeks=0;
		my $tmp1;

		if($strin =~ /[SsMmHhDdWw]/)
		{ 
			($secs, $tmp1) = ($strin  =~ m/(\d+)[Ss]/); 
			($mins, $tmp1) = ($strin  =~ m/(\d+)[Mm]/); 
			($hours, $tmp1) = ($strin =~ m/(\d+)[Hh]/); 
			($days, $tmp1) = ($strin  =~ m/(\d+)[Dd]/); 	
			($weeks, $tmp1) = ($strin =~ m/(\d+)[Ww]/); 		

			if($secs)
			{
				$secs += (($mins * 60) + ($hours * 3600) + ($days * 86400) + ($weeks * 604800)); #convert higher to secs
				$num = $secs;
				$time_d = "S";
			}
			elsif($mins)
			{ 
				$mins += (($hours * 60) + ($days * 1440) + ($weeks * 10080));  #convert higher to mins
				$num = $mins;
				$time_d = "M";
			}
			elsif($hours)
			{ 
				$hours += (($days * 24) + ($weeks * 168)) ; #convert higher to hours
				$num = $hours;
				$time_d = "H";
			}
			elsif($days)
			{ 
				$days += ($weeks * 7) ;
				$num = $days;
				$time_d = "D";
			}
			elsif($weeks)
			{ 
				$num = $weeks;
				$time_d = "W";
			}

		}
		else
		{
			$num = $strin;
			$time_d = "S";
		}

		# Now, try to see what can be expanded down (up?) to a simpler unit.
		
		if ($time_d eq "S")
		{
			if (($num % 60) eq 0)
			{
				# It shrinks!
				my $newnum = $num / 60;
				$num = $newnum;
				$time_d = "M";
			}
		}
		if ($time_d eq "M")
		{
			if (($num % 60) eq 0)
			{
				# It shrinks!
				my $newnum = $num / 60;
				$num = $newnum;
				$time_d = "H";
			}
		}
		if ($time_d eq "H")
		{
			if (($num % 24) eq 0)
			{
				# It shrinks!
				my $newnum = $num / 24;
				$num = $newnum;
				$time_d = "D";
			}
		}
		if ($time_d eq "D")
		{
			if (($num % 7) eq 0)
			{
				# It shrinks!
				my $newnum = $num / 7;
				$num = $newnum;
				$time_d = "W";
			}
		}

		# Now use this value.

		$zone_soa[$iii] = $num . $time_d;

	}
	

	# Now that we checked the input, write the file.

	open (ZFILE2, ">/home/named/var/named/$zonefilename") or return 218;

	$unusable = zone_output(0,$zonename,\*ZFILE2,\@zone_soa,\@zone_ns,\@zone_mx,\@zone_a,\@zone_cname,\@zone_ptr,\@zone_nssub);

	close (ZFILE2) or return 228;
	close (ZFILE1) or return 228;

	return 0;
}


###################################
# public function
sub zone_addrecord
{
	my ($useless,$zonename,$zonefilename,$rec_type,$rec_domain,$rec_data,$rec_mxpri) = @_;
	my @results;

	my $i;
	my $t1;
	my @a1;
	my $found;
	my $similarpri;
	my $unused;
	my $unusable;

	my (@zone_soa,@zone_ns,@zone_mx,@zone_a,@zone_cname,@zone_ptr,@zone_nssub);

	if ( ($rec_type eq '') || ($rec_domain eq '') || ($rec_data eq '') )
	{
		# Missing data fields.  Please try again.
		return 211;
	}

	open (ZFILE1, "</home/named/var/named/$zonefilename") or return 218;
	flock (ZFILE1, 2) or return 219;

	$unusable = parse_zonefile(0,$zonename,\*ZFILE1,\@zone_soa,\@zone_ns,\@zone_mx,\@zone_a,\@zone_cname,\@zone_ptr,\@zone_nssub);

	############
	# Add records here.
	############

	$found = 0;
	$similarpri = 0;

	if ($rec_type eq 'A')
	{
		for ($i = 0; $i < scalar(@zone_a) ; $i ++)
		{
			if ( ($rec_domain eq $zone_a[$i][0]) && ($rec_data eq $zone_a[$i][1]) )
			{
				$found = 1;
			}
		}

		if (!($found))
		{
			$i = scalar(@zone_a);

			$zone_a[$i][0] = $rec_domain;
			$zone_a[$i][1] = $rec_data;
		}
	}
	elsif ($rec_type eq 'CNAME')
	{
		for ($i = 0; $i < scalar(@zone_cname) ; $i ++)
		{
			if ( ($rec_domain eq $zone_cname[$i][0]) && ($rec_data eq $zone_cname[$i][1]) )
			{
				$found = 1;
			}
		}

		if (!($found))
		{
			$i = scalar(@zone_cname);

			$zone_cname[$i][0] = $rec_domain;
			$zone_cname[$i][1] = $rec_data;
		}
	}
	elsif ($rec_type eq 'MX')
	{
		if ($rec_mxpri eq '')
		{
			# Missing MX priority field.  Please give an integer as an argument.
			return 211;
		}

		for ($i = 0; $i < scalar(@zone_mx) ; $i ++)
		{
			if ( ($rec_domain eq $zone_mx[$i][0]) && ($rec_data eq $zone_mx[$i][1]) && ($rec_mxpri eq $zone_mx[$i][2]) )
			{
				$found = 1;
			}
			
			elsif ( ($rec_domain eq $zone_mx[$i][0]) && ($rec_data eq $zone_mx[$i][1]) && ($rec_mxpri ne $zone_mx[$i][2]) )
			{
				$similarpri = 1;
			}
		}

		if ( (!($found)) && (!($similarpri)) )
		{
			$i = scalar(@zone_mx);

			$zone_mx[$i][0] = $rec_domain;
			$zone_mx[$i][1] = $rec_data;
			$zone_mx[$i][2] = $rec_mxpri;
		}
	}
	elsif ($rec_type eq 'NS')
	{
		if ( ($rec_domain eq '@') || ($rec_domain eq $zonename) )
		{
			for ($i = 0; $i < scalar(@zone_ns) ; $i ++)
			{
				if ( ($rec_domain eq $zone_ns[$i][0]) && ($rec_data eq $zone_ns[$i][1]) )
				{
					$found = 1;
				}
			}

			if (!($found))
			{
				$i = scalar(@zone_ns);

				$zone_ns[$i][0] = '@';
				$zone_ns[$i][1] = $rec_data;
			}
		}
		else
		{
			for ($i = 0; $i < scalar(@zone_nssub) ; $i ++)
			{
				if ( ($rec_domain eq $zone_nssub[$i][0]) && ($rec_data eq $zone_nssub[$i][1]) )
				{
					$found = 1;
				}
			}

			if (!($found))
			{
				$i = scalar(@zone_nssub);

				$zone_nssub[$i][0] = $rec_domain;
				$zone_nssub[$i][1] = $rec_data;
			}

		}

	}
	elsif ($rec_type eq 'PTR')
	{
		for ($i = 0; $i < scalar(@zone_ptr) ; $i ++)
		{
			if ( ($rec_domain eq $zone_ptr[$i][0]) && ($rec_data eq $zone_ptr[$i][1]) )
			{
				$found = 1;
			}
		}

		if (!($found))
		{
			$i = scalar(@zone_ptr);

			$zone_ptr[$i][0] = $rec_domain;
			$zone_ptr[$i][1] = $rec_data;
		}
	}
	else
	{
		# Unsupported record type
		return 210;
	}

	if ($found)
	{
		# Sorry, record already exists as given.
		return 215;
	}
	if ($similarpri)
	{
		# Sorry, a MX record exists which is identical except for its priority value.
		# Don't try to add this twice, ya hear?
		return 250;
	}

	# Now that we checked the input, write the file.

	open (ZFILE2, ">/home/named/var/named/$zonefilename") or return 218;

	$unusable = zone_output(0,$zonename,\*ZFILE2,\@zone_soa,\@zone_ns,\@zone_mx,\@zone_a,\@zone_cname,\@zone_ptr,\@zone_nssub);

	close (ZFILE2) or return 228;
	close (ZFILE1) or return 228;


#	zone_report(0,$zonename,\@zone_soa,\@zone_ns,\@zone_mx,\@zone_a,\@zone_cname,\@zone_ptr,\@zone_nssub);	
	return 0;
}
	

###################################

# public function
sub zone_replrecord
{
	my ($useless,$zonename,$zonefilename,$rec_type,$rec_domain,$rec_data,$rec_d1,$rec_d2,$rec_d3,$rec_d4) = @_;
	my @results;

	# Calling parameters:
	# if $rec_type is 'MX'
	# -- $rec_d1 is old mxpri
	# -- $rec_d2 is new domain
	# -- $rec_d3 is new data
	# -- $rec_d4 is new mxpri
	# else
	# -- $rec_d1 is new domain
	# -- $rec_d2 is new domain

	my $i;
	my $t1;
	my @a1;
	my $found;
	my $unused;
	my $unusable;

	my (@zone_soa,@zone_ns,@zone_mx,@zone_a,@zone_cname,@zone_ptr,@zone_nssub);

	if ( ($rec_type eq '') || ($rec_domain eq '') || ($rec_data eq '') )
	{
		# Missing input data for search criteria.
		return 213;
	}

	open (ZFILE1, "</home/named/var/named/$zonefilename") or return 218;
	flock (ZFILE1, 2) or return 219;

	$unusable = parse_zonefile(0,$zonename,\*ZFILE1,\@zone_soa,\@zone_ns,\@zone_mx,\@zone_a,\@zone_cname,\@zone_ptr,\@zone_nssub);

	if ($unusable)
	{
		return $unusable;
	}

	$found = 0;

	if ($rec_type eq 'A')
	{
		if ( ($rec_d1 eq '') || ($rec_d2 eq '') )
		{
			# Missing data in argument input.
			return 214;
		}

		for ($i = 0; $i < scalar(@zone_a) ; $i ++)
		{
			if ( ($rec_domain eq $zone_a[$i][0]) && ($rec_data eq $zone_a[$i][1]) )
			{
				$found = 1;
				$zone_a[$i][0] = $rec_d1;
				$zone_a[$i][1] = $rec_d2;
			}
		}

	}
	elsif ($rec_type eq 'CNAME')
	{
		if ( ($rec_d1 eq '') || ($rec_d2 eq '') )
		{
			# Missing data in argument input.
			return 214;
		}

		for ($i = 0; $i < scalar(@zone_cname) ; $i ++)
		{
			if ( ($rec_domain eq $zone_cname[$i][0]) && ($rec_data eq $zone_cname[$i][1]) )
			{
				$found = 1;
				$zone_cname[$i][0] = $rec_d1;
				$zone_cname[$i][1] = $rec_d2;
			}
		}

	}
	elsif ($rec_type eq 'MX')
	{
		if ($rec_d1 eq '')
		{
			# Missing search criteria.
			return 213;
		}

		if ( ($rec_d2 eq '') || ($rec_d3 eq '') || ($rec_d4 eq '') )
		{
			# Missing data in argument input.
			return 214;
		}


		for ($i = 0; $i < scalar(@zone_mx) ; $i ++)
		{
			if ( ($rec_domain eq $zone_mx[$i][0]) && ($rec_data eq $zone_mx[$i][1]) && ($rec_d1 eq $zone_mx[$i][2]) )
			{
				$found = 1;
				$zone_mx[$i][0] = $rec_d2;
				$zone_mx[$i][1] = $rec_d3;
				$zone_mx[$i][2] = $rec_d4;
			}
		}

	}
	elsif ($rec_type eq 'NS')
	{
		if ( ($rec_d1 eq '') || ($rec_d2 eq '') )
		{
			# Missing data in argument input.
			return 214;
		}

		if ( ($rec_domain eq '@') || ($rec_domain eq $zonename) )
		{
			for ($i = 0; $i < scalar(@zone_ns) ; $i ++)
			{
				if ( ($rec_domain eq $zone_ns[$i][0]) && ($rec_data eq $zone_ns[$i][1]) )
				{
					$found = 1;
					$zone_ns[$i][0] = '@';
					$zone_ns[$i][1] = $rec_d2;
				}
			}

		}
		else
		{
			for ($i = 0; $i < scalar(@zone_nssub) ; $i ++)
			{
				if ( ($rec_domain eq $zone_nssub[$i][0]) && ($rec_data eq $zone_nssub[$i][1]) )
				{
					$found = 1;
					$zone_nssub[$i][0] = $rec_d1;
					$zone_nssub[$i][1] = $rec_d2;
				}
			}

		}

	}
	elsif ($rec_type eq 'PTR')
	{
		if ( ($rec_d1 eq '') || ($rec_d2 eq '') )
		{
			# Missing data in argument input.
			return 214;
		}

		for ($i = 0; $i < scalar(@zone_ptr) ; $i ++)
		{
			if ( ($rec_domain eq $zone_ptr[$i][0]) && ($rec_data eq $zone_ptr[$i][1]) )
			{
				$found = 1;
				$zone_ptr[$i][0] = $rec_d1;
				$zone_ptr[$i][1] = $rec_d2;
			}
		}

	}
	else
	{
		# Unsupported record type.
		return 210;
	}

	if (!($found))
	{
		# Sorry, no records match search criteria.
		return 217;
	}


	open (ZFILE2, ">/home/named/var/named/$zonefilename") or return 218;

	$unusable = zone_output(0,$zonename,\*ZFILE2,\@zone_soa,\@zone_ns,\@zone_mx,\@zone_a,\@zone_cname,\@zone_ptr,\@zone_nssub);

	close (ZFILE2) or return 228;
	close (ZFILE1) or return 228;


#	zone_report(0,$zonename,\@zone_soa,\@zone_ns,\@zone_mx,\@zone_a,\@zone_cname,\@zone_ptr,\@zone_nssub);	
	return $unusable;
}
	


###################################
# public function
sub zone_delrecord
{
	my ($useless,$zonename,$zonefilename,$rec_type,$rec_domain,$rec_data) = @_;
	my @results;

	my $i;
	my $t1;
	my @a1;
	my $found;
	my $unused;
	my $unusable;

	my (@zone_soa,@zone_ns,@zone_mx,@zone_a,@zone_cname,@zone_ptr,@zone_nssub);

	if ( ($rec_type eq '') || ($rec_domain eq '') || ($rec_data eq '') )
	{
		# Missing data fields.  Please try again.
		return 212;
	}

	open (ZFILE1, "</home/named/var/named/$zonefilename") or return 218;
	flock (ZFILE1, 2) or return 219;

	$unusable = parse_zonefile(0,$zonename,\*ZFILE1,\@zone_soa,\@zone_ns,\@zone_mx,\@zone_a,\@zone_cname,\@zone_ptr,\@zone_nssub);

	$found = 0;


	if ($rec_type eq 'A')
	{
		for ($i = 0; $i < scalar(@zone_a) ; $i ++)
		{
			if ( ($rec_domain eq $zone_a[$i][0]) && ($rec_data eq $zone_a[$i][1]) )
			{
				$found = 1;
				$zone_a[$i][0] = '';
				$zone_a[$i][1] = '';
			}
		}

	}
	elsif ($rec_type eq 'CNAME')
	{
		for ($i = 0; $i < scalar(@zone_cname) ; $i ++)
		{
			if ( ($rec_domain eq $zone_cname[$i][0]) && ($rec_data eq $zone_cname[$i][1]) )
			{
				$found = 1;
				$zone_cname[$i][0] = '';
				$zone_cname[$i][1] = '';
			}
		}

	}
	elsif ($rec_type eq 'MX')
	{
		for ($i = 0; $i < scalar(@zone_mx) ; $i ++)
		{
			if ( ($rec_domain eq $zone_mx[$i][0]) && ($rec_data eq $zone_mx[$i][1]) )
			{
				$found = 1;
				$zone_mx[$i][0] = '';
				$zone_mx[$i][1] = '';
				$zone_mx[$i][2] = '';
			}
		}

	}
	elsif ($rec_type eq 'NS')
	{
		if ( ($rec_domain eq '@') || ($rec_domain eq $zonename) )
		{
			for ($i = 0; $i < scalar(@zone_ns) ; $i ++)
			{
				if ( ($rec_domain eq $zone_ns[$i][0]) && ($rec_data eq $zone_ns[$i][1]) )
				{
					$found = 1;
					$zone_ns[$i][0] = '';
					$zone_ns[$i][1] = '';
				}
			}

		}
		else
		{
			for ($i = 0; $i < scalar(@zone_nssub) ; $i ++)
			{
				if ( ($rec_domain eq $zone_nssub[$i][0]) && ($rec_data eq $zone_nssub[$i][1]) )
				{
					$found = 1;
					$zone_nssub[$i][0] = '';
					$zone_nssub[$i][1] = '';
				}
			}

		}

	}
	elsif ($rec_type eq 'PTR')
	{
		for ($i = 0; $i < scalar(@zone_ptr) ; $i ++)
		{
			if ( ($rec_domain eq $zone_ptr[$i][0]) && ($rec_data eq $zone_ptr[$i][1]) )
			{
				$found = 1;
				$zone_ptr[$i][0] = '';
				$zone_ptr[$i][1] = '';
			}
		}

	}
	else
	{
		# Unsupported record type.
		return 210;
	}

	if ($found eq 0)
	{
		# Sorry, record does not exist as given.
		return 216;
	}

	open (ZFILE2, ">/home/named/var/named/$zonefilename") or return 218;

	$unusable = zone_output(0,$zonename,\*ZFILE2,\@zone_soa,\@zone_ns,\@zone_mx,\@zone_a,\@zone_cname,\@zone_ptr,\@zone_nssub);

	close (ZFILE2) or return 228;
	close (ZFILE1) or return 228;

#	zone_report(0,$zonename,\@zone_soa,\@zone_ns,\@zone_mx,\@zone_a,\@zone_cname,\@zone_ptr,\@zone_nssub);	
	return 0;
}
	


###################################
sub zone_construct_fwd
{
	my ($useless, $zonename, $zonefilename, $virthostIP, $virthostname) = @_;

	my @pz_soa;
	my @pz_ns;
	my @pz_mx;
	my @pz_a;
	my @pz_cname;
	my @pz_ptr;
	my @pz_nssub;

	my $count_ns;
	my $count_mx;
	my $count_a;
	my $count_cname;
	my $count_ptr;
	my $count_nssub;

	my $i;
	my ($t1,$t2);
	my (@a1,@a2);
	my $unusable;

	my $fqdn;
	my $fulldomain;
	my $mainhost;



	if ( ($zonename eq '') || ($zonefilename eq '') || ($virthostIP eq '') )
	{
		return 220;
	}


	$i = ($zonename =~ /[-a-zA-Z0-9.]+/);
	if (!( ($` eq '') && ($' eq '') && ($& ne '') ))
	{
		# not okay, zone name has bad characters.
		return 223;
	}


#	$i = ($zonename =~ /[-a-zA-Z0-9.]+/);
#	if (!( ($` eq '') && ($' eq '') ))
#	{
#		# not okay, zone name has bad characters.
#		return 223;
#	}


	$i = ($virthostIP =~ m/(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})/ );
	if ($i eq '')
	{
		# not okay, host IP address is not proper.
		return 224;
	}

#	print ("Given IP address is $virthostIP\n");

	@a1 = split('\.',$zonename);

	while(scalar(@a1) && defined($a1[0]) )
	{
		$t1 = shift(@a1);
		if ( ($t1 ne '') && defined($t1) )
		{
			push(@a2, $t1);
		}
	}

	$fulldomain = join ('.', @a2) . '.';

#	print ("Filtered zonename is [ $fulldomain ]\n");
	
	@a2 = ();

	$mainhost = '';

	if ($virthostname ne '')
	{
		@a1 = split('\.',$virthostname);

		while(scalar(@a1) && defined($a1[0]) )
		{
			$t1 = shift(@a1);
			if ( ($t1 ne '') && defined($t1) )
			{
				push(@a2, $t1);
			}
		}

		$mainhost = (join ('.', @a2 ) );

#		print ("Filtered virthostname is [ $mainhost ]\n");
	}

#	print ("New fulldomain = [ $fulldomain ]\n");
#	print ("New mainhost = [ $mainhost ]\n");

	if ($mainhost eq '')
	{
		$fqdn = $fulldomain;
	}
	else
	{
		$fqdn = $mainhost . '.' . $fulldomain;
	}

#	print ("New FQDN = [ $fqdn ]\n");

	## Begin assembling data structures.

	$count_ns = 0;
	$count_mx = 0;
	$count_a = 0;
	$count_cname = 0;
	$count_ptr = 0;
	$count_nssub = 0;

	$pz_soa[0] = '@';
	$pz_soa[1] = 'dns.' . $fulldomain;
	$pz_soa[2] = 'postmaster.' . $fulldomain;

	$pz_soa[3] = '1001';
	$pz_soa[4] = '3H';
	$pz_soa[5] = '1H';
	$pz_soa[6] = '1W';
	$pz_soa[7] = '12H';

	$pz_ns[0][0] = '@';
	$pz_ns[0][1] = 'dns.' . $fulldomain;
	$count_ns++;

	$pz_mx[0][0] = '@';
	$pz_mx[0][1] = 'mail.' . $fulldomain;
	$pz_mx[0][2] = '10';
	$count_mx++;

	$pz_a[0][0] = $fulldomain;
	$pz_a[0][1] = $virthostIP;
	$count_a++;

	my @currentnic = NetConfig->listNetInterfaces();
	my $cur_ETH;
	my $cur_IP;
			
	$cur_ETH = $currentnic[0]->{'fullnic'};
	$cur_IP = $currentnic[0]->{'ipaddress'};

	$pz_a[1][0] = 'dns.' . $fulldomain;
	$pz_a[1][1] = $cur_IP;
	$count_a++;


	if ( ($mainhost ne '') && ($mainhost ne 'dns') )
	{
		$pz_a[$count_a][0] = $mainhost;
		$pz_a[$count_a][1] = $virthostIP;
		$count_a++;

		if ($mainhost ne 'www')
		{
			$pz_cname[$count_cname][0] = 'www';
			$pz_cname[$count_cname][1] = $fqdn;
			$count_cname++;
		}
		if ($mainhost ne 'ftp')
		{
			$pz_cname[$count_cname][0] = 'ftp';
			$pz_cname[$count_cname][1] = $fqdn;
			$count_cname++;
		}
		if ($mainhost ne 'mail')
		{
			$pz_cname[$count_cname][0] = 'mail';
			$pz_cname[$count_cname][1] = $fqdn;
			$count_cname++;
		}

	}
	else
	{
		$pz_cname[$count_cname][0] = 'www';
		$pz_cname[$count_cname][1] = $fulldomain;
		$count_cname++;		
		$pz_cname[$count_cname][0] = 'mail';
		$pz_cname[$count_cname][1] = $fulldomain;
		$count_cname++;		
		$pz_cname[$count_cname][0] = 'ftp';
		$pz_cname[$count_cname][1] = $fulldomain;
		$count_cname++;		
	}

	# All records are ready.


	open (ZFILE1, ">/home/named/var/named/$zonefilename") or return 221;
	flock (ZFILE1, 2) or return 222;

	$unusable = zone_output(0,$fulldomain,\*ZFILE1,\@pz_soa,\@pz_ns,\@pz_mx,\@pz_a,\@pz_cname,\@pz_ptr,\@pz_nssub);

	close (ZFILE1) or return 228;

	return 0;

}


##########################################
sub zone_construct_rev
{
	my ($useless, $revsubnetIP, $zonefilename, $adminemail) = @_;

	my @pz_soa;
	my @pz_ns;
	my @pz_mx;
	my @pz_a;
	my @pz_cname;
	my @pz_ptr;
	my @pz_nssub;

	my $count_ns;
	my $count_ptr;

	my ($i,$j);
	my ($t1,$t2);
	my (@a1,@a2);
	my $ipsn;
	my $unusable;

	if ( ($revsubnetIP eq '') || ($zonefilename eq '') )
	{
		return 220;
	}

	$i = ($revsubnetIP =~ /([0-9.]+)/);
	if (!( ($` eq '') && ($' eq lc('in-addr.arpa.')) && ($& ne '') ))
	{
		# not okay, subnet IP has bad characters.
		return 225;
	}

	$ipsn = join('.', split('\.',$revsubnetIP) ) . '.';
	# Fix this!

#	print ("Filtered reverse subnet is $ipsn <- \n");

#	print ("Current reversed subnet is [" . $revsubnetIP . "]\n");

	$count_ns = 0;
	$count_ptr = 0;

	$pz_soa[0] = '@';
	$pz_soa[1] = 'localhost.';
	$pz_soa[2] = 'postmaster.' . $adminemail;

	$pz_soa[3] = '1001';
	$pz_soa[4] = '3H';
	$pz_soa[5] = '1H';
	$pz_soa[6] = '1W';
	$pz_soa[7] = '12H';

	$pz_ns[0][0] = '@';
	$pz_ns[0][1] = 'localhost.';
	$count_ns++;

	# All records are ready.

	open (ZFILE1, ">/home/named/var/named/$zonefilename") or return 221;
	flock (ZFILE1, 2) or return 222;

	$unusable = zone_output(0,$ipsn,\*ZFILE1,\@pz_soa,\@pz_ns,\@pz_mx,\@pz_a,\@pz_cname,\@pz_ptr,\@pz_nssub);

	close (ZFILE1) or return 228;

	return 0;

}


##############################################
# public function
sub zone_changeheader
{
	my ($useless,$zonename,$zonefilename,$newzone_soa) = @_;
	my @results;

	my $i;
	my $unused;
	my $unusable;
	my $unlikely;

	my (@zone_soa,@zone_ns,@zone_mx,@zone_a,@zone_cname,@zone_ptr,@zone_nssub);

	my ($newserver,$newemail);
	my (@a1,@a2);
	my ($t1,$t2);


	open (ZFILE1, "</home/named/var/named/$zonefilename") or return 218;
	flock (ZFILE1, 2) or return 219;

	$unusable = parse_zonefile(0,$zonename,\*ZFILE1,\@zone_soa,\@zone_ns,\@zone_mx,\@zone_a,\@zone_cname,\@zone_ptr,\@zone_nssub);

	if ($unusable eq 0)
	{
		# parse_zonefile returned error code $unusable

		if (scalar(@zone_soa) < 1)
		{
			# If no entries in this array, then escape
			return 231;
		}
		
		if ($$newzone_soa[1] ne '')
		{
			@a1 = split('\.',$$newzone_soa[1]);

			while(scalar(@a1) && defined($a1[0]) )
			{
				$t1 = shift(@a1);
				if ( ($t1 ne '') && defined($t1) )
				{
					push(@a2, $t1);
				}
			}

			$newserver = (join ('.', @a2 ) ) . '.';

			$zone_soa[1] = $newserver;

			@a2 = ();

		}

		if ($$newzone_soa[2] ne '')
		{
			@a1 = split('\.', join('.',split('\@',$$newzone_soa[2]))  );


			while(scalar(@a1) && defined($a1[0]) )
			{
				$t1 = shift(@a1);
				if ( ($t1 ne '') && defined($t1) )
				{
					push(@a2, $t1);
				}
			}

			$newemail = (join ('.', @a2 ) ) . '.';

			$zone_soa[2] = $newemail;

			@a2 = ();
		}


		for ($i=3; $i<8; $i++)
		{
			if ($$newzone_soa[$i] ne '')
			{
				# Copy only values that are present.
				$zone_soa[$i] = $$newzone_soa[$i];
			}
		}
	
		# Next, output all new zone values back to zone.
		
		open (ZFILE2, ">/home/named/var/named/$zonefilename") or return 218;
		$unlikely = zone_output(0,$zonename,\*ZFILE2,\@zone_soa,\@zone_ns,\@zone_mx,\@zone_a,\@zone_cname,\@zone_ptr,\@zone_nssub);

		close (ZFILE2) or return 228;
		close (ZFILE1) or return 228;

	}
	else
	{
		close (ZFILE1) or return 228;
		return $unusable;
	}

	return $unlikely;
}


##############################################
# public function
sub zone_increment
{
	my ($useless,$zonename,$zonefilename) = @_;
	my @results;

	my $i;
	my $unused;
	my $unusable;
	my $unlikely;

	my (@zone_soa,@zone_ns,@zone_mx,@zone_a,@zone_cname,@zone_ptr,@zone_nssub);

	my ($t1,$t2);

	open (ZFILE1, "</home/named/var/named/$zonefilename") or return 218;
	flock (ZFILE1, 2) or return 219;

	$unusable = parse_zonefile(0,$zonename,\*ZFILE1,\@zone_soa,\@zone_ns,\@zone_mx,\@zone_a,\@zone_cname,\@zone_ptr,\@zone_nssub);

	if ($unusable eq 0)
	{
		my $DNSDaemon	= new Daemon('named');
		
		# parse_zonefile returned error code $unusable

		if (scalar(@zone_soa) < 1)
		{
			# If no entries in this array, then escape
			return 231;
		}
		
		$t1 = @zone_soa[3]; ##################

		if ($t1 > 4000000000)
		{
			$t2 = 1;
		}
		else
		{
			$t2 = $t1 + 1;
		}

		@zone_soa[3] = $t2;

		# Next, output all new zone values back to zone.
		
		open (ZFILE2, ">/home/named/var/named/$zonefilename") or return 218;
		$unlikely = zone_output(0,$zonename,\*ZFILE2,\@zone_soa,\@zone_ns,\@zone_mx,\@zone_a,\@zone_cname,\@zone_ptr,\@zone_nssub);

		close (ZFILE2) or return 228;
		close (ZFILE1) or return 228;

		# Now begin BIND "reload" command if appropriate...
		if ($DNSDaemon->GetStatus())
		{
			# Process is running so force a reload.
			Safe::spawn_daemon("/usr/sbin/ndc reload");
		}
	}
	else
	{
		close (ZFILE1) or return 228;
		return $unusable;
	}

	return $unlikely;
}


##############################################
# public function
sub zone_erasedata
{
	my ($useless,$zonename,$zfilename) = @_;
	my $i;

	if ( ($zonename eq '.') || ($zonename eq 'bind') || ($zonename eq 'localhost') || ($zonename eq '0.0.127.in-addr.arpa') )
	{
		# Can't delete a zone which is reserved!
		return 233;
	}

	if ( ($zfilename eq 'master/127.0.0') || ($zfilename eq 'master/bind') || ($zfilename eq 'master/localhost') || ($zfilename eq 'named.root') )
	{
		# Can't delete a zone which is reserved!
		return 233;
	}


	if (-f "/home/named/var/named/$zfilename")
	{
		Safe::backtick("/bin/rm -f /home/named/var/named/$zfilename");
	}
	else
	{
		return 234;
	}

	return 0;
}


##############################################
# public function
sub zone_rawdata
{
	my ($useless,$zonename,$zonefilename) = @_;
	my @results;

	open (ZFILE1, "</home/named/var/named/$zonefilename") or return 218;
	flock (ZFILE1, 2) or return 219;

	my $outputraw = '';
	
	while (<ZFILE1>)
	{
		$outputraw .= $_;
	}

	close (ZFILE1) or return 228;

	return $outputraw;
}



###################################
# public function
sub zone_delmultiple
{
	my ($useless,$zonename,$zonefilename,$rrarr) = @_;
	my @results;

	# Pass in $rrarr as \@array_name !!!

	my $i;
	my $t1;
	my @a1;
	my $found;
	my $unused;
	my $unusable;
	
	#####################################################################################
	# Expecting $rrarr be an array of arrays, representing records to delete, like so:
	# - array of lists
	# - [type,domain,value,returncode]
	# -- [0] type is the record type (use a supported type such as A,MX,CNAME,NS,PTR)
	# -- [1] domain is the domain name (should always be '@')
	# -- [2] value is the name of the name server
	# -- [3] returncode will be filled in by this function, and will contain
	#        the output for this specific delete operation. This need not be defined
	#        in advance by the caller.
	#
	#####################################################################################
	# Function return codes:
	# - similar to zone_delrecord on a serious/fatal error only
	# - zero '0' if record delete process started smoothly
	# - check individual record returns for specific outcomes
	#
	#####################################################################################
	

	my (@zone_soa,@zone_ns,@zone_mx,@zone_a,@zone_cname,@zone_ptr,@zone_nssub);

	if ( !(defined($rrarr) ) || (scalar(@$rrarr) < 1 ))
	{
		# Missing or empty array!  Die quickly.
		return 240;
	}


	open (ZFILE1, "</home/named/var/named/$zonefilename") or return 218;
	flock (ZFILE1, 2) or return 219;

	$unusable = parse_zonefile(0,$zonename,\*ZFILE1,\@zone_soa,\@zone_ns,\@zone_mx,\@zone_a,\@zone_cname,\@zone_ptr,\@zone_nssub);

	my $rec_type;
	my $rec_domain;
	my $rec_data;
	my $rec_output;
	my $r_count = 0;
	my $r_size = scalar(@$rrarr);


	for ($r_count = 0; $r_count < $r_size; $r_count++)
	{
		# Iterate through each entry in the array,
		# pull out the relevant values, and
		# process the removal of each record if it exists.
		# Save all output and error codes for each record.
		
		$rec_type   = $$rrarr[$r_count][0];
		$rec_domain = $$rrarr[$r_count][1];
		$rec_data   = $$rrarr[$r_count][2];
		$rec_output = 0;
		$found = 0;

		# Check for errors.
		
		if ( ($rec_type eq '') || ($rec_domain eq '') || ($rec_data eq '') )
		{
			# Missing data fields.  Please try again.
			$rec_output = 212;
			goto CAPTUREPLACE;
		}

		# Begin processing work.

		if ($rec_type eq 'A')
		{
			for ($i = 0; $i < scalar(@zone_a) ; $i ++)
			{
				if ( ($rec_domain eq $zone_a[$i][0]) && ($rec_data eq $zone_a[$i][1]) )
				{
					$found = 1;
					$zone_a[$i][0] = '';
					$zone_a[$i][1] = '';
				}
			}

		}
		elsif ($rec_type eq 'CNAME')
		{
			for ($i = 0; $i < scalar(@zone_cname) ; $i ++)
			{
				if ( ($rec_domain eq $zone_cname[$i][0]) && ($rec_data eq $zone_cname[$i][1]) )
				{
					$found = 1;
					$zone_cname[$i][0] = '';
					$zone_cname[$i][1] = '';
				}
			}

		}
		elsif ($rec_type eq 'MX')
		{
			for ($i = 0; $i < scalar(@zone_mx) ; $i ++)
			{
				if ( ($rec_domain eq $zone_mx[$i][0]) && ($rec_data eq $zone_mx[$i][1]) )
				{
					$found = 1;
					$zone_mx[$i][0] = '';
					$zone_mx[$i][1] = '';
					$zone_mx[$i][2] = '';
				}
			}

		}
		elsif ($rec_type eq 'NS')
		{
			if ( ($rec_domain eq '@') || ($rec_domain eq $zonename) )
			{
				for ($i = 0; $i < scalar(@zone_ns) ; $i ++)
				{
					if ( ($rec_domain eq $zone_ns[$i][0]) && ($rec_data eq $zone_ns[$i][1]) )
					{
						$found = 1;
						$zone_ns[$i][0] = '';
						$zone_ns[$i][1] = '';
					}
				}

			}
			else
			{
				for ($i = 0; $i < scalar(@zone_nssub) ; $i ++)
				{
					if ( ($rec_domain eq $zone_nssub[$i][0]) && ($rec_data eq $zone_nssub[$i][1]) )
					{
						$found = 1;
						$zone_nssub[$i][0] = '';
						$zone_nssub[$i][1] = '';
					}
				}

			}

		}
		elsif ($rec_type eq 'PTR')
		{
			for ($i = 0; $i < scalar(@zone_ptr) ; $i ++)
			{
				if ( ($rec_domain eq $zone_ptr[$i][0]) && ($rec_data eq $zone_ptr[$i][1]) )
				{
					$found = 1;
					$zone_ptr[$i][0] = '';
					$zone_ptr[$i][1] = '';
				}
			}

		}
		else
		{
			# Unsupported record type.
			$rec_output = 210;
			goto CAPTUREPLACE;
		}

		if ($found eq 0)
		{
			# Sorry, record does not exist as given.
			$rec_output = 216;
		}


		# Now, capture the results of our hard work earlier.
		CAPTUREPLACE:

		$$rrarr[$r_count][3] = $rec_output;


	}	# End master FOR loop (for $rrarr array)


	open (ZFILE2, ">/home/named/var/named/$zonefilename") or return 218;

	$unusable = zone_output(0,$zonename,\*ZFILE2,\@zone_soa,\@zone_ns,\@zone_mx,\@zone_a,\@zone_cname,\@zone_ptr,\@zone_nssub);

	close (ZFILE2) or return 228;
	close (ZFILE1) or return 228;

	return 0;
}




#############
# END of FILE
#############
1;





