#!/usr/bin/perl
package Forward;

###############################################################################
#                                                                             #
# Filename: Forward.pm                                                        #
#                                                                             #
# Description: Contains script needed to configure mail forwarding settings.  #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use vars qw(@ISA @EXPORT @EXPORT_OK);
use strict;
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw( ); 

use IO::Handle;
use Data::Dumper; # debug

######################################################################
# global settings
######################################################################

use vars qw($forward_name $Marker $alias_tag $autoreply_tag);
$forward_name		= ".forward";
$Marker				= "Programmatic";
$alias_tag			= "Alias";
$autoreply_tag		= "AutoReply";

# the potential types of .forward entries

use vars qw($ALIAS $AUTOREPLY);
$ALIAS		= 'alias';
$AUTOREPLY	= 'autoreply';

use vars qw($entry_name_to_tag $entry_tag_to_name);
$entry_name_to_tag = {
	'alias'		=> 'Alias',
	'autoreply'	=> 'AutoReply',
};
$entry_tag_to_name = {}; # to be reverse-populated in the class constructor

######################################################################
# test routine
######################################################################

#&testroutine;

sub testroutine {

	# create a forward object
	my $forward = Forward->new("user1") || die "error";

	# Set an AUTOREPLY entry
	$forward->set("this is an autoreply entry",$Forward::AUTOREPLY);

	# Set an ALIAS entry
	$forward->set("this is an alias entry",$Forward::ALIAS);

	# Get the ALIAS entry
	print "The alias entry is: ",$forward->get($Forward::ALIAS),"\n";
	
	# Get the AUTOREPLY entry
	print "The autoreply entry is: ",$forward->get($Forward::AUTOREPLY),"\n";
	
	# Delete the AUTOREPLY entry
	$forward->set(undef,$Forward::AUTOREPLY);

	# Delete the ALIAS entry
	$forward->set(undef,$Forward::ALIAS);

	$forward->save;
}

######################################################################
# primary functions
######################################################################

#
# Class constructor
#
# Stuff that only needs to happen once for the entire class.
# (i.e., not per-instance)
#

{
        # populate $entry_tag_to_name by making a reverse hash
        foreach my $i (keys %$entry_name_to_tag) {
                $entry_tag_to_name->{$entry_name_to_tag->{$i}} = $i;
        }
}

#
# new($username)
#
# Instantiate a new Forward object, and open and parse the .forward file
# in the home directory of the user $username.
#
# An exclusive flock() is in place on the .forward file for the
# lifetime of the object.
#

sub new {
	my ($class, $username) = @_;

	# data structure
	my $self = {
		'username'	=> $username,
		'uid'		=> undef,
		'gid'		=> undef,
		'path'		=> undef,
		'handle'	=> new IO::Handle,
		'items'		=> [],
		'index_alias'	=> undef,
		'index_autoreply' => undef,
	};

	# bless!
	bless($self, $class);

	# get information about the user
	my @pwent = getpwnam($self->{username});
	if (! @pwent) { return undef; }
	$self->{path} = $pwent[7]; # home directory
	$self->{uid}  = $pwent[2];
	$self->{gid}  = $pwent[3];
	$self->{path} .= '/' . $forward_name; # full path to .forward

	# create a zero-length .forward file if one doesn't exist
	if ( ! -f $self->{path} ) {

		# create it
		open($self->{handle},"> $self->{path}") || return undef;
		close($self->{handle});

		# set the correct permissions
		chown($self->{uid},$self->{gid},$self->{path});
		chmod(0644,$self->{path});
	}

	# open the .forward file
	open($self->{handle},"+< $self->{path}") || return undef;

	# obtain an exclusive flock on the file
	flock($self->{handle},2);

	# read in the .forward file
	my $line_type = undef;
	while (my $line = readline(*{$self->{handle}})) {

		# chomp the trailing newline
		chomp $line;

		# meta or real?
		if ($line =~ /^# $Marker (.+)/) {
			# Process meta information
			if (exists $entry_tag_to_name->{$1}) {
				$line_type = $entry_tag_to_name->{$1};
				next;
			}
		} else {
			# Process real lines

			# zap any previous entry that was using this tag
			if (defined($line_type)) {
				$self->delete_by_type($line_type);
			}

			# create the entry
			my $entry = {};
			$entry->{type} = $line_type;
			$entry->{data} = $line;
			push(@{$self->{items}},$entry);

		}

		# reset the $line_type guess
		$line_type = undef;
	}

	# return a blessed object
	return $self;
}

#
# set($line,$type)
#
# Adds $line to the .forward file (in memory).  If $type is specified,
# any other existing entry of that type will be deleted first.
#

sub set {
	my ($self,$line,$type) = @_;

	# delete any old entry with this tag
	if (defined($type) && exists($entry_name_to_tag->{$type})) {
		$self->delete_by_type($type);
	}

	# A $line of undef or "" means to delete only; no need to
	# add a new entry.
	if ((! defined($line)) || ($line eq '')) {
		return;
	}

	# otherwise, add the line to the list
	push(@{$self->{items}},{'data'=>$line,'type'=>$type});
}

#
# get($type);
#
# Get a line from the .forward file based on its type
#

sub get {
	my ($self,$type) = @_;

	# iterate over the entries
	foreach my $i (@{$self->{items}}) {
		if (	defined($i->{type}) &&
			($i->{type} eq $type)  ) {
			return $i->{data};
		}
	}

	return undef;
}

#
# save
#
# Writes the .forward file back to disk
#

sub save {
	my ($self) = @_;

	# zero the .forward file
	seek($self->{handle},0,0);
	truncate($self->{handle},0);

	foreach my $i (@{$self->{items}}) {
		# print any needed tag line
		if (defined($i->{type}) && exists($entry_name_to_tag->{$i->{type}})) {
			print {$self->{handle}} '# ',$Marker,' ',$entry_name_to_tag->{$i->{type}},"\n";
		}
		# print the entry itself
		print {$self->{handle}} $i->{data},"\n";
	}
}


#
# DESTROY
#
# Destructor to delete the .forward file if it is empty
sub DESTROY {
	my ($self) = @_;

	my $delete_file = 0;

	# seek to the end of the file
	seek($self->{handle},0,2);
	# if the file position is zero (zero-length file), then
	# throw the $delete_file flag.
	if (tell($self->{handle}) == 0) {
		$delete_file = 1;
	}

	# close the file
	close($self->{handle});

	# delete the file if necessary
	if ($delete_file) {
		unlink($self->{path});
	}
}

######################################################################
# utility routines
######################################################################

#
# delete_by_type($type)
#
# Delete all entries of the type $type
#

sub delete_by_type {
	my ($self, $type) = @_;

	my $i = 0;
	while ($i < scalar(@{$self->{items}})) {
		if ( defined($self->{items}->[$i]->{type}) &&
		    ($self->{items}->[$i]->{type} eq $type) ) {
			splice(@{$self->{items}}, $i, 1);
			next;
		}
		$i++;
	}
}

1;

