package LanguageLogCGI;

###############################################################################
#                                                                             #
# Filename: LanguageLogCGI.pm                                                 #
#                                                                             #
# Description: Contains script needed to manage logged resource data.         #
#                                                                             #
# Pattern(s): Iterator                                                        #
#                                                                             #
# References: EnumeratorArray, LanguageLogCGIElement                          #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageLogCGIElement;
use EnumeratorArray;
use CGI;

###############################################################################
# LanguageLogCGI::new
###############################################################################

sub new
{
	my ($class, $LogPath, $ResourceRootDirectory, $ResourceSuffixListRef) = @_;

	# Initialize the object.
	my $this = 
	{
		'preferred-language'		=> 'en',
		'resource-root-directory' 	=> ('' ne $ResourceRootDirectory) ? $ResourceRootDirectory : undef,
		'resource-suffix-list'		=> ('' ne $ResourceSuffixListRef) ? $ResourceSuffixListRef : undef,
		'default-enumerator-size'	=> 100, # Elements
	};

	# Safe default(s).
	$this->{'resource-root-directory'}	= './' unless(defined($this->{'resource-root-directory'}));

	# Align the object with this package.
	bless($this, $class);
	
	# Polish the root directory.
	$this->{'resource-root-directory'}	= $this->PolishDirectory($this->{'resource-root-directory'});

	# Try to attach to the resource.
	$this->Attach($LogPath);

	# Return the object.
	return $this;
}

###############################################################################
# LanguageLogCGI::DESTROY
###############################################################################

sub DESTROY
{
	my $this = shift;

	# Clean up any objects in use.
	$this->Detach();
}

###############################################################################
# LanguageLogCGI::GetPreferredLanguage
###############################################################################

sub GetPreferredLanguage
{
	my $this = shift;

	# Return the requested data.
	return $this->{'preferred-language'};
}

###############################################################################
# LanguageLogCGI::SetPreferredLanguage
###############################################################################

sub SetPreferredLanguage
{
	my ($this, $PreferredLanguage) = @_;

	# Distill the language code.
	$PreferredLanguage = $this->DistillLanguageCode($PreferredLanguage);
    
    # Check to see if the preferred language is not already in use...
	if ((!defined($PreferredLanguage)) ||
		($PreferredLanguage ne $this->{'preferred-language'}))
	{
		# Detach from the current resource list.
		$this->Detach();
		
		# Assume the new value provided.
		$this->{'preferred-language'} = $PreferredLanguage;
	}
}

###############################################################################
# LanguageLogCGI::DistillLanguageCode
###############################################################################

sub DistillLanguageCode
{
	my ($this, $LanguageCode) = @_;

	# Validate the input parameter(s)...
	if (defined($LanguageCode))
	{
		# Get the purified language code.
		$LanguageCode = (split(/-/, $LanguageCode))[0];
	}

	return $LanguageCode;
}

###############################################################################
# LanguageLogCGI::Attach
###############################################################################

sub Attach
{
	my ($this, $LogPath)	= @_;
	my $Return = 0;

	# Validate the input parameters...
	if ((defined($LogPath)) &&
		('' ne $LogPath))
	{
		# Make sure the object is detached...
		if (0 == ($Return = $this->Detach(1)))
		{
			# Cache the file path.
			$this->{'file-path'} = $LogPath;
		}
	}
	else
	{
		$Return = 1;
	}

	return $Return;
}

###############################################################################
# LanguageLogCGI::Reset
###############################################################################

sub Reset
{
	my $this = shift;

	# Undefine the object elements.
    undef($this->{'file-path'});
}

###############################################################################
# LanguageLogCGI::Detach
###############################################################################

sub Detach
{
	my ($this)	= @_;
	my $Return	= 0;

	# If there is a file handle...
	if (defined($this->{'file-path'}))
	{
		# Reset the member data.
		$this->Reset();
	}

	return $Return;
}

###############################################################################
# LanguageLogCGI::GetLogPath
###############################################################################

sub GetLogPath
{
	my $this = shift;

	# Return the requested data.
	return $this->{'file-path'};
}

###############################################################################
# LanguageLogCGI::GetResourceRootDirectory
###############################################################################

sub GetResourceRootDirectory
{
	my $this = shift;

	# Return the requested data.
	return $this->{'resource-root-directory'};
}

###############################################################################
# LanguageLogCGI::SetResourceRootDirectory
###############################################################################

sub SetResourceRootDirectory
{
	my ($this, $ResourceRootDirectory) = @_;

	# Polish the root directory.
	$ResourceRootDirectory = PolishDirectory($ResourceRootDirectory);

	# Check to see if the preferred language is not already in use...
	if ((!defined($ResourceRootDirectory)) ||
		($ResourceRootDirectory ne $this->{'resource-root-directory'}))
	{
		# Detach from the current resource list.
		$this->Detach();
		
		# Assume the new value provided.
        $this->{'resource-root-directory'} = $ResourceRootDirectory;
	}
}

###############################################################################
# LanguageLogCGI::GetResourceSuffixList
###############################################################################

sub GetResourceSuffixList
{
	my $this = shift;

	# Return the requested data.
	return @{ $this->{'resource-suffix-list'} };
}

###############################################################################
# LanguageLogCGI::SetResourceSuffixList
###############################################################################

sub SetResourceSuffixList
{
	my ($this, @SuffixList) = @_;

	# Detach from the current resource list.
	$this->Detach();

	# Assume the new value provided.
	$this->{'resource-suffix-list'} = [ @SuffixList ];
}

###############################################################################
# LanguageLogCGI::PolishDirectory
###############################################################################

sub PolishDirectory
{
	my ($this, $Directory) = @_;
	
	# Validate the input parameter(s)...
	if (defined($Directory))
	{
		# If the directory does not have an ending slash...
		unless ($Directory =~ /^.*\/$/)
		{
			# Append the missing slash.
			$Directory .= '/';
		}
	}

	return $Directory;
}

###############################################################################
# LanguageLogCGI::Append
###############################################################################

sub Append
{
	my ($this, $Category, $Type, $Code, $ParameterListRef) = @_;
	my $Return = 0;

	# If there is a file path...
	if ((defined($this->{'file-path'})) &&
		((defined($this->{'resource-root-directory'})) && ('' ne $this->{'resource-root-directory'})) &&
		((defined($this->{'resource-suffix-list'})) && ('' ne $this->{'resource-suffix-list'})))
	{
		my $TimeStamp			= time();
		my $SourceFile			= $0;
        my $ResourceSuffixList;
		my $ParameterList;
		my $RemoteHost;
		my $ServerName;
		my $UserName;
		my $Entry;
		my $cgi					= new CGI;

		# Get the CGI information.
		$RemoteHost	= $cgi->remote_host();
		$ServerName	= $cgi->server_name();
		$UserName	= $cgi->user_name();

		# If there is a parameter list reference...
		if (defined($ParameterListRef))
		{
			# Cache the parameter list.
			$ParameterList = '[' . join('', @{ $ParameterListRef }) . ']';
		}
		else
		{
			# Safe default.
			$ParameterList = '';
		}

		# Cache the resource suffix list.
		$ResourceSuffixList = '[' . join('', $this->GetResourceSuffixList()) . ']';
		
		# Build the entry.
		$Entry = join('', $TimeStamp, $RemoteHost, $ServerName, $UserName, $SourceFile, $this->{'resource-root-directory'}, $ResourceSuffixList, $Category, $Type, $Code, $ParameterList);

		# Try to open the logfile...
		if (open(LOGFILE, ">> $this->{'file-path'}"))
		{
			# Lock the file.
			flock(LOGFILE, 2);

			# Spew out the entry.
			print LOGFILE $Entry . "\n";

			# Unlock the file.
			flock(LOGFILE, 8);
			
			# Close the file handle.
			close(LOGFILE);
		}
		else
		{
			$Return = 1;
		}
	}
	else
	{
		$Return = 1;
	}

	return $Return;
}

###############################################################################
# LanguageLogCGI::enumeratorElements
###############################################################################

sub enumeratorElements
{
	my ($this, $StartIndex)	= @_;
	my $Enumerator;
	my $Handle;

	# Safe default(s).
	$StartIndex = $this->{'default-enumerator-size'} unless(defined($StartIndex));

    # Try to open the logfile...
	if ((defined($this->{'file-path'})) &&
		(open(LOGFILE, "< $this->{'file-path'}")))
	{
		my $FileSize;
		
		# Lock the file.
		flock(LOGFILE, 2);

		# Get the file size.
		if ($FileSize = -s $this->{'file-path'})
		{
			my @ElementEnumeration;
			my @ElementList;
			my $ElementRaw;
			my $Element;
			my @Chunk;
			my $CountIndex	= 0;
			my $SeekOffset	= 0;
			my $LocalBase;
			my $Buffer;
			my $BufferLimit;
			my $Line;

			# Seed the seek offset and buffer limit...
			$SeekOffset = $BufferLimit = (1024 < $FileSize) ? 1024 : $FileSize;

			# Seek to the new position and read in the file...
			while ((seek(LOGFILE, ($SeekOffset * -1), 2)) &&
				   (read(LOGFILE, $Buffer, $BufferLimit)))
			{
				# Break out the buffer.
				@Chunk = split(/^/m, $Buffer);

				# Reset the local base.
				$LocalBase = 0;

				# Run through the chunk...
				while ($Line = pop(@Chunk))
				{
					# Look for a compelete line...
					if ($Line =~ /\n/)
					{
						# Increment the local base.
						$LocalBase += length($Line);

						# If the limit has been reached...
						if (($StartIndex + 1) == ++$CountIndex)
						{
							# Set the seek offset if the count index is satisfied.
							$SeekOffset -= $BufferLimit - $LocalBase;
							last;
						}
					}
				}

				# If the end has not been reached...
				if (($CountIndex != ($StartIndex + 1)) &&
					($SeekOffset != $FileSize))
				{
					# If there is enough for one more block.
					if ($FileSize > ($SeekOffset + 1024))
					{
						# Icrement the seek offset.
						$SeekOffset		+= 1024;
					}
					else
					{
						# The buffer limit is the remainder.
						$BufferLimit	= $FileSize - $SeekOffset;

						# The seek offset is the file size.
						$SeekOffset		= $FileSize; 
					}
				}
				else
				{
					last;
				}
			}

			# Seek to the position.
			seek(LOGFILE, ($SeekOffset * -1), 2);

			# Load in the list.
			@ElementList = <LOGFILE>;

			# Additional cleanup if necessary.
			shift(@ElementList) if(($StartIndex + 1) == $CountIndex);

			# Run through the elements...
			while ($ElementRaw = pop(@ElementList))
			{
				# Clean of the new line.
				chomp($ElementRaw);
                
				# Create and seed the element object.
				if ($Element = new LanguageLogCGIElement($this->{'preferred-language'}, $ElementRaw))
				{
					# Add the element to the enumeration array.
					push(@ElementEnumeration, $Element);
				}
			}
			
			# Allocate the enumerator.
			$Enumerator = new EnumeratorArray(\@ElementEnumeration);
		}

		# Unlock the file.
		flock(LOGFILE, 8);

		# Close the file.
		close(LOGFILE);
	}

    return $Enumerator;
}

1;

__END__
