package LogParser;

###############################################################################
#                                                                             #
# Filename: LogParser.pm                                                      #
#                                                                             #
# Description: This module provides log parsing functionality.                #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageResource;
use LanguageLogCGI;

# Globals
my $PreferredLanguage	= (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0];
my $StringResource		= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'LogParser.lrc' ]);

###############################################################################
# LogParser::LogMessage
###############################################################################

sub LogMessage
{
	my ($Code, $Category, $Severity, $ParameterRef) = @_;
	my $Log = new LanguageLogCGI('/usr/ins/intel/gui/logs/server.log', '/usr/ins/intel/gui/locale', [ 'LogParser.lrc' ]);

	# Log the error.
	$Log->Append($Category, $Severity, $Code, $ParameterRef);
}

###################################################################
#Function: parseServiceLogs
#Arguments: listed in order --
#	1. Code: valid codes are: "web", "mail", 
#		"system", and "all".
#	2. Locale.
#Return Value: A list of messages components of all the logs
#Description: This function checks for the existence of the 
#	requested log file, parses the file for messages 
#	corresponding to the code passed, concatenates the 
#	matching messages to form an array of date~message 
#	entries, and returns this array to the calling program.
###################################################################  

sub parseServiceLogs
{
	my $code = shift(@_);
        my $locale = shift(@_);
	my @logFileBuffer;
	my @splitLine;
	my $count = 0;
	my $line;
	my @requestedArray;
	my $logFileWithPath = "";
        my @finalRequestedArray;
        my $APPROX_CHARS_IN_LINE = ConfigHandler::getConfig(ConfigHandler::getPath("CONSTANTS"), "APPROX_CHARS_IN_LOG_LINE");

	# add 1 to $maxLogsToRead 'cause the oldest entry line is deleted from the final array
	my $maxLogsToRead = ConfigHandler::getConfig(ConfigHandler::getPath("CONSTANTS"), "MAX_LOGS_TO_DISPLAY") + 1; 
        my $useSeek = 1; #use seek for file read


        # default locale to 'en' -- don't take no crap (for locale)!
        if(!$locale || !(($locale eq "en") || ($locale eq "es") ||
                ($locale eq "fr") || ($locale eq "jp")))
        { $locale = "en"; }

	# set default value
        if(!$code) { $code = "all"; }

	if($code eq "system")
	{ $logFileWithPath = ConfigHandler::getPath("SYSLOG_FILE"); }
	elsif($code eq "web")
	{ $logFileWithPath = ConfigHandler::getPath("HTTP_LOG_FILE"); }
	elsif($code eq "mail")
	{ $logFileWithPath = ConfigHandler::getPath("MAIL_LOG_FILE"); }

	# if file doesn't exist, or has size zero, return an empty array
	if(!(-e $logFileWithPath) && !(-z $logFileWithPath))
	{ 
		my $curTime = localtime(); 
	        $curTime = substr($curTime, 4, (length($curTime) - 9));
		$finalRequestedArray[0] = "$curTime~" . $StringResource->_S('NO_LOG_ENTRY');
		return @finalRequestedArray; 
	}

	# add 1 to $maxLogsToRead 'cause the oldest entry line is deleted from the final array
	my $logLinesToRead = $maxLogsToRead;
	my $readFlag=0;		
	
	my $fileSizeInChars = -s $logFileWithPath;

	#find out if we need to use seek
	if($logLinesToRead*$APPROX_CHARS_IN_LINE >= $fileSizeInChars)
	{
		$logLinesToRead = int ($fileSizeInChars/$APPROX_CHARS_IN_LINE) + 1 ;
		$useSeek=0;
	}
	        
	# open the appropriate log file
    open (LOGFILE, "$logFileWithPath") or LogMessage('FILE_OPEN_ERROR', 'error', 'warning', [ $! ]);

	while($readFlag < 2)
	{
	 	my @requestedArray = "";
		my @logFileBuffer = "";
		my $i = 0;
		my $j = 0;

		if($useSeek == 1)
		{ seek LOGFILE, -$logLinesToRead*$APPROX_CHARS_IN_LINE, 2; } # assuming $APPROX_CHARS_IN_LINE chars per line 
		else
		{ seek LOGFILE, -0, 0; $readFlag = 2; }
	
       	        # read the contents of the file into a buffer
		while($line = <LOGFILE>)
		{ 
			$logFileBuffer[$i] = $line;
			++$i;
		}
	
		if(lc($code) eq "all")
		{ return @logFileBuffer; }

        	# search the line based on the code passed
	        for($i = 0; $i < @logFileBuffer; $i++)
       	 	{
                	if($code eq "web")
                	{
                        	# split the matched line at the '~' character
                        	@splitLine = split / {1,2}/, $logFileBuffer[$i], 7;

	                        if(($splitLine[5]) && (($splitLine[5] eq "[error]") || ($splitLine[5] eq "[notice]") || ($splitLine[5] eq "[warn]") ||
                        	        ($splitLine[5] eq "[crit]")))
                	        {
        	                        $requestedArray[$j] = join " ", $splitLine[0], $splitLine[1], $splitLine[2], $splitLine[3], $splitLine[4];

        	                        # strip the '[' and ']' characters, and the year from the datetime
        	                        $requestedArray[$j] = substr($requestedArray[$j], 4, length($requestedArray[$j])-10);

	                                $requestedArray[$j] .= "~" . $splitLine[6];
                                	$j++;
                       	 	}
                	}
                	elsif($code eq "mail")
                	{
                	        # split the matched line at the '~' character
        	                @splitLine = split / {1,2}/, $logFileBuffer[$i], 7;
				
                	        $requestedArray[$i] = join " ", $splitLine[0], $splitLine[1], $splitLine[2];
        	                $requestedArray[$i] .= "~" . $splitLine[6];
	                }
                	elsif($code eq "system")
               	 	{
        	                # split the matched line at the '~' character
        	                @splitLine = split / {1,2}/, $logFileBuffer[$i], 6;
	
				# don't display blank and useless messages written by CGIs!
				if(length($splitLine[5]) > 10)
				{
                        		$requestedArray[$i] = join " ", $splitLine[0], $splitLine[1], $splitLine[2];
                	        	$requestedArray[$i] .= "~" . $splitLine[4] . " " . $splitLine[5];
				}
				else
				{ $requestedArray[$i] = ''; }
        	        }
	        }

                for($i = 0; $i < @requestedArray; $i++)
                {
                        if(!$requestedArray[$i] eq '')
                        { $finalRequestedArray[$count] = $requestedArray[$i]; $count++; }
                }

		if($useSeek == 1)
		{
			if($readFlag < 1)
			{
				if(@finalRequestedArray < $maxLogsToRead)
				{ $logLinesToRead += $maxLogsToRead; }
				else { $readFlag = 1; }

		        	if($logLinesToRead*$APPROX_CHARS_IN_LINE >= $fileSizeInChars)
       				{
              	 			$logLinesToRead = int ($fileSizeInChars/$APPROX_CHARS_IN_LINE) + 1;
					$useSeek = 0;
             		  		$readFlag = 1;
       				}
			}
			else { $readFlag = 2; }
		}
		else { $readFlag = 2; }	
	}
	# close the file handler
       	close(LOGFILE);

	if(@finalRequestedArray == 0)
	{ 
		my $curTime = localtime(); 
	        $curTime = substr($curTime, 4, (length($curTime) - 9));
		$finalRequestedArray[0] = "$curTime~" . $StringResource->_S('NO_LOG_ENTRY');
	}
	else
	{
		my @finalRequestedArrayToReturn;

		if(@finalRequestedArray == $maxLogsToRead)
		{
                	for(my $i = 1; $i < @finalRequestedArray; $i++)
                	{
                       	 	$finalRequestedArrayToReturn[$i-1] = $finalRequestedArray[$i]; 
                	}
			return @finalRequestedArrayToReturn;
		}
	}

	return @finalRequestedArray;
}

1;
