package MonSettings;

###############################################################################
#                                                                             #
# Filename: MonSettings.pm                                                    #
#                                                                             #
# Description: A facade class for a monitor configuration settings.           #
#                                                                             #
# Pattern(s): Facade, Iterator, Strategy                                      #
#                                                                             #
# Aggregates: MonConfiguration                                                #
#                                                                             #
# References: MonAlertEmail, MonHostGroup, MonMonitor, MonPeriod, MonService, #
#             MonWatch, MonSerializer, EnumeratorArray                        #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use MonAlertEmail;
use MonHostGroup;
use MonMonitor;
use MonPeriod;
use MonWatch;
use MonConfiguration;
use MonSerializer;
use EnumeratorArray;
use Safe;

###############################################################################
# MonSettings::new
###############################################################################

sub new
{
	my ($class, $DomainName) = @_;

	# Initialize the object.
	my $this = 
	{
		'status'		=> 0,
		'configuration'	=> new MonConfiguration(new MonSerializer('/etc/mon.cf')),
		'domain-name'	=> undef,
		'domain-id'		=> undef,
	};

	# Align the object with this package.
    bless($this, $class);

	# Set the domain name.
	$this->setDomainName($DomainName);

	# Return the object.
	return $this;
}

###############################################################################
# MonSettings::open
###############################################################################

sub open
{
	my $this	= shift;
	my $Return	= 0;

	# If the object is open...
	unless ($this->{'status'})
	{
		# Load the data.
		if ((0 == ($Return = $this->{'configuration'}->open())) &&
			(0 == ($Return = $this->initializeConfiguration())))
		{
			# The object is open.
			$this->{'status'} = 1;
		}
	}

	return $Return;
}

###############################################################################
# MonSettings::close
###############################################################################

sub close
{
	my $this	= shift;
	my $Return	= 0;

	# If the object is open...
	if ($this->{'status'})
	{
		# Save the data.
		if (0 == ($Return = $this->{'configuration'}->close()))
		{
			# The object is closed.
			$this->{'status'} = 0;
		}
	}

	return $Return;
}

###############################################################################
# MonSettings::initializeConfiguration
###############################################################################

sub initializeConfiguration
{
	my $this				= shift;
	my @ServiceList 		= ( 'http', 'smtp', 'ftp' );
	my $IteratorServices;
	my $Watch;
	my $Service;
	my $Item;
	my $Return;

	# Set the baseline configuration.
	$this->{'configuration'}->setConfigurationDirectory('/etc/');
	$this->{'configuration'}->setAlertLibraryDirectory('/usr/local/mon/alert.d');
	$this->{'configuration'}->setMonitorLibraryDirectory('/usr/local/mon/mon.d');
	$this->{'configuration'}->setChildProcessLimit(20);
	$this->{'configuration'}->setHistoryLength(100);
	$this->{'configuration'}->setRandomStartSkew('60s');

	# If the echo monitor is not available...
	unless ( -e '/usr/local/mon/mon.d/echo.monitor' )
	{
		# Create the monitor file.
		Safe::system('/bin/touch /usr/local/mon/mon.d/echo.monitor');

		# Try to open the file...
		if (CORE::open(ECHOMONITOR, '> /usr/local/mon/mon.d/echo.monitor'))
		{
			# Write the program.
            print ECHOMONITOR '#!/usr/bin/perl' . "\n" . 'exit(($ARGV[0]) ? 1 : 0);' . "\n";
			
			# Close the file.
			CORE::close(ECHOMONITOR);

			# Set the permissions.
			chmod(0755, '/usr/local/mon/mon.d/echo.monitor'); 
		}
	}

	# If there is a watch iterator...
	if ((0 == ($Return = $this->getWatch('domain0-global', \$Watch))) &&
		($IteratorServices = $Watch->iteratorServices()))
	{
		my $i;
				
		# Run through the services...
		while ($IteratorServices->hasNext())
		{
			# Get the service.
			$Service = $IteratorServices->next();

			# Run through the list...
			for $i ( 0 .. $#ServiceList )  
			{
				# If the item and the service name match...
				if ($ServiceList[$i] eq $Service->getName())
				{
					# Delete the item.
					splice(@ServiceList, $i, 1);
					last;
				}
			}
		}
	}

	# Run through the list again...
	foreach $Item (@ServiceList)
	{
		# Create the new service object...
		unless ($Service = $this->createService($Item, 'echo.monitor'))
		{
			# Error.
			$Return = 20007013;
			last;
		}
		elsif (0 != ($Return = $Watch->addService($Service)))
		{
			last;
		}
	}
	
	return $Return;
}

###############################################################################
# MonSettings::setDomainName
###############################################################################

sub setDomainName
{
	my ($this, $DomainName) = @_;

	# Safe default.
	$DomainName = 'domain1' unless(defined($DomainName));
	
	# Set the data.
    $this->{'domain-name'}	= $DomainName;
	$this->{'domain-id'}	= $this->createDomainID($DomainName);

	return 0;
}

###############################################################################
# MonSettings::modifyDomainName
###############################################################################

sub modifyDomainName
{
	my ($this, $DomainName) = @_;
	my $Return				= 0;

	# Safe default.
	$DomainName = '' unless(defined($DomainName));

	# Validate the input parameter...
	if ($DomainName)
	{
		# Get the domain ID.
		my $DomainID = $this->createDomainID(('domain1' ne $this->{'domain-name'}) ? $DomainName : 'domain1');
		my $Watch;

		# Get the associated domain watch...
		if (0 == ($Return = $this->getWatch($this->{'domain-id'}, \$Watch)))
		{
			my $HostGroup;

			# Get the watch host group...
			if ($HostGroup = $Watch->getHostGroup())
			{
				# Try to set the name.
				if (0 == ($Return = $HostGroup->setName($DomainID)))
				{
					my $IteratorHosts;

					# Get the host list...
					if ($IteratorHosts = $HostGroup->iteratorHosts())
					{
						# Run through the host list...
						while ($IteratorHosts->hasNext())
						{
							# Remove the host.
							$IteratorHosts->next();
							$IteratorHosts->remove();
						}
					}
					
					# Add the host name.
					$Return = $HostGroup->addHost($DomainName);

					# Set the new domain name...
					$this->setDomainName($DomainName) if('domain1' ne $this->{'domain-name'});
				}
			}
			else
			{
				$Return = 20007001;
			}
		}
	}
	
	return $Return;
}

###############################################################################
# MonSettings::deleteDomainName
###############################################################################

sub deleteDomainName
{
	my $this	= shift;
	my $Return	= 0;

	# Root is fixed...
	if ('domain1' ne $this->{'domain-name'})
	{
		my $Iterator;
		my $HostGroup;
		
		# Get the host group list...
		if ($Iterator = $this->{'configuration'}->iteratorHostGroups())
		{
			# Run through the host groups...
			while ($Iterator->hasNext())
			{
				# Get the host group.
				$HostGroup = $Iterator->next();

				# If the host group is the one we're looking for...
				if ($this->{'domain-id'} eq $HostGroup->getName())
				{
					# Remove the host group.
					$Iterator->remove();
					last;
				}
			}
		}
		
		# Get the watch list...
		if ($Iterator = $this->{'configuration'}->iteratorWatches())
		{
			my $Watch;

			# Run through the watches...
			while ($Iterator->hasNext())
			{
				# Get the watch.
				$Watch = $Iterator->next();

				# If the host group is the one we're looking for...
				if (($HostGroup = $Watch->getHostGroup()) &&
					($this->{'domain-id'} eq $HostGroup->getName()))
				{
					# Remove the watch.
					$Iterator->remove();
					last;
				}
			}
		}
		
		# Set the new domain name to root.
		$this->setDomainName();
	}
	
	return $Return;
}

###############################################################################
# MonSettings::getDomainName
###############################################################################

sub getDomainName
{
	my $this = shift;

	# Return the requested data.
	return (('domain1' ne $this->{'domain-name'}) ? $this->{'domain-name'} : '');
}

###############################################################################
# MonSettings::createDomainID
###############################################################################

sub createDomainID
{
	my ($this, $DomainName) = @_;
	
	# Safe default.
	return (((defined($DomainName)) && ('' ne $DomainName)) ? "$DomainName-services" : '');
}

###############################################################################
# MonSettings::createWatch
###############################################################################

sub createWatch
{
	my ($this, $DomainID) = @_;
	my $HostGroup;
	my $DomainName;
	my $Watch;
	
	# If the host group is not empty...
	unless ('domain0-global' eq $DomainID)
	{
		# Resolve the domain name.
		$DomainName = ('domain1' eq $this->{'domain-name'}) ? Safe::backtick('/bin/hostname -f') : $this->{'domain-name'};
		chomp($DomainName);
	}

	# Try to get a new host group...
	if (($HostGroup = new MonHostGroup($DomainID)) &&
		(('domain0-global' eq $DomainID) || (0 == $HostGroup->addHost($DomainName))) &&
		(0 == $this->{'configuration'}->addHostGroup($HostGroup)))
	{
		# Add this machine to the host list.
		$Watch = new MonWatch($HostGroup);
	}

    return $Watch;
}

###############################################################################
# MonSettings::createService
###############################################################################

sub createService
{
	my ($this, $ServiceType, $MonitorName) = @_;
	my $Monitor;
	my $Service;

	# Safe default.
	$MonitorName = $ServiceType . '.monitor' unless(defined($MonitorName));

	# Create the service...
	if (($Service = new MonService($ServiceType)) &&
		($Monitor = new MonMonitor($MonitorName)) &&
		(0 == $Service->setMonitor($Monitor)))
	{
		# Set the default monitor interval.
		$Service->setMonitorInterval('5m');
		
		# If this is a normal service...
		if ($ServiceType eq (split(/\./, $MonitorName))[0])
		{
			# Set the dependency information.
			$Service->setDependency('domain0-global:'. $ServiceType);
			$Service->setDependencyBehavior('m');
		}
		else
		{
			# Do not use a host group.
			$Monitor->setArguments(0);
			$Monitor->setHostGroupState(0);
		}
	}

	return $Service;
}

###############################################################################
# MonSettings::createPeriod
###############################################################################

sub createPeriod
{
	my $this = shift;
	my $EmailAddressDefault;
	my $Period;
	my $Alert;

	# Build the default domain e-mail address...
	$EmailAddressDefault = ('postmaster@' . (('domain1' ne $this->{'domain-name'}) ? $this->{'domain-name'} : 'localhost'));

	# Create the service...
	if (($Period = new MonPeriod('none', 'mail_alert')) &&
		($Alert  = new MonAlertEmail(0, $EmailAddressDefault)) &&
		(0 == $Period->addAlert($Alert)))
	{
		# Set the default alert interval.
		$Period->setAlertInterval('15m');
	}

	return $Period;
}

###############################################################################
# MonSettings::getWatch
###############################################################################

sub getWatch
{
	my ($this, $DomainID, $WatchRef)	= @_;
	my $IteratorWatches;
	my $Return = 0;
	
	# Validate the parameter...
	if (defined($WatchRef))
	{
		# Clean the reference.
		undef($$WatchRef);
		
		# If there is a watch iterator...
		if ($IteratorWatches = $this->{'configuration'}->iteratorWatches())
		{
			my $HostGroupTemp;
	
			# Run through the watches...
			while ($IteratorWatches->hasNext())
			{
				# Get the watch.
				$$WatchRef = $IteratorWatches->next();
	
				# If the watch is the one we're looking for...
				if (($HostGroupTemp = $$WatchRef->getHostGroup()) &&
					($DomainID eq $HostGroupTemp->getName()))
				{
					# The watch has been found.
					last;
				}
				else
				{
					# Clean the reference.
					undef($$WatchRef);
				}
			}
		}
		
		# If the watch is not available...
		unless ($$WatchRef)
		{
			# Try to create the watch.
			if ($$WatchRef = $this->createWatch($DomainID))
			{
				# Add the watch to the configuration.
				$Return = $this->{'configuration'}->addWatch($$WatchRef);
			}
			else
			{
				$Return = 20007002;
			}
		}
	}
	else
	{
		$Return = 20007003;
	}

    return $Return;
}

###############################################################################
# MonSettings::getServices
###############################################################################

sub getServices
{
	my ($this, $ServiceHashRef) = @_;
	my $IteratorServices;
	my $Return = 0;
	
	# Validate the parameter...
	if (defined($ServiceHashRef))
	{
		my $Watch;
		my $Item;

		# If there is a watch iterator...
		if ((0 == ($Return = $this->getWatch($this->{'domain-id'}, \$Watch))) &&
			($IteratorServices = $Watch->iteratorServices()))
		{
			my $Service;

			# Run through the services...
			while ($IteratorServices->hasNext())
			{
				# Get the service.
				$Service = $IteratorServices->next();

				# Run through the hash list...
				foreach $Item (keys(%{ $ServiceHashRef }))
				{
					# If the item and the service name match...
					if ($Item eq $Service->getName())
					{
						# Cache the service reference.
						$ServiceHashRef->{$Item} = $Service;
						last;
					}
				}
			}
		}

		# Run through the hash list again...
		foreach $Item (keys(%{ $ServiceHashRef }))
		{
			# If there is no service for the item...
			unless ($ServiceHashRef->{$Item})
			{
				# Create the new service object...
				unless ($ServiceHashRef->{$Item} = $this->createService($Item))
				{
					# Error.
					$Return = 20007004;
					last;
				}
				elsif (0 != ($Return = $Watch->addService($ServiceHashRef->{$Item})))
				{
					last;
				}
			}
		}
	}
	else
	{
		$Return = 20007005;
	}
    
	return $Return;
}

###############################################################################
# MonSettings::getPeriods
###############################################################################

sub getPeriods
{
	my ($this, $PeriodListRef, $ServiceListRef) = @_;
	my %ServiceHash;
	my $Return = 0;

	# Validate the parameters...
	if ((defined($PeriodListRef)) &&
		(defined($ServiceListRef)))
	{
		my $Item;

		# Run through the items...
		foreach $Item (@{ $ServiceListRef })
		{
			# Create a hash entry.
			$ServiceHash{$Item}	= '';
		}

		# Get the service references...
		if (0 == ($Return = $this->getServices(\%ServiceHash)))
		{
			my $IteratorPeriods;
			my $Service;
			my $Period;

			# Run through the services...
			foreach $Service (keys(%ServiceHash))
			{
				# Clear the reference.
				undef($Period);
				
				# Get the period iterator...
				if ($IteratorPeriods = $ServiceHash{$Service}->iteratorPeriods())
				{
					# Run through the periods...
					while ($IteratorPeriods->hasNext())
					{
						# Get the period.
                        $Period = $IteratorPeriods->next();
						
						# If the period is the one associated with the domain...
						if ('mail_alert' eq $Period->getLabel())
						{
							# Success.
							last;
						}
						else
						{
							# Clear the reference.
							undef($Period);
						}
					}
				}
				
				# If the domain period was not found...
				unless ($Period)
				{
					# Create a new period object...
					unless ($Period = $this->createPeriod())
					{
						# Bail out.
						$Return = 20007006;
					}
					elsif (0 != ($Return = $ServiceHash{$Service}->addPeriod($Period)))
					{
						last;
					}
				}
						
				# If a period was found/created...
				if ($Period)
				{
					# Add the period to the list.
					push(@{ $PeriodListRef }, $Period); 
				}
				else
				{
					# Bail out.
					last;
				}
			}
		}
	}
	else
	{
		$Return = 20007007;
	}

	return $Return;
}

###############################################################################
# MonSettings::getMonitors
###############################################################################

sub getMonitors
{
	my ($this, $DomainID, $MonitorListRef, $ServiceListRef) = @_;
	my %ServiceHash;
	my $Return = 0;

	# Validate the parameters...
	if ((defined($DomainID)) &&
		(defined($MonitorListRef)) &&
		(defined($ServiceListRef)))
	{
		my $IteratorWatches;
		my $IteratorServices;
		my $Watch;
		my $Service;
		my $Item;

		# Run through the items...
		foreach $Item (@{ $ServiceListRef })
		{
			# Create a hash entry.
			$ServiceHash{$Item}	= '';
		}

		# If there is a watch iterator...
		if ($IteratorWatches = $this->{'configuration'}->iteratorWatches())
		{
			my $HostGroup;

			# Run through the watches...
			while ($IteratorWatches->hasNext())
			{
				# Get the current watch.
                $Watch = $IteratorWatches->next();

				# Get either all of the monitors or just some...
				if (($HostGroup = $Watch->getHostGroup()) &&
					($DomainID eq $HostGroup->getName()))
				{
					# If there is a watch iterator...
					if ($IteratorServices = $Watch->iteratorServices())
					{
						# Run through the services...
						while ($IteratorServices->hasNext())
						{
							# Get the service.
							$Service = $IteratorServices->next();

							# Run through the hash list...
							foreach $Item (keys(%ServiceHash))
							{
								# If the item and the service name match...
								if ($Item eq $Service->getName())
								{
									# Add the monitor to the list.
									push(@{ $MonitorListRef }, $Service->getMonitor()); 
									last;
								}
							}
						}
					}
				}
			}
		}
	}
	else
	{
		$Return = 20007008;
	}

	return $Return;
}

###############################################################################
# MonSettings::getMonitorState
###############################################################################

sub getMonitorState
{
	my ($this, $Alert)	= @_;
	my $State			= 0;

	# Validate the input parameter...
	if (defined($Alert))
	{
		my @Monitors;
		
		# Get the monitors...
		if (0 == $this->getMonitors($this->{'domain-id'}, \@Monitors, [ $Alert ]))
		{
			# Use just the first monitor.
			$State = $Monitors[0]->usesHostGroup();
		}
	}

	return $State;
}

###############################################################################
# MonSettings::setMonitorState
###############################################################################

sub setMonitorState
{
	my ($this, $Alert, $State)	= @_;
	my $Return				 	= 0;

	# Validate the input parameters...
	if ((defined($Alert)) &&
		(defined($State)))
	{
		my @Monitors;
		my $Monitor;

		# Get the associated alert state (primes the configuration).
		$this->getAlertState($Alert);

		# Get the monitors associated with the alert...
		if (0 == ($Return = $this->getMonitors($this->{'domain-id'}, \@Monitors, [ $Alert ])))
		{
			# Run through the monitors...
			foreach $Monitor (@Monitors)
			{
				# Set the state.
				$Monitor->setHostGroupState($State);
			}
        }

		# Clear out the data.
		undef(@Monitors);

		# Cleck for domain0 configuration...
		if (('domain1' eq $this->{'domain-name'}) &&
			(0 == ($Return = $this->getMonitors('domain0-global', \@Monitors, [ $Alert ]))))
		{
			# Use just the first monitor.
			$Monitors[0]->setArguments(($State) ? 0 : 1);
			$Monitors[0]->setHostGroupState(0);
		}
	}
	else
	{
		$Return = 20007009;
	}

	return $Return;
}

###############################################################################
# MonSettings::getAlertState
###############################################################################

sub getAlertState
{
	my ($this, $Alert)	= @_;
	my $State			= 0;

	# Validate the input parameter...
	if (defined($Alert))
	{
		my @Periods;
		
		# Get the periods for the domain...
		if (0 == $this->getPeriods(\@Periods, [ $Alert ]))
		{
			# Use just the first period.
			$State = $Periods[0]->getPeriod();
			$State = ('none' eq $State) ? 0 : 1;
		}
	}

	return $State;
}

###############################################################################
# MonSettings::setAlertState
###############################################################################

sub setAlertState
{
	my ($this, $Alert, $State)	= @_;
	my $Return					= 0;

	# Validate the input parameters...
	if ((defined($Alert)) &&
		(defined($State)))
	{
		my @Periods;
		
		# Get the periods for the domain...
		if (0 == ($Return = $this->getPeriods(\@Periods, [ $Alert ])))
		{
			# Use just the first period.
			$Periods[0]->setPeriod(($State) ? '' : 'none');
		}
	}
	else
	{
		$Return = 20007010;
	}

	return $Return;
}

###############################################################################
# MonSettings::updateEmailAddresses
###############################################################################

sub updateEmailAddresses
{
	my ($this, $Command, $EmailAddress) = @_;
	my $Return							= 0;

	# Validate the input parameter...
	if ((defined($Command)) &&
		((2 == $Command) || (defined($EmailAddress))))
	{
		my @Periods;
		
		# Get the periods for the domain...
		if (0 == ($Return = $this->getPeriods(\@Periods, [ 'http', 'ftp', 'smtp' ])))
		{
			my $IteratorAlerts;
			my $IteratorEmailAddresses;
			my $Period;
			my $Alert;

			# Run through the periods...
			foreach $Period (@Periods)
			{
				# If there are alerts (created by createPeriod)...
				if ($IteratorAlerts = $Period->iteratorAlerts())
				{
					# Run through the alerts...
					while ($IteratorAlerts->hasNext())
					{
						# Get the alert.
						$Alert = $IteratorAlerts->next();
						
						# If the alert is an e-mail alert and the type is a 'down' alert...
						if (($Alert->isa('MonAlertEmail')) &&
							(0 == $Alert->getType()))
						{
							# Based on the command...
							if (0 == $Command)
							{
								# Delete the e-mail address.
								$Alert->deleteEmailAddress($EmailAddress);
							}
							elsif (1 == $Command)
							{
								# Add the e-mail address.
								$Alert->addEmailAddress($EmailAddress);
							}
							elsif (2 == $Command)
							{
								# Get the e-mail address enumerator.
								if ($IteratorEmailAddresses = $Alert->iteratorEmailAddresses())
								{
									# Run through the e-mail addresses...
									while ($IteratorEmailAddresses->hasNext())
									{
										# Delete the entry.
										$IteratorEmailAddresses->next();
										$IteratorEmailAddresses->remove();
									}
								}
							}
						}
					}
				}
				else
				{
					# Internal error.
					$Return = 20007011;
					last;
				}
			}
		}
	}
	else
	{
		$Return = 20007012;
	}

	return $Return;
}

###############################################################################
# MonSettings::addEmailAddress
###############################################################################

sub addEmailAddress
{
	my ($this, $EmailAddress) = @_;
	
	# Update the addresses.
	return $this->updateEmailAddresses(1, $EmailAddress);
}

###############################################################################
# MonSettings::deleteEmailAddress
###############################################################################

sub deleteEmailAddress
{
	my ($this, $EmailAddress) = @_;
	
	# Update the addresses.
	return $this->updateEmailAddresses(0, $EmailAddress);
}

###############################################################################
# MonSettings::deleteAllEmailAddresses
###############################################################################

sub deleteAllEmailAddresses
{
	my $this = shift;
	
	# Update the addresses.
	return $this->updateEmailAddresses(2);
}

###############################################################################
# MonSettings::enumeratorEmailAddresses
###############################################################################

sub enumeratorEmailAddresses
{
	my $this = shift;
    my @EmailAddresses;
	my @Periods;
	my $EnumeratorEmailAddresses;

	# Get the 'representative' period for the domain...
	if (0 == $this->getPeriods(\@Periods, [ 'http' ]))
	{
		my $IteratorAlerts;

		# If there are alerts (created by createPeriod)...
		if ($IteratorAlerts = $Periods[0]->iteratorAlerts())
		{
			my $IteratorEmailAddresses;
			my $Alert;
            
			# Run through the alerts...
			while ($IteratorAlerts->hasNext())
			{
				# Get the alert.
				$Alert = $IteratorAlerts->next();

				# If the alert is an e-mail alert and the type is a 'down' alert...
				if (($Alert->isa('MonAlertEmail')) &&
					(0 == $Alert->getType()))
				{
					# Get the e-mail address enumerator.
					if ($IteratorEmailAddresses = $Alert->iteratorEmailAddresses())
					{
						# Run through the e-mail addresses...
						while ($IteratorEmailAddresses->hasNext())
						{
							# Add the address to the list...
							push(@EmailAddresses, $IteratorEmailAddresses->next());
						}

						# There should only be one 'down' alert.
						last;
					}
				}
			}
		}
	}

	# If there are any e-mail addresses...
	if (scalar(@EmailAddresses))
	{
		# Create the enumerator.
		$EnumeratorEmailAddresses = new EnumeratorArray(\@EmailAddresses);
	}

	return $EnumeratorEmailAddresses;
}

1;

__END__
