package Password;
$VERSION="0.30";

###############################################################################
#                                                                             #
# Filename: Password.pm                                                       #
#                                                                             #
# Description: This module provides password configuration support.           #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib "/usr/ins/intel/gui/perllib";

use strict;
use LanguageResource;
use LanguageLogCGI;
use Authen::PAM;
use POSIX;
use Safe;


# Globals
my $PreferredLanguage	= (defined($ENV{'HTTP_ACCEPT_LANGUAGE'})) ? (split(/,/, $ENV{'HTTP_ACCEPT_LANGUAGE'}))[0] : 'en-US';
my $StringResource	= new LanguageResource($PreferredLanguage, '/usr/ins/intel/gui/locale', [ 'Password.lrc' ]);


###############################################################################
# Password::LogMessage
###############################################################################

sub LogMessage
{
	my ($Code, $Category, $Severity, $ParameterRef, $Domain) = @_;
	my $Location	= ($Domain) ? "/home/domain/$Domain/logs/error.log" : '/usr/ins/intel/gui/logs/server.log';
	my $Log		= new LanguageLogCGI($Location, '/usr/ins/intel/gui/locale', [ 'Password.lrc' ]);

	# Log the error.
	$Log->Append($Category, $Severity, $Code, $ParameterRef);
}

################################################################################
# Password String Validation (private) 
################################################################################
#	Checks for reasonable password length
################################################################################

sub validatePWStrings {

        my ($password, $username) = @_;
        my ($feedback, $lenuser, $lenpwd);

################################################################################
# This section is probably unneccessary.  Remove later.
################################################################################
#	if ($username) { $lenuser = length($username); }
#	else { $lenuser = "6"; }
#        if ( !( $lenuser && $lenpwd ) { return "BAD_PASSWD:Username or Password Not Provided"; }
#        elsif (!($lenusername  >=5) or !($lenuser <=32) )  { return "BAD_PASSWD:Invalid Username"; }
################################################################################

        $lenpwd  = length($password);
        if (!($lenpwd >=5) or !($lenpwd  <=32) )  { return ("BAD_PASSWD:Invalid Password"); }
        else { return ("VALID"); }

}

################################################################################
# Check Password - verifies that a password is correct 
################################################################################
# 	The PAM conversation mechanism is funny -- requires a sub within a sub
################################################################################

sub checkPassword {

	my ($self, $username, $password, $Domain) = @_;

	my $service = "passwd";
	my $pamh;

	my $pamCheckConversation = sub {

		my @result;
		while ( @_ ) {

			my $code = shift;
			my $msg = shift;
			my $reply = "";

			$reply = $username if ($code == PAM_PROMPT_ECHO_ON() );
			$reply = $password if ($code == PAM_PROMPT_ECHO_OFF() );

			push @result, PAM_SUCCESS();
			push @result, $reply;
		}

		push @result, PAM_SUCCESS();
		return @result;
	};

	ref($pamh = new Authen::PAM($service, $username, $pamCheckConversation)) ||
                return "Error code $pamh during PAM init!";

	my $feedback = $pamh->pam_authenticate;
	my $details = '';

################################################################################
#
#  Errors
#     0 - Correct Password
#     1 - No Password??
#     7 - Incorrect Password
#     9 - Password too short?
#    10 - Invalid User 
#    20 - Current Password Invalid 
#
################################################################################
	if    ($feedback eq "0")  { $feedback = "VALID"; }
	elsif ($feedback eq "7")  { 
		$details = $StringResource->_S('INVALID_PASSWORD', [ $username ] ); 
                LogMessage(1900002, 'security', 'information', [ $details ], $Domain);
		$feedback = 1900002; 
	}
	elsif ($feedback eq "9")  { 
		$details = $StringResource->_S('SHORT_PASSWORD', [ $username ] ); 
                LogMessage(1900002, 'security', 'information', [ $details ], $Domain);
		$feedback = 1900002; 
	}
	elsif ($feedback eq "10") { LogMessage(1900005, 'security', 'information', [ $username ], $Domain); $feedback = 1900005; } 
	else                      { LogMessage(1900002, 'error', 'information', [ $username ], $Domain); $feedback = 1900002; } 

	return ($feedback);

}

################################################################################
# Change Password - changes the users password
################################################################################

# There is a bug in RedHat's PAM module pam_cracklib.so that causes it to open
# three file descriptors to its dictionary files, then never close them.  This
# wrecks havoc when PAM operations are performed in persistent environments
# such as our Apache mod_perl server.  To work around this bug, always fork
# before performing the actual "change password" operation, so that the unclosed
# file descriptors will be reclaimed by the kernel when the child process exits,
# instead of accumulating.  This &changePassword subroutine merely carries out
# this forking and calls &changePasswordForked, which contain the meat of the
# operation.

sub changePassword {

	my ($self, $username, $newpw, $currentpw, $Domain) = @_;
	my $details = '';
        
	my $pid;
	if (($pid = fork) == 0) {
		####################
		# child process
		####################

		# close all file descriptors above STDERR (2).
		Safe::closefd();

		# call the routine that actually changes the password
		# and exit with its return value.
		my $feedback =  &changePasswordForked($self, $username, $newpw, $currentpw);

		# exit with the result of the password change operation
		POSIX::_exit($feedback);

	}
	####################
	# parent process
	####################

	# get the result of the password change operation
	waitpid($pid,0); my $feedback = ($? >> 8);
	if (! (defined $feedback) ) { $feedback = 128; }

        # take the appropriate action
	if    ($feedback == 0 )  { 
                LogMessage(1900006, 'security', 'information', [ $username ], $Domain);
	}
	elsif ($feedback == 7 )  { 
                $details = $StringResource->_S('INVALID_PASSWORD', [ $username ]);
                LogMessage(1900004, 'security', 'information', [ $details ], $Domain); 
	}
	elsif ($feedback == 9 )  { 
	        $details = $StringResource->_S('SHORT_PASSWORD', [ $username ]);
                LogMessage(1900002, 'security', 'information', [ $details ], $Domain);
	}
	elsif ($feedback == 10)  { 
                $details = $StringResource->_S('INVALID_USER', [ $username ]);
                LogMessage(1900005, 'security', 'information', [ $details ], $Domain);
	}
	elsif ($feedback == 64)  { LogMessage(1900002, 'security', 'information', [ $username ], $Domain); } 
	else { LogMessage(1900003, 'error', 'information', [ $username ], $Domain); } 

	return $feedback;
}

sub changePasswordForked {

	my ($self, $username, $newpw, $currentpw) = @_;

	my $service = "passwd";
	my ($pamh,$state,$feedback);

################################################################################
# The next two lines are new, added to fix problems with a new user (no pw)
# and works fine for root (no oldpw needed).
################################################################################

	if ($currentpw) { $state = "0"; }
	else { $state = "1"; }

	my $validatepw = validatePWStrings($newpw);


################################################################################
# This is temporary until all the password code (such as in user add) is synced.
################################################################################
	$validatepw = 'VALID';
################################################################################


	if ( $validatepw ne "VALID") { return 64; }

	my $pamChangeConversation = sub {

		my (@result, $state);
		while ( @_ ) {

			my $code = shift;
			my $msg = shift;
			my $reply = "";

			$reply = $username if ($code == PAM_PROMPT_ECHO_ON() );
			if ($code == PAM_PROMPT_ECHO_OFF() ) {

				if ($state == 0)    { $reply = $newpw; }
				else 		    { $reply = $currentpw; }
################################################################################
# Old Stuff.  Notice reverse of state from PAM example.  Remove.
################################################################################
#				$reply = $newpw 	if ($state == 0);
#				$reply = $currentpw     if ($state == 1);
#				$reply = $currentpw     if ($state == 2);
################################################################################

				$state++;
			}
			push @result, PAM_SUCCESS();
			push @result, $reply;
		}
		push @result, PAM_SUCCESS();
		return @result;
	};

	ref($pamh = new Authen::PAM($service, $username, $pamChangeConversation)) ||
		die "Error code $pamh during PAM init!";

	$state = 0;
	$feedback = $pamh->pam_chauthtok;

	return $feedback;
}

1;

