package RouteConfig;

###############################################################################
#                                                                             #
# Filename: RouteConfig.pm                                                    #
#                                                                             #
# Description: Contains script needed to manage route configuration data.     #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib "/usr/ins/intel/gui/perllib/";

use strict;
use LanguageLogCGI;
use ConfigurationData;
use Usable;
use Safe;

###############################################################################
# listRoutes
###############################################################################

sub listRoutes
{
	my $RouteDataRaw;
	my @RouteData;

	# Try to get the routing information..
	if ($RouteDataRaw = Safe::backtick('/sbin/route -n'))
	{
		my @RouteDataList = split(/^/m, $RouteDataRaw);
		my $Line;

		# Run through the data...
		foreach $Line (@RouteDataList)
		{
			my %Route;
			
			# Clean off the line cap data.
			$Line =~ s/\r|\n//g;
			
			# Ignore unneeded lines...
			if (($Line !~ /^\S+/) ||
				($Line =~ /Dest/) ||
				($Line =~ /Kern/) ||
				($Line =~ /lo/))
			{
				# Skip the line.
				next;
			}
			else
			{
				# Break out the information.
				($Route{'destination'},
				 $Route{'gateway'},
				 $Route{'netmask'},
				 $Route{'flags'},
				 $Route{'metric'},
				 $Route{'ref'},
				 $Route{'use'},
				 $Route{'nic'}) = split(/\s+/, $Line);
				
				# Add the line to the list.
				push(@RouteData, \%Route);
			}
		}
	}

	return @RouteData;
}

###############################################################################
# configRoute
###############################################################################

sub configRoute
{
	my ($self,$Destination, $Gateway, $Netmask, $nic, $Confirm) = @_;
	my $Item;
	my $Return = 0;

	# If the destination is not the default gateway...
	if ('0.0.0.0' ne $Destination)
	{
		# For each of the items...
		for $Item ($Destination, $Gateway)
		{
			# Chech the address format.
			if ('1' ne Usable::check_ip($Item))
			{
				$Return = 1;
				last;
			}
		}
	}	
	
	# Check the netmask format.
	$Return = 1 if(($Netmask) && ('1' ne Usable::check_ip($Netmask)));

	# If things are still going well...
	if (0 == $Return)
	{
		my @RouteList = listRoutes();
		my $Route;

		# Run through the route list...
		foreach $Route (@RouteList)
		{
			# If the route matches...
			if (($Route->{'destination'} eq $Destination) &&
				($Route->{'nic'}         eq $nic))
			{
				# If there is an override.
				if ('yes' eq $Confirm)
				{
					# Try to delete the route first.
					unless (0 == ($Return = RouteConfig->deleteRoute($Destination, $Gateway, $Netmask, $nic)))
					{
						# Break out.
						last;
					}
				}
				else
				{
					my $Log	= new LanguageLogCGI('/usr/ins/intel/gui/logs/server.log', '/usr/ins/intel/gui/locale', [ 'RouteConfig.lrc' ]);

					# Log the event.
					$Log->Append('error', 'information', 1200002, [ $Destination, $Gateway, $Netmask, $nic ]);

					return 0;
				}
			}
		}

		# If things are still going well...
		if (0 == $Return)
		{
			# Try to create the route.
			$Return = RouteConfig->createRoute($Destination, $Gateway, $Netmask, $nic);
		}
	}

	return $Return;
}

###############################################################################
# deleteRoute
###############################################################################

sub deleteRoute
{
	my ($self, $Destination, $Gateway, $Netmask, $nic, $BrokenDataRef) = @_;
	my $Return = 0;
	my @ArgumentList;

	# If there is a valid destination...
	if ((!$Destination) || 
		((!$Gateway) && ('0.0.0.0' ne $Destination)))
	{
		# Invalid parameter(s).
		$Return = 1;
	}
	else
	{
		
        my $Target;
		
		# Safe default.
		$nic = 'eth0' unless(('eth0' eq $nic) || ('eth1' eq $nic));
		
		# If there is a broken data hash reference...
		if (($BrokenDataRef) &&
			('yes' eq $BrokenDataRef->{'route'}))
		{
			# Prime the locals with the broken? data...
			$Destination	= $BrokenDataRef->{'destination'};
			$Gateway 		= $BrokenDataRef->{'gateway'};
			$Netmask		= ($Netmask) ? $BrokenDataRef->{'netmask'} : '255.0.0.0';
			$nic			= $BrokenDataRef->{'nic'};
		}

		# If there is a network mask...
		if (($Netmask) &&
			('255.255.255.255' ne $Netmask))
		{
			my @DestinationTuples	= split(/\./, $Destination);
			my @NetmaskTuples		= split(/\./, $Netmask);
			my $i;

			# Assume a network.
			$Target = '-net';

			# Run through the tuples...
			for ($i = 0; $i < 4; ++$i)
			{
				# Invert the tuple.
				$NetmaskTuples[$i] = ~int($NetmaskTuples[$i]);
				
				# Check for a host in the tuple...
				if ($NetmaskTuples[$i] & $DestinationTuples[$i])
				{
					# The target is a host.
					$Target = '-host';
					last;
				}
			}
		}
		else
		{
			# Default to host.
			$Target = '-host';
		}

		# Prime the argument list.
		@ArgumentList = ($Target, $Destination);

		# Try to delete the route...
		if (($Return = Safe::system("/sbin/route del @ArgumentList")) && 
			($Gateway))
		{
			# Add the other arguments to the list.
			push (@ArgumentList, 'gw', $Gateway, 'netmask', $Netmask, $nic);
			
			# Try to remove the route again...
			unless (0 == ($Return = Safe::system ("/sbin/route del @ArgumentList")))
			{
				my $Log	= new LanguageLogCGI('/usr/ins/intel/gui/logs/server.log', '/usr/ins/intel/gui/locale', [ 'RouteConfig.lrc' ]);

				# Log the event.
				$Log->Append('error', 'warning', 1200000, [ ($Return / 256), $Destination, $Gateway, $Netmask, $nic ]);
				
				# Set the return code.
				$Return = 1200000;
			}
		}
		
		# If things went well...
		if (0 == $Return)
		{
			my $ServerConfiguration	= new ConfigurationData();

			# Get rid of the dash.
			$Target =~ s/^-(.*)$/$1/;
			
			# Prime the argument list.
			@ArgumentList = ($Target, $Destination);
			
			# Add the gateway to the argument list if specified...
			push (@ArgumentList, 'gw', $Gateway) if(($Gateway) && ('0.0.0.0' ne $Gateway));

			# Add the netmask to the argument list if specified...
			push (@ArgumentList, 'netmask', $Netmask) if(('net' eq $Target) && ($Netmask));

			# Try to open the configuration file...
			if (0 == $ServerConfiguration->Attach('/etc/sysconfig/static-routes', { 'auto-save' => 1, 'mode' => O_RDWR, 'permissions' => 0644, 'loose-match' => 1 }))
			{
				my $Value = join(' ', @ArgumentList);
				
				# Remove the data from the file.
				$ServerConfiguration->DeleteDirectiveList($nic, [ $Value ]);
			}
			else
			{
				my $Log	= new LanguageLogCGI('/usr/ins/intel/gui/logs/server.log', '/usr/ins/intel/gui/locale', [ 'RouteConfig.lrc' ]);

				# Log the event.
				$Log->Append('error', 'critical', 1200004);
			}
		}
	}

	return $Return;
}


###############################################################################
# createDefaultGateway
###############################################################################

sub createDefaultGateway
{
	my ($self, $Destination, $nic) = @_;
	my $Return = 0;
	
	# Check for valid destination IP address for the default GW.
	if ($Destination =~ /^(\d+).(\d+).(\d+).(\d+)$/)
	{
		# Check to make sure we get a valid NIC device.
		$nic = ( (defined($nic)) && (($nic eq "eth0") || ($nic eq "eth1")) ) ? "$nic" : 'eth0';

		# Configure the route...
		unless (0 == ($Return = Safe::system("/sbin/route add default gw $Destination dev $nic")))
		{
			my $Log	= new LanguageLogCGI('/usr/ins/intel/gui/logs/server.log', '/usr/ins/intel/gui/locale', [ 'RouteConfig.lrc' ]);

			# Log the event.                                                                   
			$Log->Append('error', 'warning', 1200006, [ ($Return / 256), $Destination, $nic ]);

			# Set the return code.
			$Return = 1200006;
		}

		else
		{
			my $ServerConfiguration	= new ConfigurationData();
			
			# Try to open the configuration file...
			if (0 == $ServerConfiguration->Attach('/etc/sysconfig/network', { 'auto-save' => 1, 'mode' => O_RDWR, 'permissions' => 0644 }))
			{
				# Remove the data from the file.
				$ServerConfiguration->SetValueKeyed('GATEWAY', "\"$Destination\"" );
				$ServerConfiguration->SetValueKeyed('GATEWAYDEV', "\"$nic\"" );
			}
			else
			{
				my $Log	= new LanguageLogCGI('/usr/ins/intel/gui/logs/server.log', '/usr/ins/intel/gui/locale', [ 'RouteConfig.lrc' ]);

				# Log the event.
				$Log->Append('error', 'critical', 1200007);
			}
		
		}

	}
	else
	{
		$Return = 1200006;
	}

	return $Return;
}


###############################################################################
# deleteDefaultGateway
###############################################################################

sub deleteDefaultGateway
{
	my ($self, $nic) = @_;
	my $Return = 0;
	
	# Check to make sure we get a valid NIC device.
	$nic = ( (defined($nic)) && (($nic eq "eth0") || ($nic eq "eth1")) ) ? "$nic" : 'eth0';

	# Configure the route...
	unless (0 == ($Return = Safe::system("/sbin/route del default dev $nic")))
	{
		my $Log	= new LanguageLogCGI('/usr/ins/intel/gui/logs/server.log', '/usr/ins/intel/gui/locale', [ 'RouteConfig.lrc' ]);

		# Log the event.                                                                   
		$Log->Append('error', 'warning', 1200005, [ ($Return / 256), $nic ]);

		# Set the return code.
		$Return = 1200005;
	}
		
	else
	{
		my $ServerConfiguration	= new ConfigurationData();

		# Try to open the configuration file...
		if (0 == $ServerConfiguration->Attach('/etc/sysconfig/network', { 'auto-save' => 1, 'mode' => O_RDWR, 'permissions' => 0644 }))
		{
			# Remove the data from the file.
			$ServerConfiguration->DeleteValueKeyed('GATEWAY');
			$ServerConfiguration->DeleteValueKeyed('GATEWAYDEV');
		}
		else
		{
			my $Log	= new LanguageLogCGI('/usr/ins/intel/gui/logs/server.log', '/usr/ins/intel/gui/locale', [ 'RouteConfig.lrc' ]);

			# Log the event.
			$Log->Append('error', 'critical', 1200007);
		}

	}

	return $Return;
}


###############################################################################
# createRoute
###############################################################################

sub createRoute
{
	my ($self, $Destination, $Gateway, $Netmask, $nic, $isstatic) = @_;
	my @ArgumentList;
	my $Target;
	my $Return = 0;

	# Make sure that NIC/gateway/netmask make sense.
	$nic = 'eth0' unless(('eth0' eq $nic) || ('eth1' eq $nic));
	undef($Gateway) unless($Gateway =~ /(\d+).(\d+).(\d+).(\d+)/);
	undef($Netmask) unless($Netmask =~ /(\d+).(\d+).(\d+).(\d+)/);

	$isstatic = 1 unless (defined($isstatic));

	# If there is a network mask...
	if ($Netmask)
	{
		my @DestinationTuples	= split(/\./, $Destination);
		my @NetmaskTuples		= split(/\./, $Netmask);
		my $i;

		# Assume a network.
		$Target = '-net';

		# Run through the tuples...
		for ($i = 0; $i < 4; ++$i)
		{
			# Invert the tuple.
			$NetmaskTuples[$i] = ~int($NetmaskTuples[$i]);

			# Check for a host in the tuple...
			if ($NetmaskTuples[$i] & $DestinationTuples[$i])
			{
				# The target is a host.
				$Target	= '-host';
				last;
			}
		}
	}
	else
	{
		# Default to host.
		$Target = '-host';
	}

	# Prime the argument list.
	@ArgumentList = ($Target, $Destination);

	# Add the gateway to the argument list if specified...
	push (@ArgumentList, 'gw', $Gateway) if(($Gateway)  && ('0.0.0.0' ne $Gateway));

	# Add the netmask to the argument list if specified...
	push (@ArgumentList, 'netmask', $Netmask) if(('-net' eq $Target) && ($Netmask));

	# Add the NIC to the argument list if specified...
	push (@ArgumentList, 'dev', $nic) if($nic);
        
	# Configure the route...
	unless (0 == ($Return = Safe::system("/sbin/route add @ArgumentList")))
	{
		my $Log	= new LanguageLogCGI('/usr/ins/intel/gui/logs/server.log', '/usr/ins/intel/gui/locale', [ 'RouteConfig.lrc' ]);

		# Log the event.
		$Log->Append('error', 'warning', 1200001, [ ($Return / 256), $Destination, $Gateway, $Netmask, $nic ]);

		# Set the return code.
		$Return = 1200001;
	}
	elsif ($isstatic) 
	{
		my $ServerConfiguration	= new ConfigurationData();

		# Get rid of the dash.
		$Target =~ s/^-(.*)$/$1/;

		# Prime the argument list.
		@ArgumentList = ($Target, $Destination);

		# Add the gateway to the argument list if specified...
		push (@ArgumentList, 'gw', $Gateway) if(($Gateway) && ('0.0.0.0' ne $Gateway));
		
		# Add the netmask to the argument list if specified...
		push (@ArgumentList, 'netmask', $Netmask) if(('net' eq $Target) && ($Netmask));
		
		# Try to open the configuration file...
		if (0 == $ServerConfiguration->Attach('/etc/sysconfig/static-routes', { 'auto-save' => 1, 'mode' => O_RDWR, 'permissions' => 0644, 'loose-match' => 1 }))
		{
			my $Value = join(' ', @ArgumentList);

			# Set the data into the file.
			$ServerConfiguration->SetDirectiveList($nic, [ $Value ]);
		}
		else
		{
			my $Log	= new LanguageLogCGI('/usr/ins/intel/gui/logs/server.log', '/usr/ins/intel/gui/locale', [ 'RouteConfig.lrc' ]);
	
			# Log the event.
			$Log->Append('error', 'critical', 1200003);
		}
	}

	return $Return;
}

1;

__END__
