package Virtusertable;

###############################################################################
#                                                                             #
# Filename: Virtusertable.pm                                                  #
#                                                                             #
# Description: This module provides configuration support for managing        #
#              mail virtual user tables.                                      #
#                                                                             #
###############################################################################
###############################################################################
#                                                                             #
#  Copyright  2000 - 2001 Intel Corporation.                                 #
#  Intel Corporation All Rights Reserved.                                     #
#                                                                             #
#  The source code contained or described herein and all documents related to #
#  the source code ("Material") are owned by Intel Corporation or its         #
#  suppliers or licensors.  Title to the Material remains with Intel          #
#  Corporation or its suppliers and licensors.  The Material contains trade   #
#  secrets and proprietary and confidential information of Intel or its       #
#  suppliers and licensors.  The Material is protected by worldwide copyright #
#  and trade secret laws and treaty provisions.  No part of the Material may  #
#  be used, copied, reproduced, modified, published, uploaded, posted,        #
#  transmitted, distributed, or disclosed in any way without Intel's prior    #
#  express written permission.                                                #
#                                                                             #
#  No license under any patent, copyright, trade secret or other intellectual #
#  property right is granted to or conferred upon you by disclosure or        #
#  delivery of the Materials, either expressly, by implication, inducement,   #
#  estoppel or otherwise. Any license under such intellectual property rights #
#  must be express and approved by Intel in writing.                          #
#                                                                             #
###############################################################################

# Put the PERL library in the path.
use lib '/usr/ins/intel/gui/perllib';

use strict;
use LanguageLogCGI;
use IO::File;
use Mail;
use Safe;

use vars qw(@ISA @EXPORT @EXPORT_OK);
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw( ); 

######################################################################
# global settings
######################################################################

# path to the virtusertable
use vars qw($virtusertable_path);
$virtusertable_path = "/etc/mail/virtusertable";

# a flock() is on this file while virtusertable operations
# are in progress
use vars qw($lockfile_name);
$lockfile_name = "/var/lock/virtusertable.lock";

######################################################################
# global variables
######################################################################

# the file handle for the lock file
use vars qw($lockfile_handle);
$lockfile_handle = undef;

######################################################################
# test routine
######################################################################

#&testroutine;

sub testroutine {
	my $ret;
	$ret = Virtusertable::update_alias_list("testuser","test1.com",
		"postmaster", "webmaster", "testuser1", "testuser2");
	print $ret,"\n" if ($ret);

	#&delete_by_username("testuser");

	#$ret = Virtusertable::update_alias_list("testuser","test1.com");
	#print $ret,"\n" if ($ret);

}

###############################################################################
# Virtusertable::LogMessage
###############################################################################

sub LogMessage
{
	my ($Code, $Category, $Severity, $ParameterRef, $Domain) = @_;
	my $Location	= ($Domain) ? "/home/domain/$Domain/logs/error.log" : '/usr/ins/intel/gui/logs/server.log';
	my $Log			= new LanguageLogCGI($Location, '/usr/ins/intel/gui/locale', [ 'Virtusertable.lrc' ]);

	# Log the error.
	$Log->Append($Category, $Severity, $Code, $ParameterRef);
}

######################################################################
# supporting functions
######################################################################

#
# &update_by_hash($alias_hash, $delete_value);
#
# $alias_hash   - a hash reference containing the list of
#                 aliases to add or change
# $delete_value - delete all virtusertable lines in which the
#                 right-hand-side matches this value
#

sub update_by_hash {
	my ($AliasHashRef, $User, $Domain) = @_;
	my $delete_value = $User . '@' . $Domain;

	# lock 
	&Virtusertable::lock();

	# determine a temporary filename to use
	my $tempfile = "$virtusertable_path.$$";

	# open the existing virtusertable and a new one
	if (! open(OLD,$virtusertable_path)) {
		# unlock before returning
		Virtusertable::unlock();
		
		# Log the error.
		LogMessage(1600000, 'error', 'critical', [ $virtusertable_path ], $Domain);
		
		return 1600000;
	}
	if (! open(NEW,"> $virtusertable_path.$$")) {
		# close this file handle that we won't be using now.
		close(OLD);
		
		# unlock before returning
		Virtusertable::unlock();
		
		# Log the error.
		LogMessage(1600001, 'error', 'critical', [ "$virtusertable_path.$$" ], $Domain);
		
		return 1600001;
	}

	# loop through the contents of the existing virtusertable
	while(<OLD>) {
		# chomp the trailing newline
		chomp;

		# seperate the left and the right hand side
		my ($lhs, $rhs);
		if (/^(\S+)\s+(.+)$/) {
			$lhs = $1;
			$rhs = $2;
		} else {
			next;
		}

		# skip any entries with a right-hand-side
		# of $delete_value, if a $delete_value was given.
		if ((defined $delete_value) &&
			($delete_value eq $rhs)) {
			next;
		}

		# skip any entries with a left-hand-side that is
		# a key we need to overwrite.
		if (exists $AliasHashRef->{$lhs}) {
			next;
		}

		# otherwise, just copy the line.
		print NEW "$lhs\t\t$rhs\n";
	}

	# write the new entries
	foreach my $i (keys %$AliasHashRef) {
		print NEW $i,"\t\t",$AliasHashRef->{$i},"\n";
	}

	# close files
	close(NEW);
	close(OLD);

	# make sure our new file has the right mode.
	chmod(0644,$tempfile);

	# overwrite the old file with our new one
	rename($tempfile,$virtusertable_path);

	# recreate the .db file to reflect our changes
	&makemap;

	# unlock
	Virtusertable::unlock();

	return 0;
}

#
# &delete_by_username($User)
#
# $User - a username to be deleted
#
# Deletes all entries from the virtusertable containing the
# specified username (with any domain suffix) in the right-hand-side.
#

sub delete_by_username {
	my ($User, $Domain) = @_;

	# lock
	&Virtusertable::lock();

	# determine a temporary filename to use
	my $tempfile = "$virtusertable_path.$$";

	# open the existing virtusertable and a new one
	if (! open(OLD,$virtusertable_path)) {
		# unlock before returning
		Virtusertable::unlock();
		
		# Log the error.
		LogMessage(1600000, 'error', 'critical', [ $virtusertable_path ], $Domain);
		
		return 1600000;
	}
	if (! open(NEW,"> $virtusertable_path.$$")) {
		# close this file handle that we won't be using now.
		close(OLD);
		
		# unlock before returning
		Virtusertable::unlock();
		
		# Log the error.
		LogMessage(1600001, 'error', 'critical', [ "$virtusertable_path.$$" ], $Domain);
		
		return 1600001;
	}

	# loop through the contents of the existing virtusertable
	while(<OLD>) {
		# chomp the trailing newline
		chomp;

		# seperate the left and the right hand side
		my ($lhs, $rhs);
		if (/^(\S+)\s+(.+)$/) {
			$lhs = $1;
			$rhs = $2;
		} else {
			next;
		}

		# skip any entries with a right-hand-side
		# containing the username that we wish to delete
		if ($rhs =~ /^([^@]+)@?/) { # extract this username
			if ($1 eq $User) { # is this our user?
				next;  # skip
			}
		}

		# otherwise, just copy the line.
		print NEW "$lhs\t\t$rhs\n";
	}

	# close files
	close(NEW);
	close(OLD);

	# make sure our new file has the right mode.
	chmod(0644,$tempfile);

	# overwrite the old file with our new one
	rename($tempfile,$virtusertable_path);

	# recreate the .db file to reflect our changes
	&makemap;

	# unlock
	&Virtusertable::unlock();

	return 0;
}

#
# &normalize_aliases($DomainOld,$DomainNew)
#

sub normalize_aliases {
	my ($DomainOld, $DomainNew) = @_;

	#
	# obtain a list of all valid domains (the main domain plus all aliases)
	# for this domain.  presumably, other code has already changed
	# $DomainOld to $DomainNew in the Mail.pm stuff, so we query based on this.
	#

	my $site = Mail::get_site_from_domain($DomainNew);
	my @valid_domain_list = Mail::get_mail_domain_aliases($site);

	# An array of one element which is an empty string is Mail.pm's way of
	# saying that there are no aliases.
	if ( (scalar(@valid_domain_list) == 1) && ($valid_domain_list[0] eq '') ) {
		@valid_domain_list = ();
	}
	####print STDERR "domains aliases are: ",join(',',@valid_domain_list)," (",scalar(@valid_domain_list)," entries)\n";
	unshift(@valid_domain_list,$DomainNew); # add the canonical domain name
	####print STDERR "Valid domains are: ",join(',',@valid_domain_list),"\n";

	# lock
	&Virtusertable::lock();

	# determine a temporary filename to use
	my $tempfile = "$virtusertable_path.$$";

	# open the existing virtusertable and a new one
	if (! open(OLD,$virtusertable_path)) {
		# unlock before returning
		Virtusertable::unlock();
		
		# Log the error.
		LogMessage(1600000, 'error', 'critical', [ $virtusertable_path ], $DomainNew);
		
		return 1600000;
	}
	if (! open(NEW,"> $virtusertable_path.$$")) {
		# close this file handle that we won't be using now.
		close(OLD);
		
		# unlock before returning
		Virtusertable::unlock();
		
		# Log the error.
		LogMessage(1600001, 'error', 'critical', [ "$virtusertable_path.$$" ], $DomainNew);
		
		return 1600001;
	}

	# The aliasmap holds the contents of all aliases found that belong
	# to the target domain.  The form of this data structure is intended
	# for easy iteration and lookup later, when we scan this structure
	# for missing aliases.
	# Format: $aliasmap->{$rhs_user}->{$lhs_user}->{$lhs_domain} = 1;
	my $aliasmap = {};

	# loop through the contents of the existing virtusertable
	while(<OLD>) {
		# chomp the trailing newline
		chomp;

		# if the entry matches the expected format, extract
		# the two domain names
		my ($lhs_user, $lhs_domain, $rhs_user, $rhs_domain);
		if (/^(\S+?)@(\S+)\s+(\S+?)@(.+)$/) {
			# store the components
			$lhs_user   = $1;
			$lhs_domain = $2;
			$rhs_user   = $3;
			$rhs_domain = $4;

			# perform any needed domain substitutions
			if ($lhs_domain eq $DomainOld) { $lhs_domain = $DomainNew; }
			if ($rhs_domain eq $DomainOld) { $rhs_domain = $DomainNew; }

			# does this alias entry belong to the target domain?
			if ($rhs_domain eq $DomainNew) {
				# does this alias have a valid $lhs_domain ?
				if (grep {$_ eq $lhs_domain} @valid_domain_list) {
					# add this alias to the aliasmap
					$aliasmap->{$rhs_user}->{$lhs_user}->{$lhs_domain} = 1;
				} else {
					# this alias is not valid -- remove it by ending
					# this iteration without printing the line.
					next;
				}
			}

			# write the updated entry
			print NEW "$lhs_user\@$lhs_domain\t\t$rhs_user\@$rhs_domain\n";
		} else {
			# it didn't match our expected format, but it still may be
			# an important line that the user manually entered, so keep it.
			print NEW $_,"\n";
			next;
		}

	}

	# Iterate over the aliasmap in order to add missing entries...

	# iterate over rhs users (the target domain's users who own aliases)
	foreach my $i (keys %$aliasmap) {
		# iterate over lhs users (each email alias belonging to the user)
		foreach my $j (keys %{$aliasmap->{$i}}) {
			# scan the list of valid domains to make sure this email alias
			# has a form alias@domain for each.
			foreach my $k (@valid_domain_list) {
				# add a new entry if the user is missing this
				# alias@domain form...
				if (! exists $aliasmap->{$i}->{$j}->{$k}) {
					printf NEW "%s\@%s\t\t%s\@%s\n",
						$j, $k, $i, $DomainNew;
				}
			}
		}
	}

	# close files
	close(NEW);
	close(OLD);

	# make sure our new file has the right mode.
	chmod(0644,$tempfile);

	# overwrite the old file with our new one
	rename($tempfile,$virtusertable_path);

	# recreate the .db file to reflect our changes
	&makemap;

	# unlock
	&Virtusertable::unlock();

	return 0;
}

#
# &makemap
#
# Run the external "makemap" utility to bring the virtusertable.db
# database up to date with our modifications to the text version of the file.
#

sub makemap {
	Safe::system('/usr/bin/makemap hash /etc/mail/virtusertable.db < /etc/mail/virtusertable');
}

#
# &lock
#
# Open a lock file for append (creating it if needed) and
# try to get a flock() on it.
#

sub lock {
    $lockfile_handle = new IO::File;
    $lockfile_handle->open(">> $lockfile_name");
    flock($lockfile_handle,2);
}

#
# &unlock
#
# Close the file handle to the lock file, thereby
# releasing the lock.  We DO NOT try to unlink the file,
# since that would raise concurrency issues.
#

sub unlock {
    $lockfile_handle->close;
    undef $lockfile_handle;
}

######################################################################
# Interfaces
######################################################################

#
# &update_alias_list($User, $Domain, @aliases);
#
# $User - the user that we are adding aliases for
# $Domain   - the domain the user belongs to
# @aliases  - all the aliases that the user wants
#
# This routine adds all the "$User@$Domain $AliasHashRef[$x]@$Domain" lines
# needed to set the user up with the requested aliases.
#

sub update_alias_list {
	my ($User, $Domain, @aliases) = @_;
	my $AliasHashRef = {};
	my $Return;

	# To provide "combination" support (email_alias@domain_alias,
	# regard SCR 0486) we must obtain a list of all domain aliases
	# for this domain.
	my $site = Mail::get_site_from_domain($Domain);
	my @domain_list = Mail::get_mail_domain_aliases($site);
	unshift(@domain_list,$Domain);

	# add an alias to the aliashash for all combinations of
	# user email aliases and domain name/aliases.
	foreach my $domain_alias (@domain_list) {
		foreach my $i (@aliases) {
			# The domain alias must exist!
			if ($domain_alias) {
				$AliasHashRef->{$i . '@' . $domain_alias} = $User . '@' . $Domain;
			}
		}
	}

	if (0 == ($Return = check_conflicts($AliasHashRef, $Domain)))
	{
		update_by_hash($AliasHashRef, $User, $Domain);
	}

	return $Return;
}

#
# &check_conflicts($AliasHashRef)
#
# $AliasHashRef - a hash of potential virtusertable aliases to be checked
#
# Check each potential alias for conflicts with
#   1. users in the passwd file
#   2. other virtusertable aliases owned by other users in his domain
#
# Return an appropriate feedback-style error if a conflict exists,
# otherwise return undef.
#

sub check_conflicts {
	my ($AliasHashRef, $Domain) = @_;

	# check the passwd database
	foreach my $i (keys %$AliasHashRef) {
		my ($User,$Domain) = split(/@/,$i);
		if (defined(getpwnam($User))) {
			# Alias user name exists.
			return 1600002;
		}
	}

	# check in virtusertable
	if (! open(VUT,$virtusertable_path)) {
		# Log the error.
		LogMessage(1600000, 'error', 'critical', [ $virtusertable_path ], $Domain);
		
		return 1600000;
	}
	while(<VUT>) {
		chomp;

		# seperate the left and the right hand side
		my ($lhs, $rhs);
		if (/^(\S+)\s+(.+)$/) {
			$lhs = $1;
			$rhs = $2;
		} else {
			next;
		}

		if (exists $AliasHashRef->{$lhs} &&
			($AliasHashRef->{$lhs} ne $rhs)) {
			# Alias already exists.
			return 1600003;
		}

	}
	close(VUT);

	return 0;
}

1;

__END__
